@@ -31,6 +31,8 @@
 #include "Query.h"
 #include "SplitLedger.h"
 #include "Transaction.h"
+#include "FileDialog.h"
+#include "global-options.h"
 #include "util.h"
 
 
@@ -127,6 +129,9 @@ ledgerIsMember (xaccLedgerDisplay *reg, Account * acc)
 
    if (acc == reg->leader) return 1;
 
+   /* Simple hack. Always return true for search registers. */
+   if (reg->type == SEARCH_LEDGER) return 1;
+
    if (! (reg->displayed_accounts)) return 0; 
 
    n = 0;
@@ -147,7 +152,7 @@ ledgerIsMember (xaccLedgerDisplay *reg, Account * acc)
 
 xaccLedgerDisplay *
 xaccLedgerDisplaySimple (Account *acc)
-  {
+{
   xaccLedgerDisplay *retval;
   int acc_type;
   int reg_type = -1;
@@ -188,7 +193,7 @@ xaccLedgerDisplaySimple (Account *acc)
       reg_type = CURRENCY_REGISTER;
       break;
     default:
-      PERR ("xaccLedgerDisplaySimple(): unknown account type %d\n", acc_type);
+      PERR ("unknown account type %d\n", acc_type);
       return NULL;
   }
 
@@ -197,7 +202,7 @@ xaccLedgerDisplaySimple (Account *acc)
 
   retval = xaccLedgerDisplayGeneral (acc, NULL, reg_type);
   return retval;
-  }
+}
 
 /********************************************************************\
  * xaccLedgerDisplayAccGroup                                        *
@@ -229,8 +234,8 @@ xaccLedgerDisplayAccGroup (Account *acc)
     case CREDIT:
     case LIABILITY:
        /* if any of the sub-accounts have STOCK or MUTUAL types,
-        * then we must use the PORTFOLIO type ledger.  Otherise,
-        * a plain old GEN_LEDGER will do. */
+        * then we must use the PORTFOLIO_LEDGER ledger. Otherwise,
+        * a plain old GENERAL_LEDGER will do. */
        ledger_type = GENERAL_LEDGER;
 
        le = list[0];
@@ -238,7 +243,7 @@ xaccLedgerDisplayAccGroup (Account *acc)
        while (le) {
           le_type = xaccAccountGetType (le);
           if ((STOCK == le_type) || (MUTUAL == le_type)) {
-             ledger_type = PORTFOLIO;
+             ledger_type = PORTFOLIO_LEDGER;
           }
           n++;
           le = list[n];
@@ -247,9 +252,9 @@ xaccLedgerDisplayAccGroup (Account *acc)
 
     case STOCK:
     case MUTUAL:
-       ledger_type = PORTFOLIO;
+       ledger_type = PORTFOLIO_LEDGER;
        break;
-    
+
     case INCOME:
     case EXPENSE:
        ledger_type = INCOME_LEDGER;
@@ -260,7 +265,7 @@ xaccLedgerDisplayAccGroup (Account *acc)
        break;
 
     default:
-      PERR ("xaccLedgerDisplayAccGroup(): unknown account type \n");
+      PERR ("unknown account type \n");
       _free (list);
       return NULL;
   }
@@ -303,7 +308,7 @@ xaccLedgerDisplaySetHelp(void *user_data, const char *help_str)
 }
 
 /********************************************************************\
- * xaccLedgerDisplayLedger                                          *
+ * xaccLedgerDisplayGeneral                                         *
  *   opens up a ledger window for a list of accounts                *
  *                                                                  *
  * Args:   lead_acc - the account associated with this register     *
@@ -314,9 +319,11 @@ xaccLedgerDisplaySetHelp(void *user_data, const char *help_str)
 \********************************************************************/
 
 xaccLedgerDisplay *
-xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
+xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist,
+                          int ledger_type)
 {
   xaccLedgerDisplay *regData = NULL;
+  gboolean show_all;
 
   /******************************************************************\
   \******************************************************************/
@@ -333,7 +340,7 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
    *
    * A third possibility exists: a multiple-account register, with
    * no leader account.  In such a case, the list of accounts being
-   * displayed have no particular relationshp to each other.  There
+   * displayed have no particular relationship to each other.  There
    * can be an arbitrary number of multiple-account leader-less
    * registers.
    */
@@ -345,7 +352,7 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
        FETCH_FROM_LIST (xaccLedgerDisplay, ledgerList, lead_acc, leader, regData);
      }
   }
-  
+
   /* if regData is null, then no leader account was specified */
   if (!regData) {
     regData = (xaccLedgerDisplay *) malloc (sizeof (xaccLedgerDisplay));
@@ -368,16 +375,32 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
   regData->displayed_accounts = accListCopy (acclist);
   regData->type = ledger_type;
 
+  show_all = gnc_lookup_boolean_option("Register",
+                                       "Show All Transactions",
+                                       TRUE);
+
   /* set up the query filter */
   regData->query = xaccMallocQuery();
-  xaccQuerySetAccounts (regData->query, regData->displayed_accounts);
+
+  /* This is a bit of a hack. The number of splits should be
+   * configurable, or maybe we should go back a time range instead
+   * of picking a number, or maybe we should be able to exclude
+   * based on reconciled status. Anyway, this works for now. */
+  if (!show_all && ((ledger_type & REG_TYPE_MASK) != SEARCH_LEDGER))
+    xaccQuerySetMaxSplits(regData->query, 30);
+
+  xaccQuerySetGroup(regData->query, gncGetCurrentGroup());
+  if(regData->displayed_accounts) {
+    xaccQueryAddAccountMatch(regData->query, 
+                             regData->displayed_accounts,
+                             ACCT_MATCH_ANY, QUERY_OR);
+  }
   if ((regData->leader != NULL) &&
-      !accListHasAccount(regData->displayed_accounts, regData->leader))
-    xaccQueryAddAccount (regData->query, regData->leader);
-
-  /* by default, display only thirty transactions */
-  xaccQuerySetMaxSplits (regData->query, MAX_QUERY_SPLITS);
-
+      !accListHasAccount(regData->displayed_accounts, regData->leader)) {
+    xaccQueryAddSingleAccountMatch(regData->query, regData->leader,
+                                   QUERY_OR);
+  }
+  
   /* add this register to the list of registers */
   fullList = ledgerListAdd (fullList, regData);
 
@@ -395,7 +418,7 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
 
   regData->dirty = 1;
   xaccLedgerDisplayRefresh (regData);
-  
+
   return regData;
 }
 
@@ -426,7 +449,8 @@ xaccLedgerDisplayRefresh (xaccLedgerDisplay *regData)
    * If the GUI wants to display yet other stuff, it's on its own. */
   regData->balance = xaccAccountGetBalance (regData->leader);
   regData->clearedBalance = xaccAccountGetClearedBalance (regData->leader);
-  regData->reconciledBalance = xaccAccountGetReconciledBalance (regData->leader);
+  regData->reconciledBalance = 
+    xaccAccountGetReconciledBalance(regData->leader);
 
   /* OK, now tell this specific GUI window to redraw itself ... */
   if (regData->redraw)
@@ -557,6 +581,26 @@ xaccAccListDisplayRefresh (Account **acc_list)
 /********************************************************************\
 \********************************************************************/
 
+void
+xaccAccGListDisplayRefresh (GList *accounts)
+{
+  GList *node;
+
+  node = accounts;
+  while (node) {
+    MarkDirtyAllRegs (node->data);
+    node = node->next;
+  }
+  node = accounts;
+  while (node) {
+    RefreshAllRegs (node->data);
+    node = node->next;
+  }
+}
+
+/********************************************************************\
+\********************************************************************/
+
 void 
 xaccTransDisplayRefresh (Transaction *trans)
 {
@@ -636,10 +680,6 @@ xaccLedgerDisplayClose (xaccLedgerDisplay *regData)
   if (!regData) return;
   acc = regData->leader;
 
-  /* Save any unsaved changes */
-  if (xaccSRSaveRegEntry (regData->ledger, NULL))
-    xaccSRRedrawRegEntry (regData->ledger);
-
   xaccDestroySplitRegister (regData->ledger);
 
   /* whether this is a single or multi-account window, remove it */
