@@ -13,8 +13,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 /*
@@ -41,9 +44,10 @@
  * to update xaccAccountTypeEnumAsString --- used for text file exports
  */
 
-enum 
+typedef enum 
 {
   BAD_TYPE = -1,
+  NO_TYPE = -1,
   /* Not a type */
 
   BANK = 0,
@@ -98,7 +102,7 @@ enum
   SAVINGS = 12,
   MONEYMRKT = 13,
   CREDITLINE = 14,     /* line of credit */
-};
+} GNCAccountType;
 
 char * xaccAccountGetTypeStr (int type); /* GUI names */
 
