@@ -15,8 +15,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
  *   Author: Rob Clark                                              *
  * Internet: rclark@cs.hmc.edu                                      *
@@ -38,9 +41,6 @@
 
 /** PROTOTYPES ******************************************************/
 
-AccountGroup *xaccReadQIFAccountGroup (char *datafile);
-int           xaccGetQIFIOError (void);
-
 /*
  * The xaccReadAccountGroupFD() and xaccWriteAccountGroupFD()
  * routines read and write the GnuCash "xacc" byte stream (file) 
