@@ -1,7 +1,7 @@
 /********************************************************************\
  * Group.c -- the main data structure of the program                *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997-2000 Linas Vepstas                            *
+ * Copyright (C) 1997-2000 Linas Vepstas <linas@linas.org>          *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -14,13 +14,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
 \********************************************************************/
 
 #include <assert.h>
@@ -30,11 +29,15 @@
 
 #include "Account.h"
 #include "AccountP.h"
+#include "BackendP.h"
+#include "GNCIdP.h"
 #include "Group.h"
 #include "GroupP.h"
 #include "TransactionP.h"
-#include "util.h"
 #include "gnc-common.h"
+#include "util.h"
+
+/* static short module = MOD_ENGINE; */
 
 /********************************************************************\
  * Because I can't use C++ for this project, doesn't mean that I    *
@@ -45,26 +48,34 @@
 
 /********************************************************************\
 \********************************************************************/
+
 void
 xaccInitializeAccountGroup (AccountGroup *grp)
 {
   grp->saved       = GNC_T;
-  
+
   grp->parent      = NULL;
   grp->numAcc      = 0;
   grp->account     = _malloc (sizeof (Account *));
   grp->account[0]  = NULL;   /* null-terminated array */
-  
+
   grp->balance     = 0.0;
+
+  xaccGUIDNew(&grp->guid);
+  xaccStoreEntity(grp, &grp->guid, GNC_ID_GROUP);
+
+  grp->backend     = NULL;
+
 }
 
 /********************************************************************\
 \********************************************************************/
+
 AccountGroup *
 xaccMallocAccountGroup( void )
 {
   AccountGroup *grp = (AccountGroup *)_malloc(sizeof(AccountGroup));
-  
+
   xaccInitializeAccountGroup (grp);
 
   return grp;
@@ -72,16 +83,19 @@ xaccMallocAccountGroup( void )
 
 /********************************************************************\
 \********************************************************************/
+
 void
 xaccFreeAccountGroup( AccountGroup *grp )
 {
   int i;
+
   if (NULL == grp) return;
 
-  for( i=0; i<grp->numAcc; i++ ) {
+  xaccRemoveEntity(&grp->guid);
+
+  for( i=0; i<grp->numAcc; i++ )
     xaccFreeAccount( grp->account[i] );
-  }
-    
+
   _free( grp->account );
 
   /* null everything out, just in case somebody 
@@ -90,14 +104,15 @@ xaccFreeAccountGroup( AccountGroup *grp )
   grp->numAcc      = 0;
   grp->account     = NULL;
   grp->balance     = 0.0;
-  
+
   _free(grp);
 }
 
 /********************************************************************\
 \********************************************************************/
+
 void
-xaccAccountGroupMarkSaved (AccountGroup *grp)
+xaccGroupMarkSaved (AccountGroup *grp)
 {
    int i;
 
@@ -105,14 +120,15 @@ xaccAccountGroupMarkSaved (AccountGroup *grp)
    grp->saved = GNC_T;
 
    for (i=0; i<grp->numAcc; i++) {
-      xaccAccountGroupMarkSaved (grp->account[i]->children); 
+      xaccGroupMarkSaved (grp->account[i]->children); 
    }
 }
 
 /********************************************************************\
 \********************************************************************/
+
 void
-xaccAccountGroupMarkNotSaved (AccountGroup *grp)
+xaccGroupMarkNotSaved (AccountGroup *grp)
 {
    if (!grp) return;
    grp->saved = GNC_F;
@@ -120,8 +136,9 @@ xaccAccountGroupMarkNotSaved (AccountGroup *grp)
 
 /********************************************************************\
 \********************************************************************/
+
 int
-xaccAccountGroupNotSaved (AccountGroup *grp)
+xaccGroupNotSaved (AccountGroup *grp)
 {
    int not_saved;
    int i;
@@ -130,12 +147,50 @@ xaccAccountGroupNotSaved (AccountGroup *grp)
    if (GNC_F == grp->saved) return 1;
 
    for (i=0; i<grp->numAcc; i++) {
-      not_saved = xaccAccountGroupNotSaved (grp->account[i]->children); 
+      not_saved = xaccGroupNotSaved (grp->account[i]->children); 
       if (not_saved) return 1;
    }
    return 0;
 }
 
+/********************************************************************\
+\********************************************************************/
+
+const GUID *
+xaccGroupGetGUID (AccountGroup *group)
+{
+  if (!group)
+    return xaccGUIDNULL();
+
+  return &group->guid;
+}
+
+/********************************************************************\
+\********************************************************************/
+
+void 
+xaccGroupSetGUID (AccountGroup *group, GUID *guid)
+{
+  if (!group || !guid) return;
+
+  xaccRemoveEntity(&group->guid);
+
+  group->guid = *guid;
+
+  xaccStoreEntity(group, &group->guid, GNC_ID_GROUP);
+}
+
+/********************************************************************\
+\********************************************************************/
+
+AccountGroup *
+xaccGroupLookup (const GUID *guid)
+{
+  if (!guid) return NULL;
+
+  return xaccLookupEntity(guid, GNC_ID_GROUP);
+}
+
 /********************************************************************\
  * Get the number of accounts, including subaccounts                *
 \********************************************************************/
@@ -522,7 +577,7 @@ xaccGroupInsertAccount( AccountGroup *grp, Account *acc )
   int i,nacc;
   Account **arr;
   int ralo = 1;
-  
+
   if (NULL == grp) return;
   if (NULL == acc) return;
 
@@ -537,7 +592,7 @@ xaccGroupInsertAccount( AccountGroup *grp, Account *acc )
   }
   grp->saved = GNC_F;
 
-  /* set back-pointer to the accounts parent */
+  /* set back-pointer to the account's parent */
   acc->parent = grp;
 
   nacc = grp->numAcc;
@@ -758,7 +813,7 @@ xaccConcatGroups (AccountGroup *togrp, AccountGroup *fromgrp)
 
    if (!togrp) return;
    if (!fromgrp) return;
-   
+
    /* The act of inserting the account into togrp also causes
     * it to automatically be deleted from fromgrp. But use a
     * saved copy of fromgrp's numAcc member since, after the
@@ -838,26 +893,6 @@ xaccMergeAccounts (AccountGroup *grp)
 /********************************************************************\
 \********************************************************************/
 
-void 
-xaccConsolidateGrpTransactions (AccountGroup *grp)
-{
-   Account * acc;
-   int i;
-
-   if (!grp) return;
-   
-   for (i=0; i<grp->numAcc; i++) {
-      acc = grp->account[i];
-      xaccConsolidateTransactions (acc);
-
-      /* recursively do the children */
-      xaccConsolidateGrpTransactions (acc->children);
-   }
-}
-
-/********************************************************************\
-\********************************************************************/
-
 int     
 xaccGroupGetNumAccounts (AccountGroup *grp)
 {
@@ -897,7 +932,7 @@ xaccGroupGetDepth (AccountGroup *grp)
    int i, depth=0, maxdepth=0;
    if (!grp) return 0;
 
-   for (i=0; i<grp->numAcc; i++) {
+   for (i=0; i < grp->numAcc; i++) {
       depth = xaccGroupGetDepth (grp->account[i]->children);
       if (depth > maxdepth) maxdepth = depth;
    }
@@ -909,26 +944,83 @@ xaccGroupGetDepth (AccountGroup *grp)
 /********************************************************************\
 \********************************************************************/
 
+void
+xaccSplitsBeginStagedTransactionTraversals (Split **splits)
+{
+  Transaction *trans;
+  Split **sptr;
+
+  if (splits == NULL) return;
+
+  for (sptr = splits; *sptr != NULL; sptr++) {
+    trans = (*sptr)->parent;
+    if (trans != NULL)
+      trans->marker = 0;
+  }
+}
+
+void
+xaccAccountBeginStagedTransactionTraversals (Account *account)
+{
+  if (account == NULL) return;
+
+  xaccSplitsBeginStagedTransactionTraversals(account->splits);
+}
+
+void
+xaccAccountsBeginStagedTransactionTraversals (Account **accounts)
+{
+  Account **aptr;
+
+  if (accounts == NULL) return;
+
+  for (aptr = accounts; *aptr != NULL; aptr++)
+    xaccAccountBeginStagedTransactionTraversals(*aptr);
+}
+
+gncBoolean
+xaccTransactionTraverse(Transaction *trans, int stage)
+{
+  if (trans == NULL) return GNC_F;
+
+  if (trans->marker < stage)
+  {
+    trans->marker = stage;
+    return GNC_T;
+  }
+
+  return GNC_F;
+}
+
+gncBoolean
+xaccSplitTransactionTraverse(Split *split, int stage)
+{
+  if (split == NULL) return GNC_F;
+
+  return xaccTransactionTraverse(split->parent, stage);
+}
+
 void
 xaccGroupBeginStagedTransactionTraversals (AccountGroup *grp) 
 {
   unsigned int numAcc;
   unsigned int i;
-  
+
   if (!grp) return;
-  
+
   numAcc = grp->numAcc;
   for(i = 0; i < numAcc; i++) {
     unsigned int n = 0;
     Account *acc;
     Split *s = NULL;
+
     acc = xaccGroupGetAccount(grp, i);
-    
-    if(!acc) return;
-    
+
+    if (!acc) return;
+
     /* recursively do sub-accounts */
     xaccGroupBeginStagedTransactionTraversals(acc->children);
-    
+
     s = acc->splits[0];
     while (s) {
       Transaction *trans = s->parent;
@@ -941,20 +1033,22 @@ xaccGroupBeginStagedTransactionTraversals (AccountGroup *grp)
 
 int
 xaccAccountStagedTransactionTraversal (Account *acc,
-                        unsigned int stage,
-                        int (*callback)(Transaction *t, void *cb_data),
-                        void *cb_data) 
+                                       unsigned int stage,
+                                       int (*callback)(Transaction *t,
+                                                       void *cb_data),
+                                       void *cb_data)
 {
   unsigned int n = 0;
   Split *s = NULL;
-  
-  if(!acc) return 0;
-  
+
+  if (!acc) return 0;
+
   s = acc->splits[0];
   if (callback) {
     int retval;
     while (s) {
       Transaction *trans = s->parent;
+
       if (trans && (trans->marker < stage)) {
         trans->marker = stage;
         retval = callback(trans, cb_data);
@@ -966,6 +1060,7 @@ xaccAccountStagedTransactionTraversal (Account *acc,
   } else {
     while (s) {
       Transaction *trans = s->parent;
+
       if (trans && (trans->marker < stage)) {
         trans->marker = stage;
       }
@@ -973,35 +1068,40 @@ xaccAccountStagedTransactionTraversal (Account *acc,
       s = acc->splits[n];
     }
   }
+
   return 0;
 }
 
-
 int
 xaccGroupStagedTransactionTraversal(AccountGroup *grp,
                                     unsigned int stage,
-                                    int (*callback)(Transaction *t, void *cb_data),
-                                    void *cb_data) 
+                                    int (*callback)(Transaction *t,
+                                                    void *cb_data),
+                                    void *cb_data)
 {
   unsigned int numAcc;
   unsigned int i;
-  
+
   if (!grp) return 0;
-  
+
   numAcc = grp->numAcc;
   for(i = 0; i < numAcc; i++) {
     int retval;
     Account *acc;
+
     acc = xaccGroupGetAccount(grp, i);
-    
+
     /* recursively do sub-accounts */
-    retval = xaccGroupStagedTransactionTraversal 
-               (acc->children, stage, callback, cb_data);
+    retval = xaccGroupStagedTransactionTraversal (acc->children, stage,
+                                                  callback, cb_data);
     if (retval) return retval;
-    retval = xaccAccountStagedTransactionTraversal (acc, stage, callback, cb_data);
+
+    retval = xaccAccountStagedTransactionTraversal (acc, stage,
+                                                    callback, cb_data);
     if (retval) return retval;
   }
+
   return 0;
 }
 
-/****************** END OF FILE *************************************/
+/************************* END OF FILE ********************************/
