@@ -1,17 +1,7 @@
-/*
- * FILE:
- * GroupP.h
- *
- * FUNCTION:
- * This is the *private* account group structure.
- * This header should *not* be included by any code outside of the
- * engine.
- *
- */
 /********************************************************************\
  * GroupP.h -- the main data structure of the program               *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998 Linas Vepstas                           *
+ * Copyright (C) 1997, 1998, 1999, 2000 Linas Vepstas               *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -24,21 +14,34 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
 \********************************************************************/
 
+/*
+ * FILE:
+ * GroupP.h
+ *
+ * FUNCTION:
+ * This is the *private* account group structure.
+ * This header should *not* be included by any code outside of the
+ * engine.
+ *
+ */
+
 #ifndef __XACC_ACCOUNT_GROUP_P_H__
 #define __XACC_ACCOUNT_GROUP_P_H__
 
 #include "config.h"
+#include "BackendP.h"
+#include "GNCId.h"
 #include "Transaction.h"
 
+
 /** STRUCTS *********************************************************/
 struct _account_group {
   /* The flags: */
@@ -50,9 +53,18 @@ struct _account_group {
   int      numAcc;                 /* number of accounts in array */
   Account  **account;              /* array of account pointers   */
 
+  GUID     guid;                   /* globally unique id */
+
+  Backend *backend;                /* persistant storage backend */
+
   /* cached parameters */
   double balance;
-
 };
 
+
+/* Set the group's GUID. This should only be done when reading
+ * a group from a datafile, or some other external source. Never
+ * call this on an existing group! */
+void xaccGroupSetGUID (AccountGroup *group, GUID *guid);
+
 #endif /* __XACC_ACCOUNT_GROUP_P_H__ */
