@@ -13,8 +13,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
@@ -94,7 +97,7 @@ xaccGroupToList (Account *acc)
 
    if (!acc) return NULL;
 
-   ENTER ("xaccGroupToList(): acc=%p \n", acc);
+   ENTER ("acc=%p \n", acc);
    nacc = xaccGetNumAccounts (acc->children);
    nacc ++;  /* add one for this account */
 
@@ -129,7 +132,7 @@ xaccGroupToList (Account *acc)
       }
    }
    list[n] = NULL;
-   LEAVE ("xaccGroupToList(): n=%d nacc=%d \n", n, nacc);
+   LEAVE ("n=%d nacc=%d \n", n, nacc);
    assert (n==nacc);
 
    return list;
