@@ -14,23 +14,25 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
 \********************************************************************/
 
 #ifndef __XACC_TRANSACTION_H__
 #define __XACC_TRANSACTION_H__
 
 #include "config.h"
-#include "gnc-common.h"
 
 #include <time.h>
 
+#include "gnc-common.h"
+#include "GNCId.h"
+
+
 /* Values for the reconciled field in Transaction: */
 #define CREC 'c'              /* The transaction has been cleared        */
 #define YREC 'y'              /* The transaction has been reconciled     */
@@ -146,6 +148,17 @@ void          xaccTransRollbackEdit (Transaction *);
 
 gncBoolean    xaccTransIsOpen (Transaction *trans);
 
+/*
+ * The xaccTransGetGUID() subroutine will return the
+ *    globally unique id associated with that transaction.
+ *
+ * The xaccTransLookup() subroutine will return the
+ *    transaction associated with the given id, or NULL
+ *    if there is no such transaction.
+ */
+const GUID  * xaccTransGetGUID (Transaction *trans);
+Transaction * xaccTransLookup (const GUID *guid);
+
 /* Convert a day, month, and year to a Timespec */
 Timespec      gnc_dmy2timespec(int day, int month, int year);
 
@@ -230,9 +243,9 @@ Split *       xaccTransGetSplit (Transaction *trans, int i);
 /* These routines return the Num (or ID field), the description, 
  * and the date field.
  */
-char *        xaccTransGetNum (Transaction *);
-char *        xaccTransGetDescription (Transaction *);
-char *        xaccTransGetDocref (Transaction *);
+const char *  xaccTransGetNum (Transaction *);
+const char *  xaccTransGetDescription (Transaction *);
+const char *  xaccTransGetDocref (Transaction *);
 time_t        xaccTransGetDate (Transaction *);
 #ifndef SWIG  /* swig chokes on long long */
 long long     xaccTransGetDateL (Transaction *);
@@ -249,8 +262,8 @@ int           xaccTransCountSplits (Transaction *trans);
  * xaccTransFindCommonCurrency. This method is useful for determining
  * whether two accounts can have transactions in common.
  */
-gncBoolean xaccIsCommonCurrency(char *currency_1, char *security_1,
-				char *currency_2, char *security_2);
+gncBoolean xaccIsCommonCurrency(const char *currency_1, const char *security_1,
+				const char *currency_2, const char *security_2);
 
 /* The xaccTransFindCommonCurrency () method returns a string value 
  *    indicating a currency denomination that all of the splits in this
@@ -263,7 +276,7 @@ gncBoolean xaccIsCommonCurrency(char *currency_1, char *security_1,
  *    If all of the splits share both a common security and a common currency,
  *    then the string name for the currency is returned.
  */
-char * xaccTransFindCommonCurrency (Transaction *trans);
+const char * xaccTransFindCommonCurrency (Transaction *trans);
 
 /* The xaccTransIsCommonCurrency () method compares the input string
  *    to the currency/security denominations of all splits in the
@@ -276,12 +289,27 @@ char * xaccTransFindCommonCurrency (Transaction *trans);
  *    currency and a common security.  If the desired match is the
  *    security, a simple string match won't reveal this fact.
  *
- *    This routine is useful in dealing
- *    with currency trading accounts and/or with "stock boxes", where
- *    transaction have in common.  This routine is useful in dealing
- *    securities of differing types are moved across accounts.
+ *    This routine is useful in dealing with currency trading accounts
+ *    and/or with "stock boxes", where transaction have a security in
+ *    common. This routine is useful in dealing with securities of
+ *    differing types as they are moved across accounts.
  */
-char * xaccTransIsCommonCurrency (Transaction *trans, char * currency);
+const char * xaccTransIsCommonCurrency (Transaction *trans,
+                                        const char * currency);
+
+/* The xaccTransIsCommonExclSCurrency () method compares the input
+ *    string to the currency/security denominations of all splits in
+ *    the transaction except the one given as parameter, and returns
+ *    the input string if it is common with all splits except the one
+ *    given, otherwise, it returns NULL.
+ *   
+ * This is useful when changing one split such that the old entry in
+ *    that split is of no relevance when determining whether the new entry
+ *    has a common currency with the other splits.
+ */
+const char * xaccTransIsCommonExclSCurrency (Transaction *trans,
+                                        const char * currency, 
+					     Split *excl_split);
 
 /* The xaccTransGetImbalance() method returns the total value of the
  *    transaction.  In a pure double-entry system, this imbalance
@@ -298,6 +326,17 @@ double xaccTransGetImbalance (Transaction * trans);
 Split       * xaccMallocSplit (void);
 void          xaccInitSplit   (Split *);    /* clears a split struct */
 
+/*
+ * The xaccSplitGetGUID() subroutine will return the
+ *    globally unique id associated with that split.
+ *
+ * The xaccSplitLookup() subroutine will return the
+ *    split associated with the given id, or NULL
+ *    if there is no such split.
+ */
+const GUID * xaccSplitGetGUID (Split *split);
+Split      * xaccSplitLookup (const GUID *guid);
+
 /* The memo is an arbitrary string associated with a split.
  *    Users typically type in free form text from the GUI.
  */
@@ -314,7 +353,7 @@ void          xaccSplitSetAction (Split *, const char *);
 void          xaccSplitSetDocref (Split *, const char *);
 
 /* The Reconcile is a single byte, whose values are typically
- * are "no", "cleared" and "reconciled"
+ * are "N", "C" and "R"
  */
 void          xaccSplitSetReconcile (Split *, char);
 void          xaccSplitSetDateReconciledSecs (Split *, time_t);
@@ -349,7 +388,8 @@ void         xaccSplitSetSharePriceAndAmount (Split *, double price,
 void         xaccSplitSetShareAmount (Split *, double);
 void         xaccSplitSetSharePrice (Split *, double);
 void         xaccSplitSetValue (Split *, double);
-void         xaccSplitSetBaseValue (Split *s, double value, char * base_currency);
+void         xaccSplitSetBaseValue (Split *s, double value,
+                                    const char * base_currency);
 
 
 /* The following four subroutines return the running balance up
@@ -374,17 +414,19 @@ double xaccSplitGetBalance (Split *);
 double xaccSplitGetClearedBalance (Split *);
 double xaccSplitGetReconciledBalance (Split *);
 double xaccSplitGetShareBalance (Split *);
+double xaccSplitGetShareClearedBalance (Split *);
+double xaccSplitGetShareReconciledBalance (Split *);
 double xaccSplitGetCostBasis (Split *);
-double xaccSplitGetBaseValue (Split *s, char *base_currency);
+double xaccSplitGetBaseValue (Split *s, const char *base_currency);
 
 
 /* return the parent transaction of the split */
 Transaction * xaccSplitGetParent (Split *);
 
 /* return the memo, action strings */
-char *        xaccSplitGetMemo (Split *split);
-char *        xaccSplitGetAction (Split *split);
-char *        xaccSplitGetDocref (Split *split);
+const char *  xaccSplitGetMemo (Split *split);
+const char *  xaccSplitGetAction (Split *split);
+const char *  xaccSplitGetDocref (Split *split);
 
 /* return the value of the reconcile flag */
 char          xaccSplitGetReconcile (Split *split);
