@@ -15,16 +15,13 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
- *                                                                  * 
- * TODO: - for now, every year is leap year                         * 
+ *   Author: Rob Clark rclark@cs.hmc.edu                            *
  *                                                                  * 
 \********************************************************************/
 
@@ -103,8 +100,7 @@ printDate (char * buff, int day, int month, int year)
    * right.
    */
   switch(dateFormat)
-    {
-      
+  {
     case DATE_FORMAT_UK:
       sprintf (buff, "%2d/%2d/%-4d", day, month, year);
       break;
@@ -130,8 +126,7 @@ printDate (char * buff, int day, int month, int year)
     default:
       sprintf (buff, "%2d/%2d/%-4d", month, day, year);
       break;
-    }
-  return;
+  }
 }
 
 void 
