@@ -2,7 +2,7 @@
  * util.c -- utility functions that are used everywhere else for    *
  *           xacc (X-Accountant)                                    *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998, 1999, 2000 Linas Vepstas               *
+ * Copyright (C) 1997-2000 Linas Vepstas <linas@linas.org>          *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -15,19 +15,21 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
+ *   Author: Rob Clark (rclark@cs.hmc.edu)                          *
+ *   Author: Linas Vepstas (linas@linas.org)                        *
 \********************************************************************/
 
-#include <malloc.h>
+#include <stdlib.h>
 #include <math.h>
 #include <string.h>
 #include <locale.h>
+#include <limits.h>
 #include <ctype.h>
 
 /* #include <glib.h> */
@@ -41,25 +43,85 @@
 char * stpcpy (char *dest, const char *src);
 
 /** GLOBALS *********************************************************/
-/* 
-   0 == disable all messages
-   1 == enble only error messages
-   2 == print warnings
-   3 == print info messages
-   4 == print debugging messages
- */
-int loglevel[MODULE_MAX] =
-{0,      /* DUMMY */
- 2,      /* ENGINE */
- 2,      /* IO */
- 2,      /* REGISTER */
- 2,      /* LEDGER */
- 2,      /* HTML */
- 2,      /* GUI */
- 2,      /* SCRUB */
- 2,      /* GTK_REG */
+static int min_decimal_places = 0;
+
+gncLogLevel loglevel[MOD_NUM] =
+{
+  GNC_LOG_NOTHING,      /* DUMMY */
+  GNC_LOG_WARNING,      /* ENGINE */
+  GNC_LOG_WARNING,      /* IO */
+  GNC_LOG_WARNING,      /* REGISTER */
+  GNC_LOG_WARNING,      /* LEDGER */
+  GNC_LOG_WARNING,      /* HTML */
+  GNC_LOG_WARNING,      /* GUI */
+  GNC_LOG_WARNING,      /* SCRUB */
+  GNC_LOG_WARNING,      /* GTK_REG */
+  GNC_LOG_WARNING,      /* GUILE */
+  GNC_LOG_DEBUG,        /* BACKEND */
+  GNC_LOG_WARNING,      /* QUERY */
 };
 
+/* Set the logging level of the given module. */
+void
+gnc_set_log_level(gncModuleType module, gncLogLevel level)
+{
+  if ((module < 0) || (module > MOD_LAST))
+    return;
+
+  loglevel[module] = level;
+}
+
+/* Set the logging level for all modules. */
+void
+gnc_set_log_level_global(gncLogLevel level)
+{
+  gncModuleType module;
+
+  for (module = GNC_LOG_NOTHING; module < MOD_NUM; module++)
+    loglevel[module] = level;
+}
+
+void
+gnc_set_mininum_decimal_places (int places)
+{
+  if (places < 0)
+    return;
+
+  if (places > 8)
+    return;
+
+  min_decimal_places = places;
+}
+
+/********************************************************************\
+\********************************************************************/
+/* prettify() cleans up subroutine names.
+ * AIX/xlC has the habit of printing signatures not names; clean this up.
+ * On other operating systems, truncate name to 30 chars.
+ * Note this routine is not thread safe. Note we wouldn't need this
+ * routine if AIX did something more reasonable.  Hope thread safety
+ * doesn't poke us in eye.
+ */
+char *
+prettify (const char *name)
+{
+   static char bf[35];
+   char *p;
+   strncpy (bf, name, 29); bf[28] = 0;
+   p = strchr (bf, '(');
+   if (p)
+   {
+      *(p+1) = ')';
+      *(p+2) = 0x0;
+   }
+   else
+   {
+      strcpy (&bf[26], "...()");
+   }
+   return bf;
+}
+
+
 /********************************************************************\
  * DEBUGGING MEMORY ALLOCATION STUFF                                * 
 \********************************************************************/
@@ -100,6 +162,102 @@ dcoresize(void)
 }
 #endif
 
+/********************************************************************\
+\********************************************************************/
+
+#define UPPER(c) (((c) >= 'a' && (c) <= 'z') ? (c) + 'A' - 'a' : (c))
+
+/* Search for str2 in first nchar chars of str1, ignore case.. 
+ * Return pointer to first match, or null.
+ */
+
+char *
+strncasestr(const char *str1, const char *str2, size_t len) 
+{
+    while (*str1 && len--) 
+   {
+      if (UPPER(*str1) == UPPER(*str2)) 
+      {
+         if (strncasecmp(str1,str2,strlen(str2)) == 0) 
+         {
+            return (char *) str1;
+         }
+      }
+      str1++;
+   }
+   return NULL;
+}
+
+/* Search for str2 in str1, ignore case. 
+ * Return pointer to first match, or null. 
+ */
+
+char *
+strcasestr(const char *str1, const char *str2) 
+{
+   size_t len = strlen (str1);
+   char * retval = strncasestr (str1, str2, len);
+   return retval;
+}
+
+/* Reversed strstr -- search for a needle in the haystack,
+ * from the far end 
+ */
+
+char *
+rstrstr (const char *haystack, const char * needle)
+{
+    int haylen = strlen (haystack);
+    int neelen = strlen (needle);
+
+    const char * hp = haystack + haylen - 1;
+    const char * np = needle + neelen - 1;
+
+    if ((0 == neelen) || (0 == haylen)) return 0x0;
+
+    while (hp >= haystack+neelen) {
+        if (*hp == *np) {
+            --np;
+            if (np < needle) return (char *) hp;
+        } else {
+            np = needle + neelen - 1;
+        }
+        --hp;
+    }
+
+    return 0x0;
+}
+
+
+/* The strpskip() function locates the first occurrence in the
+ * string s that does not match any of the characters in "reject".
+ * This is the opposite of strpbrk()
+ */
+
+char * 
+strpskip (const char * s, const char *reject)
+{
+   size_t i, rlen;
+   char * retval;
+
+   if (!s) return NULL;
+   if (!reject) return (char *) s;
+
+   rlen = sizeof (reject);
+   retval = (char *) s;
+
+   while (*retval) {
+      int match = 0;
+      for (i=0; i<rlen; i++) {
+         if (reject[i] == *retval) {match=1; break; }
+      }
+      if (!match) return retval;
+      retval ++;
+   }
+   return NULL;
+}
+
+
 /********************************************************************\
 \********************************************************************/
 
@@ -148,7 +306,7 @@ ultostr (unsigned long val, int base)
     if (10>broke[i]) {
        buf[places-1-i] = 0x30+broke[i];  /* ascii digit zero */
     } else {
-       buf[places-1-i] = 0x41-10+broke[i];  /* ascii capaital A */
+       buf[places-1-i] = 0x41-10+broke[i];  /* ascii capital A */
     }
   }
   buf[places] = 0x0;
@@ -253,7 +411,7 @@ gnc_lconv_set(char **p_value, char *default_value)
 static void
 gnc_lconv_set_char(char *p_value, char default_value)
 {
-  if ((p_value != NULL) && (*p_value == 127))
+  if ((p_value != NULL) && (*p_value == CHAR_MAX))
     *p_value = default_value;
 }
 
@@ -334,6 +492,9 @@ PrintAmt(char *buf, double val, int prec,
   if (val < 0.0)
     val = DABS(val);
 
+  if (prec < min_decimal_places)
+    prec = min_decimal_places;
+
   util_fptostr(tempBuf, val, prec);
 
   /* Here we strip off trailing decimal zeros per the argument. */
@@ -358,7 +519,19 @@ PrintAmt(char *buf, double val, int prec,
 
   if (!use_separators)
   {
-    /* If we're not using separators, then the whole string is copied */
+    /* fix up the decimal place, if there is one */
+    stringLength = strlen(tempBuf);
+    numWholeDigits = -1;
+    for (i = stringLength - 1; i >= 0; i--) {
+      if (tempBuf[i] == '.') {
+        if (monetary)
+          tempBuf[i] = lc->mon_decimal_point[0];
+        else
+          tempBuf[i] = lc->decimal_point[0];
+        break;
+      }
+    }
+
     strcpy(buf, tempBuf);
   }
   else
@@ -367,7 +540,7 @@ PrintAmt(char *buf, double val, int prec,
     stringLength = strlen(tempBuf);
     numWholeDigits = -1;
     for (i = stringLength - 1; i >= 0; i--) {
-      if (tempBuf[i] == '.') {
+      if ((tempBuf[i] == '.') || (tempBuf[i] == lc->decimal_point[0])) {
         numWholeDigits = i;
         if (monetary)
           tempBuf[i] = lc->mon_decimal_point[0];
@@ -405,14 +578,17 @@ PrintAmt(char *buf, double val, int prec,
 }
 
 int
-xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
-                         int min_trailing_zeros)
+xaccSPrintAmountGeneral (char * bufp, double val,
+                         GNCPrintAmountFlags flags,
+                         int precision,
+                         int min_trailing_zeros,
+                         const char *curr_sym)
 {
    struct lconv *lc;
 
    char *orig_bufp = bufp;
-   char *currency_symbol;
-   char *sign;
+   const char *currency_symbol;
+   const char *sign;
 
    char cs_precedes;
    char sep_by_space;
@@ -427,7 +603,7 @@ xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
    if (DEQ(val, 0.0))
      val = 0.0;
 
-   if (shrs & PRTSHR)
+   if (flags & PRTSHR)
    {
      currency_symbol = "shrs";
      cs_precedes = 0;  /* currency symbol follows amount */
@@ -435,7 +611,19 @@ xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
    }
    else
    {
-     currency_symbol = lc->currency_symbol;
+     if (flags & PRTEUR)
+     {
+       currency_symbol = "EUR ";
+     }
+     else if (curr_sym == NULL)
+     {
+       currency_symbol = lc->currency_symbol;
+     }
+     else
+     {
+       currency_symbol = curr_sym;
+     }
+
      if (val < 0.0)
      {
        cs_precedes  = lc->n_cs_precedes;
@@ -473,7 +661,7 @@ xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
      if (print_sign && (sign_posn == 3))
        bufp = stpcpy(bufp, sign);
 
-     if (shrs & PRTSYM)
+     if (flags & PRTSYM)
      {
        bufp = stpcpy(bufp, currency_symbol);
        if (sep_by_space)
@@ -490,8 +678,8 @@ xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
      bufp = stpcpy(bufp, "(");
 
    /* Now print the value */
-   bufp += PrintAmt(bufp, DABS(val), precision, shrs & PRTSEP,
-                    !(shrs & PRTNMN), min_trailing_zeros);
+   bufp += PrintAmt(bufp, DABS(val), precision, flags & PRTSEP,
+                    !(flags & PRTNMN), min_trailing_zeros);
 
    /* Now see if we print parentheses */
    if (print_sign && (sign_posn == 0))
@@ -504,7 +692,7 @@ xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
      if (print_sign && (sign_posn == 3))
        bufp = stpcpy(bufp, sign);
 
-     if (shrs & PRTSYM)
+     if (flags & PRTSYM)
      {
        if (sep_by_space)
          bufp = stpcpy(bufp, " ");
@@ -525,35 +713,61 @@ xaccSPrintAmountGeneral (char * bufp, double val, short shrs, int precision,
 }
 
 int
-xaccSPrintAmount (char * bufp, double val, short shrs) 
+xaccSPrintAmount (char * bufp, double val, GNCPrintAmountFlags flags,
+                  const char *curr_code) 
 {
+   struct lconv *lc;
    int precision;
    int min_trailing_zeros;
+   char curr_sym[5];
 
-   if (shrs & PRTSHR)
+   lc = gnc_localeconv();
+
+   if (curr_code && (strncmp(curr_code, lc->int_curr_symbol, 3) == 0))
+     curr_code = NULL;
+   else if (curr_code)
+   {
+     strncpy(curr_sym, curr_code, 3);
+     curr_sym[3] = '\0';
+     strcat(curr_sym, " ");
+     curr_code = curr_sym;
+   }
+
+   if (curr_code && (strncmp(curr_code, "EUR", 3) == 0))
+     flags |= PRTEUR;
+
+   if (flags & PRTCUR)
+   {
+     precision = 5;
+     min_trailing_zeros = 0;
+   }
+   else if (flags & PRTSHR)
    {
      precision = 4;
      min_trailing_zeros = 0;
    }
+   else if (flags & PRTEUR)
+   {
+     precision = 2;
+     min_trailing_zeros = 2;
+   }
    else
    {
-     struct lconv *lc = gnc_localeconv();
-
      precision = lc->frac_digits;
      min_trailing_zeros = lc->frac_digits;
    }
 
-   return xaccSPrintAmountGeneral(bufp, val, shrs, precision,
-                                  min_trailing_zeros);
+   return xaccSPrintAmountGeneral(bufp, val, flags, precision,
+                                  min_trailing_zeros, curr_code);
 }
 
 char *
-xaccPrintAmount (double val, short shrs) 
+xaccPrintAmount (double val, GNCPrintAmountFlags flags, const char *curr_code) 
 {
    /* hack alert -- this is not thread safe ... */
    static char buf[BUFSIZE];
 
-   xaccSPrintAmount (buf, val, shrs);
+   xaccSPrintAmount (buf, val, flags, curr_code);
 
    /* its OK to return buf, since we declared it static */
    return buf;
@@ -561,15 +775,16 @@ xaccPrintAmount (double val, short shrs)
 
 char *
 xaccPrintAmountArgs (double val, gncBoolean print_currency_symbol,
-                     gncBoolean print_separators, gncBoolean is_shares_value)
+                     gncBoolean print_separators, gncBoolean is_shares_value,
+                     const char *curr_code)
 {
-  short shrs = 0;
+  GNCPrintAmountFlags flags = 0;
 
-  if (print_currency_symbol) shrs |= PRTSYM;
-  if (print_separators)      shrs |= PRTSEP;
-  if (is_shares_value)       shrs |= PRTSHR;
+  if (print_currency_symbol) flags |= PRTSYM;
+  if (print_separators)      flags |= PRTSEP;
+  if (is_shares_value)       flags |= PRTSHR;
 
-  return xaccPrintAmount(val, shrs);
+  return xaccPrintAmount(val, flags, curr_code);
 }
 
 
@@ -585,15 +800,17 @@ xaccPrintAmountArgs (double val, gncBoolean print_currency_symbol,
 double xaccParseAmount (const char * instr, gncBoolean monetary)
 {
    struct lconv *lc = gnc_localeconv();
+   gncBoolean isneg = GNC_F;
    char *mstr, *str, *tok;
    double amount = 0.0;
    char negative_sign;
    char thousands_sep;
    char decimal_point;
    int len;
-   int isneg = 0;
 
    if (!instr) return 0.0;
+   if (*instr == '\0') return 0.0;
+
    mstr = strdup (instr);
    str = mstr;
 
@@ -609,23 +826,78 @@ double xaccParseAmount (const char * instr, gncBoolean monetary)
      decimal_point = lc->decimal_point[0];
    }
 
-   /* strip off garbage at end of the line */
-   tok = strchr (str, '\r');
-   if (tok) *tok = 0x0;
-   tok = strchr (str, '\n');
-   if (tok) *tok = 0x0;
+   /* strip off garbage at the beginning of the line */
+   while (*str != '\0')
+   {
+     switch (*str)
+     {
+       case '\r':
+       case '\n':
+       case ' ':
+       case '\t':
+         str++;
+         continue;
+         break;
+     }
 
-   /* search for a negative sign */
-   tok = strchr (str, negative_sign);
-   if (tok) {
-      isneg = 1;
-      str = tok + sizeof(char);
+     break;
    }
 
+   /* look for a negative sign */
+   if (*str == negative_sign) {
+      isneg = GNC_T;
+      str++;
+   }
+   /* also accept (num) as negative */
+   else if (*str == '(') {
+     /* go to end of string */
+     for (tok = str; *tok != '\0'; tok++)
+       ;
+     tok--;
+
+     if (*tok != ')')
+       return 0.0;
+
+     isneg = GNC_T;
+     str++;
+     *tok = '\0';
+   }
+
+   if (*str == '\0') return 0.0;
+
+   /* go to end of string */
+   for (tok = str; *tok != '\0'; tok++)
+     ;
+
+   /* strip off garbage at end of the line */
+   while (--tok != str)
+   {
+     switch (*tok)
+     {
+       case '\r':
+       case '\n':
+       case ' ':
+       case '\t':
+         continue;
+         break;
+     }
+
+     break;
+   }
+
+   /* look for a negative sign at the end, some locales allow it,
+    * we'll just allow it everywhere. */
+   if (*tok == negative_sign) {
+      isneg = GNC_T;
+      *tok = '\0';
+   }
+
+   if (*str == '\0') return 0.0;
+
    /* remove thousands separator */
    tok = strchr (str, thousands_sep);
    while (tok) {
-      *tok = 0x0;
+      *tok = '\0';
       amount *= 1000.0;
       amount += ((double) (1000 * atoi (str)));
       str = tok + sizeof(char);
@@ -635,7 +907,7 @@ double xaccParseAmount (const char * instr, gncBoolean monetary)
    /* search for a decimal point */
    tok = strchr (str, decimal_point);
    if (tok) {
-      *tok = 0x0;
+      *tok = '\0';
       amount += ((double) (atoi (str)));
       str = tok + sizeof(char);
 
@@ -645,7 +917,7 @@ double xaccParseAmount (const char * instr, gncBoolean monetary)
 
          /* strip off garbage at end of the line */
          tok = strchr (str, ' ');
-         if (tok) *tok = 0x0;
+         if (tok) *tok = '\0';
    
          /* adjust for number of decimal places */
          len = strlen(str);
@@ -680,122 +952,5 @@ double xaccParseAmount (const char * instr, gncBoolean monetary)
 }
 
 
-/********************************************************************\
- * xaccParseQIFAmount                                               * 
- *   parses monetary strings in QIF files                           *
- *                                                                  * 
- * Args:   str -- pointer to string rep of sum                      * 
- * Return: double -- the parsed amount                              * 
- *
- * Note: be careful changing this algorithm.  The Quicken-file-format
- * parser depends a lot on the ability of this routine to do what it's
- * doing.  Don't break it!
-\********************************************************************/
-
-/* The following tokens are used to define the US-style monetary
- * strings.  With a bit of cleverness, it should be possible to modify
- * these to handle various international styles ... maybe ... */
-
-#define MINUS_SIGN '-'
-#define K_SEP      ','      /* thousands separator */
-#define DEC_SEP    '.'      /* decimal point */
-
-double xaccParseQIFAmount (const char * instr) 
-{
-   char decimal_point = DEC_SEP;
-   char thousands_sep = K_SEP;
-   char *mstr, *str, *tok;
-   double dollars = 0.0;
-   int isneg = 0;
-   int len;
-
-   if (!instr) return 0.0;
-   mstr = strdup (instr);
-   str = mstr;
-
-   /* strip off garbage at end of the line */
-   tok = strchr (str, '\r');
-   if (tok) *tok = 0x0;
-   tok = strchr (str, '\n');
-   if (tok) *tok = 0x0;
-
-   /* search for a minus sign */
-   tok = strchr (str, MINUS_SIGN);
-   if (tok) {
-      isneg = 1;
-      str = tok+sizeof(char);
-   }
-
-   /* figure out separators */
-   {
-     char *tok1, *tok2;
-
-     tok1 = strrchr(str, DEC_SEP);
-     tok2 = strrchr(str, K_SEP);
-
-     if (tok1 < tok2)
-     {
-       decimal_point = K_SEP;
-       thousands_sep = DEC_SEP;
-     }
-   }
-
-   /* remove comma's */
-   tok = strchr (str, thousands_sep);
-   while (tok) {
-      *tok = 0x0;
-      dollars *= 1000.0;
-      dollars += ((double) (1000 * atoi (str)));
-      str = tok+sizeof(char);
-      tok = strchr (str, thousands_sep);
-   }
-
-   /* search for a decimal point */
-   tok = strchr (str, decimal_point);
-   if (tok) {
-      *tok = 0x0;
-      dollars += ((double) (atoi (str)));
-      str = tok+sizeof(char);
-
-      /* if there is anything trailing the decimal 
-       * point, convert it  */
-      if (str[0]) {
-
-         /* strip off garbage at end of the line */
-         tok = strchr (str, ' ');
-         if (tok) *tok = 0x0;
-   
-         /* adjust for number of decimal places */
-         len = strlen(str);
-         if (6 == len) {
-            dollars += 0.000001 * ((double) atoi (str));
-         } else
-         if (5 == len) {
-            dollars += 0.00001 * ((double) atoi (str));
-         } else
-         if (4 == len) {
-            dollars += 0.0001 * ((double) atoi (str));
-         } else
-         if (3 == len) {
-            dollars += 0.001 * ((double) atoi (str));
-         } else
-         if (2 == len) {
-            dollars += 0.01 * ((double) atoi (str));
-         } else 
-         if (1 == len) {
-            dollars += 0.1 * ((double) atoi (str));
-         } 
-      }
-
-   } else {
-      dollars += ((double) (atoi (str)));
-   }
-
-   if (isneg) dollars = -dollars;
-
-   free (mstr);
-   return dollars;
-}
-
 /************************* END OF FILE ******************************\
 \********************************************************************/
