@@ -17,8 +17,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "top-level.h"
@@ -122,8 +125,8 @@ gnc_account_tree_init(GNCAccountTree *tree)
   gnc_account_tree_set_view_info_real(tree);
 
   gtk_ctree_construct(GTK_CTREE(tree),
-		      tree->num_columns, 0,
-		      tree->column_headings);
+                      tree->num_columns, 0,
+                      (gchar **) tree->column_headings);
 
   gtk_clist_set_shadow_type(GTK_CLIST(tree), GTK_SHADOW_IN);
   gtk_clist_column_titles_passive(GTK_CLIST(tree));
@@ -131,6 +134,15 @@ gnc_account_tree_init(GNCAccountTree *tree)
   gtk_clist_set_column_justification(GTK_CLIST(tree),
 				     tree->balance_column,
 				     GTK_JUSTIFY_RIGHT);
+  gtk_clist_set_column_justification(GTK_CLIST(tree),
+				     tree->balance_column + 1,
+				     GTK_JUSTIFY_RIGHT);
+  gtk_clist_set_column_justification(GTK_CLIST(tree),
+				     tree->total_column,
+				     GTK_JUSTIFY_RIGHT);
+  gtk_clist_set_column_justification(GTK_CLIST(tree),
+				     tree->total_column + 1,
+				     GTK_JUSTIFY_RIGHT);
 
   {
     GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(tree));
@@ -697,7 +709,7 @@ gnc_init_account_view_info(AccountViewInfo *avi)
 
   avi->show_field[ACCOUNT_NAME] = GNC_T;
   avi->show_field[ACCOUNT_DESCRIPTION] = GNC_T;
-  avi->show_field[ACCOUNT_BALANCE] = GNC_T;
+  avi->show_field[ACCOUNT_TOTAL] = GNC_T;
 }
 
 /********************************************************************\
@@ -735,6 +747,11 @@ gnc_account_tree_set_view_info_real(GNCAccountTree *tree)
 
   tree->balance_column = i;
   tree->column_fields[i++] = ACCOUNT_BALANCE;
+  tree->column_fields[i++] = ACCOUNT_BALANCE_EURO;
+
+  tree->total_column = i;
+  tree->column_fields[i++] = ACCOUNT_TOTAL;
+  tree->column_fields[i++] = ACCOUNT_TOTAL_EURO;
 
   tree->column_fields[i++] = ACCOUNT_NOTES;
 
@@ -815,15 +832,19 @@ gnc_account_tree_select_row(GtkCTree *ctree,
   GList *node;
 
   tree->ignore_unselect = GNC_F;
-
   account = gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(row));
-  node = g_list_find(tree->current_accounts, account);
-  if (node == NULL)
-    tree->current_accounts = g_list_prepend(tree->current_accounts, account);
 
-  gtk_signal_emit(GTK_OBJECT(ctree),
-		  account_tree_signals[SELECT_ACCOUNT],
-		  account);
+  if (account != NULL)
+  {
+    node = g_list_find(tree->current_accounts, account);
+
+    if (node == NULL)
+      tree->current_accounts = g_list_prepend(tree->current_accounts, account);
+
+    gtk_signal_emit(GTK_OBJECT(ctree),
+                    account_tree_signals[SELECT_ACCOUNT],
+                    account);
+  }
 
   GTK_CTREE_CLASS(parent_class)->tree_select_row(ctree, row, column);
 }
@@ -910,16 +931,19 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
 			    GtkCTreeNode *sibling,
 			    Account *acc)
 {
-  int i;
   gchar *text[NUM_ACCOUNT_FIELDS + 1];
   GtkCTreeNode *node;
+  gint i;
 
   if (acc == NULL)
     return NULL;
 
   for (i = 0; i < tree->num_columns; i++)
-    text[i] = gnc_ui_get_account_field_value_string(acc,
-						    tree->column_fields[i]);
+  {
+    text[i] =
+      g_strdup(gnc_ui_get_account_field_value_string(acc,
+                                                     tree->column_fields[i]));
+  }
 
   text[tree->num_columns] = NULL;
 
@@ -927,13 +951,16 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
 			       text, 0, NULL, NULL, NULL, NULL,
 			       FALSE, FALSE);
 
+  for (i = 0; i < tree->num_columns; i++)
+    g_free(text[i]);
+
 #if !USE_NO_COLOR
   {
     GtkStyle *style;
     double balance;
     gboolean deficit;
 
-    balance = gnc_ui_get_account_full_balance(acc);
+    balance = gnc_ui_account_get_balance(acc, FALSE);
     deficit = (balance < 0) && !DEQ(balance, 0);
 
     if (deficit)
@@ -944,6 +971,22 @@ gnc_account_tree_insert_row(GNCAccountTree *tree,
     if (style != NULL)
       gtk_ctree_node_set_cell_style(GTK_CTREE(tree), node,
 				    tree->balance_column, style);
+
+    balance = gnc_ui_account_get_balance(acc, TRUE);
+    deficit = (balance < 0) && !DEQ(balance, 0);
+
+    if (deficit)
+      style = tree->deficit_style;
+    else
+      style = gtk_widget_get_style(GTK_WIDGET(tree));
+
+    if (style != NULL)
+    {
+      gtk_ctree_node_set_cell_style(GTK_CTREE(tree), node,
+				    tree->total_column, style);
+      gtk_ctree_node_set_cell_style(GTK_CTREE(tree), node,
+				    tree->total_column + 1, style);
+    }
   }
 #endif
 
