@@ -1,7 +1,7 @@
 /********************************************************************\
  * dialog-add.c -- window for creating new accounts for GnuCash     *
- * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998, 1999 Linas Vepstas                     *
+ * Copyright (C) 1997 Robin D. Clark <rclark@cs.hmc.edu>            *
+ * Copyright (C) 1997, 1998, 1999 Linas Vepstas <linas@linas.org>   *
  * Copyright (C) 1999 Jeremy Collins                                *
  * Copyright (C) 2000 Dave Peticolas <petcola@cs.ucdavis.edu>       *
  *                                                                  *
@@ -16,13 +16,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "top-level.h"
@@ -76,7 +74,7 @@ struct _accwindow
  *         data      - gtk user data pointer, will point to the     *
  *                     accWindow structure                          *
  * Return: none                                                     *
- \********************************************************************/
+\********************************************************************/
 static void 
 gnc_ui_accWindow_list_select_cb(GtkCList * type_list, gint row, gint column,
 				GdkEventButton * event, gpointer data)
@@ -109,7 +107,7 @@ gnc_ui_accWindow_list_select_cb(GtkCList * type_list, gint row, gint column,
 
 
 /********************************************************************\
- * Function: gnc_ui_accWindow_list_unselect_cb -  a callback for the*
+ * Function: gnc_ui_accWindow_list_unselect_cb - a callback for the *
  *           list of account types window                           *
  *                                                                  *
  * Args:   type_list - the list widget                              *
@@ -118,7 +116,7 @@ gnc_ui_accWindow_list_select_cb(GtkCList * type_list, gint row, gint column,
  *         data      - gtk user data pointer, will point to the     *
  *                     accWindow structure                          *
  * Return: none                                                     *
- \********************************************************************/
+\********************************************************************/
 static void 
 gnc_ui_accWindow_list_unselect_cb(GtkCList * type_list, gint row, gint column,
 				  GdkEventButton * event, gpointer data)
@@ -345,29 +343,33 @@ static void
 gnc_ui_accWindow_create_account(Account * account, Account * parent,
 				gint type, AccountFieldStrings * strings)
 {
+  xaccAccountBeginEdit(parent, 0);
   xaccAccountBeginEdit(account, 0);
 
+  xaccAccountSetType (account, type);
+
+  gnc_ui_install_field_strings(account, strings, TRUE);
+
+#if 0
+  { /* make a default transaction */
+    Transaction *trans = xaccMallocTransaction();
+
+    xaccTransBeginEdit(trans, 1);
+    xaccTransSetDateToday (trans);
+    xaccTransSetDescription (trans, OPEN_BALN_STR);
+    xaccTransCommitEdit(trans);
+
+    xaccAccountInsertSplit (account, xaccTransGetSplit (trans, 0) );
+  }
+#endif
+
   if (parent != NULL)
     xaccInsertSubAccount (parent, account);
   else
     xaccGroupInsertAccount(gncGetCurrentGroup(), account);
 
-  xaccAccountSetType (account, type);
-
-  gnc_ui_install_field_strings(account, strings, TRUE);
-
-  { /* make a default transaction */
-    Transaction  *trans = xaccMallocTransaction();
- 
-    xaccTransBeginEdit(trans, 1);
-    xaccTransSetDateToday (trans);
-    xaccTransSetDescription (trans, OPEN_BALN_STR);
-    xaccTransCommitEdit(trans);
-            
-    xaccAccountInsertSplit (account, xaccTransGetSplit (trans, 0) );
-  }
-
   xaccAccountCommitEdit (account);
+  xaccAccountCommitEdit (parent);
 
   gnc_account_tree_insert_account(gnc_get_current_account_tree(), account);
 
@@ -530,6 +532,11 @@ accWindow (AccountGroup *this_is_not_used)
     break;
   }
 
+  g_free(accData);
+
+  if (result < 0)
+    return NULL;
+
   DEBUG("destroying account add window\n");
 
   gdk_window_get_geometry(dialog->window, NULL, NULL,
@@ -538,7 +545,6 @@ accWindow (AccountGroup *this_is_not_used)
   gnc_save_window_size("account_add_win", last_width, last_height);
 
   gtk_widget_destroy(dialog);
-  g_free(accData);
 
   return NULL;
 }
