@@ -13,8 +13,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include <top-level.h>
@@ -24,10 +27,15 @@
 #include "dialog-options.h"
 #include "dialog-utils.h"
 #include "option-util.h"
+#include "guile-util.h"
 #include "query-user.h"
 #include "gnc-helpers.h"
 #include "account-tree.h"
+#include "gnc-dateedit.h"
+#include "gnc-currency-edit.h"
 #include "global-options.h"
+#include "query-user.h"
+#include "window-help.h"
 #include "messages.h"
 #include "util.h"
 
@@ -84,6 +92,18 @@ gnc_option_set_ui_value(GNCOption *option, gboolean use_default)
     else
       bad_value = TRUE;
   }
+  else if (safe_strcmp(type, "currency") == 0)
+  {
+    if (gh_string_p(value))
+    {
+      char *string = gh_scm2newstr(value, NULL);
+      gnc_currency_edit_set_currency(GNC_CURRENCY_EDIT(option->widget),
+                                     string);
+      free(string);
+    }
+    else
+      bad_value = TRUE;
+  }
   else if (safe_strcmp(type, "multichoice") == 0)
   {
     int index;
@@ -105,7 +125,7 @@ gnc_option_set_ui_value(GNCOption *option, gboolean use_default)
     if (gnc_timepair_p(value))
     {
       ts = gnc_timepair2timespec(value);
-      gnome_date_edit_set_time(GNOME_DATE_EDIT(option->widget), ts.tv_sec);
+      gnc_date_edit_set_time(GNC_DATE_EDIT(option->widget), ts.tv_sec);
     }
     else
       bad_value = TRUE;
@@ -141,7 +161,7 @@ gnc_option_set_ui_value(GNCOption *option, gboolean use_default)
       value = gh_cdr(value);
 
       row = gnc_option_permissible_value_index(option, item);
-      if (index < 0)
+      if (row < 0)
       {
         bad_value = TRUE;
         break;
@@ -189,12 +209,12 @@ gnc_option_set_ui_value(GNCOption *option, gboolean use_default)
   }
   else
   {
-    PERR("gnc_option_set_ui_value: Unknown type. Ignoring.\n");
+    PERR("Unknown type. Ignoring.\n");
   }
 
   if (bad_value)
   {
-    PERR("gnc_option_set_ui_value: bad value\n");
+    PERR("bad value\n");
   }
 
   free(type);
@@ -234,6 +254,16 @@ gnc_option_get_ui_value(GNCOption *option)
     result = gh_str02scm(string);
     g_free(string);
   }
+  else if (safe_strcmp(type, "currency") == 0)
+  {
+    GtkEditable *editable;
+    char * string;
+
+    editable = GTK_EDITABLE(GTK_COMBO(option->widget)->entry);
+    string = gtk_editable_get_chars(editable, 0, -1);
+    result = gh_str02scm(string);
+    g_free(string);
+  }
   else if (safe_strcmp(type, "multichoice") == 0)
   {
     gpointer _index;
@@ -249,7 +279,7 @@ gnc_option_get_ui_value(GNCOption *option)
   {
     Timespec ts;
 
-    ts.tv_sec  = gnome_date_edit_get_date(GNOME_DATE_EDIT(option->widget));
+    ts.tv_sec  = gnc_date_edit_get_date(GNC_DATE_EDIT(option->widget));
     ts.tv_nsec = 0;
 
     result = gnc_timespec2timepair(ts);
@@ -317,7 +347,7 @@ gnc_option_get_ui_value(GNCOption *option)
   }
   else
   {
-    PERR("gnc_option_get_ui_value: Unknown type for refresh. Ignoring.\n");
+    PERR("Unknown type for refresh. Ignoring.\n");
   }
 
   free(type);
@@ -663,7 +693,7 @@ gnc_option_create_list_widget(GNCOption *option, char *name)
     }
     else
     {
-      PERR("gnc_option_create_list_widget: bad value name\n");
+      PERR("bad value name\n");
     }
   }
 
@@ -790,6 +820,35 @@ gnc_option_set_ui_widget(GNCOption *option,
 		     gnc_option_create_default_button(option, tooltips),
 		     FALSE, FALSE, 0);
   }
+  else if (safe_strcmp(type, "currency") == 0)
+  {
+    GtkWidget *label;
+    gchar *colon_name;
+
+    colon_name = g_strconcat(name, ":", NULL);
+    label = gtk_label_new(colon_name);
+    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+    g_free(colon_name);
+
+    enclosing = gtk_hbox_new(FALSE, 5);
+    value = gnc_currency_edit_new();
+
+    option->widget = value;
+    gnc_option_set_ui_value(option, FALSE);
+
+    if (documentation != NULL)
+      gtk_tooltips_set_tip(tooltips, GTK_COMBO(value)->entry,
+                           documentation, NULL);
+
+    gtk_signal_connect(GTK_OBJECT(GTK_COMBO(value)->entry), "changed",
+		       GTK_SIGNAL_FUNC(gnc_option_changed_cb), option);
+
+    gtk_box_pack_start(GTK_BOX(enclosing), label, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(enclosing), value, FALSE, FALSE, 0);
+    gtk_box_pack_end(GTK_BOX(enclosing),
+		     gnc_option_create_default_button(option, tooltips),
+		     FALSE, FALSE, 0);
+  }
   else if (safe_strcmp(type, "multichoice") == 0)
   {
     GtkWidget *label;
@@ -830,19 +889,19 @@ gnc_option_set_ui_widget(GNCOption *option,
     use24 = gnc_lookup_boolean_option("International",
                                       "Use 24-hour time format", FALSE);
 
-    value = gnome_date_edit_new(time(NULL), show_time, use24);
+    value = gnc_date_edit_new(time(NULL), show_time, use24);
 
     option->widget = value;
     gnc_option_set_ui_value(option, FALSE);
 
-    entry = GNOME_DATE_EDIT(value)->date_entry;
+    entry = GNC_DATE_EDIT(value)->date_entry;
     gtk_tooltips_set_tip(tooltips, entry, documentation, NULL);
     gtk_signal_connect(GTK_OBJECT(entry), "changed",
 		       GTK_SIGNAL_FUNC(gnc_option_changed_cb), option);
 
     if (show_time)
     {
-      entry = GNOME_DATE_EDIT(value)->time_entry;
+      entry = GNC_DATE_EDIT(value)->time_entry;
       gtk_tooltips_set_tip(tooltips, entry, documentation, NULL);
       gtk_signal_connect(GTK_OBJECT(entry), "changed",
                          GTK_SIGNAL_FUNC(gnc_option_changed_cb), option);
@@ -1013,7 +1072,7 @@ gnc_option_set_ui_widget(GNCOption *option,
   }
   else
   {
-    PERR("gnc_option_set_ui_widget: Unknown type. Ignoring.\n");
+    PERR("Unknown type. Ignoring.\n");
   }
 
   if (!packed && (enclosing != NULL))
@@ -1040,7 +1099,7 @@ gnc_options_dialog_add_option(GtkWidget *page,
   gnc_option_set_ui_widget(option, GTK_BOX(page), tooltips);
 }
 
-static void
+static gint
 gnc_options_dialog_append_page(GnomePropertyBox *propertybox,
                                GNCOptionSection *section,
                                GtkTooltips *tooltips)
@@ -1050,11 +1109,12 @@ gnc_options_dialog_append_page(GnomePropertyBox *propertybox,
   GtkWidget *page_content_box;
   gint num_options;
   gchar *name;
+  gint page;
   gint i;
 
   name = gnc_option_section_name(section);
   if (strncmp(name, "__", 2) == 0)
-    return;
+    return -1;
 
   page_label = gtk_label_new(_(name));
   gtk_widget_show(page_label);
@@ -1063,7 +1123,8 @@ gnc_options_dialog_append_page(GnomePropertyBox *propertybox,
   gtk_container_set_border_width(GTK_CONTAINER(page_content_box), 5);
   gtk_widget_show(page_content_box);
 
-  gnome_property_box_append_page(propertybox, page_content_box, page_label);
+  page = gnome_property_box_append_page(propertybox,
+                                        page_content_box, page_label);
 
   num_options = gnc_option_section_num_options(section);
   for (i = 0; i < num_options; i++)
@@ -1071,6 +1132,8 @@ gnc_options_dialog_append_page(GnomePropertyBox *propertybox,
     option = gnc_get_option_section_option(section, i);
     gnc_options_dialog_add_option(page_content_box, option, tooltips);
   }
+
+  return page;
 }
 
 
@@ -1089,16 +1152,97 @@ gnc_build_options_dialog_contents(GnomePropertyBox *propertybox,
 {
   GtkTooltips *tooltips;
   GNCOptionSection *section;
+  gchar *default_section_name;
+  gint default_page = -1;
   gint num_sections;
+  gint page;
   gint i;
 
   tooltips = gtk_tooltips_new();
 
   num_sections = gnc_option_db_num_sections(odb);
+  default_section_name = gnc_option_db_get_default_section(odb);
 
   for (i = 0; i < num_sections; i++)
   {
+    char *section_name;
+
     section = gnc_option_db_get_section(odb, i);
-    gnc_options_dialog_append_page(propertybox, section, tooltips);
+    page = gnc_options_dialog_append_page(propertybox, section, tooltips);
+
+    section_name = gnc_option_section_name(section);
+    if (safe_strcmp(section_name, default_section_name) == 0)
+      default_page = page;
   }
+
+  if (default_page >= 0)
+    gtk_notebook_set_page(GTK_NOTEBOOK(propertybox->notebook), default_page);
+
+  if (default_section_name != NULL)
+    free(default_section_name);
+}
+
+
+static void
+gnc_options_dialog_apply_cb(GnomePropertyBox *propertybox,
+			    gint arg1, gpointer user_data)
+{
+  GNCOptionDB *global_options = user_data;
+
+  if (arg1 == -1)
+    gnc_option_db_commit(global_options);
+}
+
+static void
+gnc_options_dialog_help_cb(GnomePropertyBox *propertybox,
+			   gint arg1, gpointer user_data)
+{
+  helpWindow(NULL, HELP_STR, HH_GLOBPREFS);
+}
+
+/* Options dialog... this should house all of the config options     */
+/* like where the docs reside, and whatever else is deemed necessary */
+void
+gnc_show_options_dialog()
+{
+  static GnomePropertyBox *options_dialog = NULL;
+  GNCOptionDB *global_options;
+
+  global_options = gnc_get_global_options();
+
+  if (gnc_option_db_num_sections(global_options) == 0)
+  {
+    gnc_warning_dialog("No options!");
+    return;
+  }
+
+  if (gnc_option_db_dirty(global_options))
+  {
+    if (options_dialog != NULL)
+      gtk_widget_destroy(GTK_WIDGET(options_dialog));
+
+    options_dialog = NULL;
+  }
+
+  if (options_dialog == NULL)
+  {
+    options_dialog = GNOME_PROPERTY_BOX(gnome_property_box_new());
+    gnome_dialog_close_hides(GNOME_DIALOG(options_dialog), TRUE);
+
+    gnc_build_options_dialog_contents(options_dialog, global_options);
+    gnc_option_db_clean(global_options);
+
+    gtk_window_set_title(GTK_WINDOW(options_dialog), GNC_PREFS);
+
+    gtk_signal_connect(GTK_OBJECT(options_dialog), "apply",
+		       GTK_SIGNAL_FUNC(gnc_options_dialog_apply_cb),
+		       global_options);
+
+    gtk_signal_connect(GTK_OBJECT(options_dialog), "help",
+		       GTK_SIGNAL_FUNC(gnc_options_dialog_help_cb),
+		       global_options);
+  }
+
+  gtk_widget_show(GTK_WIDGET(options_dialog));  
+  gdk_window_raise(GTK_WIDGET(options_dialog)->window);
 }
