@@ -14,8 +14,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #define _GNU_SOURCE
@@ -24,6 +27,8 @@
 
 #include <gnome.h>
 #include <stdio.h>
+#include <sys/time.h>
+#include <unistd.h>
 
 #include <guile/gh.h>
 
@@ -43,14 +48,11 @@
 #include "query-user.h"
 #include "util.h"
 
-
 struct _qifimportwindow
 {
   /* on the Files tab */
   GtkWidget * dialog;
   GtkWidget * currency_entry;
-  GtkWidget * radix_picker;
-  GtkWidget * date_picker;
   GtkWidget * filename_entry;
   GtkWidget * acct_auto_button;
   GtkWidget * acct_entry;
@@ -85,9 +87,6 @@ QIFImportWindow *
 gnc_ui_qif_import_dialog_make() 
 {
   QIFImportWindow * retval;
-  GtkWidget * menu;
-  GtkWidget * active;
-  int i;
   
   SCM  load_map_prefs;
   SCM  mapping_info;
@@ -97,18 +96,17 @@ gnc_ui_qif_import_dialog_make()
   int  scm_strlen;
 
   retval = g_new0(QIFImportWindow, 1);
-  
+
   retval->dialog = create_QIF_File_Import_Dialog();
-  retval->imported_files = 
-    SCM_EOL;
-  retval->selected_file = SCM_BOOL_F;
+
+  retval->imported_files    = SCM_UNDEFINED;
+  retval->selected_file     = SCM_UNDEFINED;
+  retval->mapping_info      = SCM_UNDEFINED;
+  retval->cat_display_info  = SCM_UNDEFINED;
+  retval->acct_display_info = SCM_UNDEFINED;
 
   retval->currency_entry = 
     gtk_object_get_data(GTK_OBJECT(retval->dialog), "qif_currency_entry");
-  retval->radix_picker   = 
-    gtk_object_get_data(GTK_OBJECT(retval->dialog), "qif_radix_picker");
-  retval->date_picker    = 
-    gtk_object_get_data(GTK_OBJECT(retval->dialog), "qif_date_picker");
   retval->filename_entry =
     gtk_object_get_data(GTK_OBJECT(retval->dialog), "qif_filename_entry");
   retval->acct_auto_button = 
@@ -122,10 +120,6 @@ gnc_ui_qif_import_dialog_make()
     gtk_object_get_data(GTK_OBJECT(retval->dialog), "account_page_list");
   retval->cat_list = 
     gtk_object_get_data(GTK_OBJECT(retval->dialog), "category_page_list");
-  
-  
-
-
 
   gtk_object_set_data(GTK_OBJECT(retval->dialog),
                       "qif_window_struct", retval);
@@ -138,52 +132,22 @@ gnc_ui_qif_import_dialog_make()
 
   mapping_info = gh_call0(load_map_prefs);
   retval->mapping_info = mapping_info;
-  
+  scm_protect_object(retval->mapping_info);
+
   default_currency = gh_call1(lookup_value,
                               gh_call2(lookup_option,
                                        gh_str02scm("International"),
                                        gh_str02scm("Default Currency")));
-  
+
+  retval->imported_files = SCM_EOL;
   scm_protect_object(retval->imported_files);
-  scm_protect_object(retval->mapping_info);
 
   /* set the currency entry to the GNC default currency */
   gtk_entry_set_text(GTK_ENTRY(retval->currency_entry), 
                      gh_scm2newstr(default_currency, &scm_strlen));
 
-  /* repair the option menus to associate "option_index" with the 
-   * index number for each menu item */
-  menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(retval->radix_picker));
-
-  for(i = 0; i < 3; i++) {
-    gtk_option_menu_set_history(GTK_OPTION_MENU(retval->radix_picker), i);
-    active = gtk_menu_get_active(GTK_MENU(menu));
-    gtk_object_set_data(GTK_OBJECT(active), "option_index",
-                        GINT_TO_POINTER(i));
-  }
-  gtk_option_menu_set_history(GTK_OPTION_MENU(retval->radix_picker), 0);
+  gtk_widget_show_all(retval->dialog);
   
-
-  menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(retval->date_picker));
-
-  for(i = 0; i < 5; i++) {
-    gtk_option_menu_set_history(GTK_OPTION_MENU(retval->date_picker), i);
-    active = gtk_menu_get_active(GTK_MENU(menu));
-    gtk_object_set_data(GTK_OBJECT(active), 
-                        "option_index",
-                        GINT_TO_POINTER(i));
-  }
-  gtk_option_menu_set_history(GTK_OPTION_MENU(retval->date_picker), 0);
-  
-  gtk_widget_show(retval->dialog);
-
-  if (retval->dialog->window == NULL) {
-    free(retval);
-    return NULL;
-  }
-
-  gdk_window_raise(retval->dialog->window);
-
   return retval;
 }
 
@@ -200,11 +164,20 @@ gnc_ui_qif_import_dialog_destroy (QIFImportWindow * window)
     gnome_dialog_close(GNOME_DIALOG(window->dialog));
   }
 
-  scm_unprotect_object(window->imported_files);
-  scm_unprotect_object(window->selected_file);
-  scm_unprotect_object(window->mapping_info);
-  scm_unprotect_object(window->cat_display_info);
-  scm_unprotect_object(window->acct_display_info);
+  if (window->imported_files != SCM_UNDEFINED)
+    scm_unprotect_object(window->imported_files);
+
+  if (window->selected_file != SCM_UNDEFINED)
+    scm_unprotect_object(window->selected_file);
+
+  if (window->mapping_info != SCM_UNDEFINED)
+    scm_unprotect_object(window->mapping_info);
+
+  if (window->cat_display_info != SCM_UNDEFINED)
+    scm_unprotect_object(window->cat_display_info);
+
+  if (window->acct_display_info != SCM_UNDEFINED)
+    scm_unprotect_object(window->acct_display_info);
 
   g_free(window);
 }
@@ -224,12 +197,10 @@ gnc_ui_qif_import_select_file_cb(GtkButton * button,
   QIFImportWindow * wind = 
     gtk_object_get_data(GTK_OBJECT(dialog), "qif_window_struct");
   
-  char * new_file_name;
+  const char * new_file_name;
 
   new_file_name = fileBox(_("Select QIF File"), "*.qif");
 
-
-
   if(new_file_name && (access(new_file_name, R_OK) == 0)) {
 
     /* set the filename entry for what was selected */
@@ -248,17 +219,7 @@ gnc_ui_qif_import_select_file_cb(GtkButton * button,
     if(wind->acct_entry) {
       gtk_entry_set_text(GTK_ENTRY(wind->acct_entry),
                          "");
-    }
-    
-    /* radix and date formats are auto-determined by default */
-    if(wind->date_picker) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->date_picker),
-                                  0);
-    }
-    if(wind->radix_picker) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->radix_picker),
-                                  0);
-    }
+    }    
   }
 }
 
@@ -267,13 +228,13 @@ gnc_ui_qif_import_select_file_cb(GtkButton * button,
  * gnc_ui_qif_import_load_file_cb
  * 
  * Invoked when the "load file" button is clicked on the first page of
- * the QIF Import notebook.  Filename, currency, radix format, and
+ * the QIF Import notebook.  Filename, currency, and
  * date format are read from the UI and passed to the Scheme side.
 \********************************************************************/
 
 void
-gnc_ui_qif_import_load_file_cb         (GtkButton       *button,
-                                        gpointer         user_data) {
+gnc_ui_qif_import_load_file_cb(GtkButton * button, gpointer user_data) {
+
   GtkWidget       * dialog = GTK_WIDGET(user_data);
   QIFImportWindow * wind = 
     gtk_object_get_data(GTK_OBJECT(dialog), "qif_window_struct");
@@ -282,41 +243,31 @@ gnc_ui_qif_import_load_file_cb         (GtkButton       *button,
   char * qif_account;
   char * currency;
   char * error_string = NULL;
-  int  radix_format;
-  int  date_format;
-
-  GtkWidget * menu;
-  GtkWidget * menuitem;
+  
+  struct timeval start, end;
   
   SCM make_qif_file, qif_file_load, qif_file_loaded, unload_qif_file;
-  SCM scm_filename, scm_currency, scm_radix, scm_date, scm_qif_account;
+  SCM qif_file_parse;
+  SCM scm_filename, scm_currency, scm_qif_account;
   SCM scm_qiffile;
   SCM imported_files = SCM_EOL;
-  SCM load_return;
+  SCM load_return, parse_return;
 
-  char * radix_symbols [] = { "unknown", "decimal", "comma" };  
-  char * date_symbols [] = { "unknown", "m-d-y", "d-m-y", 
-                             "y-m-d", "y-d-m" };
-  
   /* get the UI elements */
   path_to_load = gtk_entry_get_text(GTK_ENTRY(wind->filename_entry));
   currency     = gtk_entry_get_text(GTK_ENTRY(wind->currency_entry));
   qif_account  = gtk_entry_get_text(GTK_ENTRY(wind->acct_entry));
 
-  menu         = gtk_option_menu_get_menu(GTK_OPTION_MENU(wind->radix_picker));
-  menuitem     = gtk_menu_get_active(GTK_MENU(menu));
-  radix_format = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(menuitem),
-                                                     "option_index"));
-  
-  menu         = gtk_option_menu_get_menu(GTK_OPTION_MENU(wind->date_picker));
-  menuitem     = gtk_menu_get_active(GTK_MENU(menu));
-  date_format  = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(menuitem),
-                                                     "option_index"));
-  
   if(strlen(path_to_load) == 0) {
     gnc_error_dialog_parented(GTK_WINDOW(wind->dialog), 
                               _("You must specify a file to load."));
   }
+  else if ((strlen(path_to_load) > 0) && access(path_to_load, R_OK) < 0) {
+    /* stay here if bad file */
+    gnc_error_dialog_parented(GTK_WINDOW(wind->dialog), 
+                              _("File not found or read permission denied.\n"
+                                "Please select another file."));
+  }
   else if(strlen(currency) == 0) {
     gnc_error_dialog_parented(GTK_WINDOW(wind->dialog), 
                               _("You must specify a currency."));
@@ -325,6 +276,7 @@ gnc_ui_qif_import_load_file_cb         (GtkButton       *button,
     /* find the make and load functions. */
     make_qif_file   = gh_eval_str("make-qif-file");
     qif_file_load   = gh_eval_str("qif-file:read-file");
+    qif_file_parse  = gh_eval_str("qif-file:parse-fields");
     qif_file_loaded = gh_eval_str("qif-dialog:qif-file-loaded?");
     unload_qif_file = gh_eval_str("qif-dialog:unload-qif-file");
     
@@ -339,8 +291,6 @@ gnc_ui_qif_import_load_file_cb         (GtkButton       *button,
       /* convert args */
       scm_filename = gh_str02scm(path_to_load);
       scm_currency = gh_str02scm(currency);
-      scm_radix = gh_symbol2scm(radix_symbols[radix_format]);
-      scm_date = gh_symbol2scm(date_symbols[date_format]);
 
       if(gtk_toggle_button_get_active
          (GTK_TOGGLE_BUTTON(wind->acct_auto_button))) {
@@ -355,7 +305,7 @@ gnc_ui_qif_import_load_file_cb         (GtkButton       *button,
       if(gh_call2(qif_file_loaded, scm_filename, wind->imported_files)
          == SCM_BOOL_T) {
         if(gnc_verify_dialog_parented
-           (GTK_WINDOW(wind->dialog),
+           (wind->dialog,
             _("QIF File already loaded. Reload with current settings?"), 
             TRUE)) {
           imported_files = 
@@ -369,46 +319,91 @@ gnc_ui_qif_import_load_file_cb         (GtkButton       *button,
       /* turn on the busy cursor */
       gnc_set_busy_cursor(NULL);
 
+      gettimeofday(&start, NULL);
+
       /* create the <qif-file> object */
       scm_qiffile = gh_apply(make_qif_file, 
-                             SCM_LIST4(scm_qif_account, scm_radix, 
-                                       scm_date, scm_currency));
+                             SCM_LIST2(scm_qif_account, scm_currency));
       
       imported_files = 
         gh_cons(scm_qiffile, imported_files);
 
       wind->selected_file = scm_qiffile;
 
-      /* I think I have to do this since it's a global but not in 
-       * guile-space */
       scm_protect_object(wind->selected_file);      
       
       load_return = gh_call2(qif_file_load,  gh_car(imported_files),
                              scm_filename);
       
-      /* import the file into it */
-      if(load_return  != SCM_BOOL_T) {
-        if(gh_list_p(load_return)) {
-          asprintf(&error_string,
-                   QIF_LOAD_FAILED_FORMAT_MSG,
-                   gh_scm2newstr(gh_cadr(load_return), NULL));
-        }
-        else {
-          error_string = QIF_LOAD_FAILED_DEFAULT_MSG;
-        }
+      /* a list returned is (#f error-message) for an error, 
+       * (#t error-message) for a warning */
+      if(gh_list_p(load_return) &&
+         (gh_car(load_return) == SCM_BOOL_T)) {
+        asprintf(&error_string,
+                 QIF_LOAD_WARNING_FORMAT_MSG,
+                 gh_scm2newstr(gh_cadr(load_return), NULL));
+        gnc_warning_dialog_parented(GTK_WIDGET(wind->dialog), error_string);
+      }      
+        
+      if((load_return != SCM_BOOL_T) &&
+         (!gh_list_p(load_return) || 
+          (gh_car(load_return) != SCM_BOOL_T))) {
+        asprintf(&error_string,
+                 QIF_LOAD_FAILED_FORMAT_MSG,
+                 gh_scm2newstr(gh_cadr(load_return), NULL));
         gnc_error_dialog_parented(GTK_WINDOW(wind->dialog), error_string);
         
         imported_files = 
           gh_call2(unload_qif_file, scm_filename, imported_files);
       }
+      else {
+        parse_return = gh_call1(qif_file_parse, gh_car(imported_files));
+        
+        if(gh_list_p(parse_return) && 
+           (gh_car(parse_return) == SCM_BOOL_T)) {
+          asprintf(&error_string,
+                   QIF_PARSE_WARNING_FORMAT_MSG,
+                   gh_scm2newstr(gh_cadr(parse_return), NULL));
+          gnc_warning_dialog_parented(GTK_WIDGET(wind->dialog), error_string);
+        }
+        if((parse_return != SCM_BOOL_T) &&
+           (!gh_list_p(parse_return) ||
+            (gh_car(parse_return) != SCM_BOOL_T))) {
+          asprintf(&error_string,
+                   QIF_PARSE_FAILED_FORMAT_MSG,
+                   gh_scm2newstr(gh_cadr(parse_return), NULL));
+          gnc_error_dialog_parented(GTK_WINDOW(wind->dialog), error_string);
+          imported_files = 
+            gh_call2(unload_qif_file, scm_filename, imported_files);
+        }
+      }
+      
       wind->imported_files = imported_files;
       scm_protect_object(wind->imported_files);
       
+      gettimeofday(&end, NULL);
+#if 0
+      printf("QIF file load took %f ms total.\n", 
+             1000.0*(end.tv_sec - start.tv_sec) +
+             .001*(end.tv_usec - start.tv_usec));
+#endif      
+      gettimeofday(&start, NULL);
+      
       /* now update the Accounts and Categories pages in the notebook */
       update_file_page(wind);
       update_accounts_page(wind); 
       update_categories_page(wind);
 
+      gettimeofday(&end, NULL);
+
+#if 0
+      printf("QIF Category/account tab update took %f ms.\n", 
+             1000.0*(end.tv_sec - start.tv_sec) +
+             .001*(end.tv_usec - start.tv_usec));
+#endif
+
+      gettimeofday(&end, NULL);
+
       /* turn back the cursor */
       gnc_unset_busy_cursor(NULL);
     }
@@ -422,17 +417,15 @@ gnc_ui_qif_import_select_loaded_file_cb(GtkList   * list,
                                         gpointer  user_data) {
   GtkWidget       * dialog = GTK_WIDGET(user_data);
   QIFImportWindow * wind = 
-    gtk_object_get_data(GTK_OBJECT(dialog), "qif_window_struct");
+    gtk_object_get_data(GTK_OBJECT(dialog), "qif_window_struct");  
+  int file_index = 
+    GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget), "file_index"));
   
-  SCM scm_qiffile;
+  wind->selected_file = gh_list_ref(wind->imported_files, 
+                                    gh_int2scm(file_index));
   
-  scm_qiffile = GPOINTER_TO_INT
-    (gtk_object_get_data(GTK_OBJECT(widget), "scm-object"));
-
-  wind->selected_file = scm_qiffile;
   scm_protect_object(wind->selected_file);
-  update_file_info(wind, scm_qiffile);
-
+  update_file_info(wind, wind->selected_file);
 }
 
 
@@ -444,54 +437,24 @@ gnc_ui_qif_import_select_loaded_file_cb(GtkList   * list,
 void
 gnc_ui_qif_import_ok_cb(GtkButton * button, gpointer user_data) {
 
-  SCM  save_map_prefs;
-  SCM  qif_to_gnc;
-  SCM  hash_set;
-  SCM  hash_data;
-  char * qif_acct_name;
-  char * qif_cat_name;
-  int row;
+  SCM  save_map_prefs = gh_eval_str("qif-import:save-map-prefs");
+  SCM  qif_to_gnc = gh_eval_str("qif-import:qif-to-gnc");
 
-  GtkWidget       * dialog = GTK_WIDGET(user_data);
   QIFImportWindow * wind = 
-    gtk_object_get_data(GTK_OBJECT(dialog), "qif_window_struct");
+    gtk_object_get_data(GTK_OBJECT(user_data), "qif_window_struct");
  
-  save_map_prefs = gh_eval_str("qif-import:save-map-prefs");
-  qif_to_gnc     = gh_eval_str("qif-import:qif-to-gnc");
-  hash_set       = gh_eval_str("hash-set!");
-
   /* busy cursor */
   gnc_set_busy_cursor(NULL);
-
-  /* transfer the info from the account / category pickers to 
-   * the mapping info hash tables */
-  for(row=0; row < GTK_CLIST(wind->acct_list)->rows; row++) {
-    gtk_clist_get_text(GTK_CLIST(wind->acct_list), row, 0, &qif_acct_name);
-    
-    hash_data = (SCM)gtk_clist_get_row_data(GTK_CLIST(wind->acct_list), row);
-    gh_call3(hash_set, gh_cadr(wind->mapping_info), 
-             gh_str02scm(qif_acct_name), 
-             hash_data);
-  }
-
-  for(row=0; row < GTK_CLIST(wind->cat_list)->rows; row++) {
-    gtk_clist_get_text(GTK_CLIST(wind->cat_list), row, 0, &qif_cat_name);
-
-    hash_data = (SCM)gtk_clist_get_row_data(GTK_CLIST(wind->cat_list), row);
-    gh_call3(hash_set, gh_caddr(wind->mapping_info), 
-             gh_str02scm(qif_cat_name), 
-             hash_data);
-  }
   
   /* call a scheme function to do the work */
   gh_call2(qif_to_gnc, wind->imported_files, 
            wind->mapping_info);
-
+  
   /* write out mapping info before destroying the window */
   gh_call1(save_map_prefs, wind->mapping_info);
-
+  
   gnc_unset_busy_cursor(NULL);
-
+  
   gnc_ui_qif_import_dialog_destroy(wind);
   wind = NULL;
 }
@@ -518,24 +481,30 @@ void
 gnc_ui_qif_import_account_line_select_cb(GtkCList * clist, gint row,
                                          gint column, GdkEvent * event,
                                          gpointer user_data) {
-  char * initial_string;
-  int initial_type;
+  QIFImportWindow * wind = 
+    gtk_object_get_data(GTK_OBJECT(user_data), "qif_window_struct");
+  char  * gnc_acct_text = NULL;
+  char  * qif_acct_text = NULL;
+ 
+  int   initial_type;
+  SCM   scm_acct;
+  SCM   mapping_info;
+  SCM   scm_hash_ref = gh_eval_str("hash-ref");
 
-  SCM  scm_acct;
-  SCM  old_info;
-  SCM  munge_func = gh_eval_str("qif-dialog:munge-account-mapping");
+  gtk_clist_get_text(GTK_CLIST(clist), row, 0, &qif_acct_text);
+  gtk_clist_get_text(GTK_CLIST(clist), row, 2, &gnc_acct_text);
 
-  old_info = (SCM)gtk_clist_get_row_data(GTK_CLIST(clist), row);
-
-  gtk_clist_get_text(GTK_CLIST(clist), row, 2, &initial_string);
-
-  initial_type = gh_scm2int(gh_list_ref(old_info, gh_int2scm(2)));
-
-  scm_acct = accountPickerBox(initial_string, initial_type);
+  mapping_info = gh_call2(scm_hash_ref, gh_cadr(wind->mapping_info),
+                          gh_str02scm(qif_acct_text));
+  initial_type = gh_scm2int(gh_list_ref(mapping_info, gh_int2scm(2)));
 
+  scm_acct = accountPickerBox(gnc_acct_text, initial_type);
+  
   if(gh_list_p(scm_acct)) {
-    gh_call2(munge_func, old_info, scm_acct);
-
+    scm_list_set_x(mapping_info, gh_int2scm(1), gh_car(scm_acct));
+    scm_list_set_x(mapping_info, gh_int2scm(2), gh_cadr(scm_acct));
+    scm_list_set_x(mapping_info, gh_int2scm(5), gh_caddr(scm_acct));
+    
     gtk_clist_set_text(GTK_CLIST(clist), row, 2, 
                        gh_scm2newstr(gh_car(scm_acct), NULL));
     gtk_clist_set_text(GTK_CLIST(clist), row, 3,
@@ -548,23 +517,30 @@ void
 gnc_ui_qif_import_category_line_select_cb(GtkCList * clist, gint row,
                                           gint column, GdkEvent * event,
                                           gpointer user_data) {
-  char * initial_string;
-  int initial_type;
-
-  SCM  scm_acct;
-  SCM  old_info;
-  SCM  munge_func = gh_eval_str("qif-dialog:munge-account-mapping");
-
-  old_info = (SCM)gtk_clist_get_row_data(GTK_CLIST(clist), row);
-
-  gtk_clist_get_text(GTK_CLIST(clist), row, 2, &initial_string);
-  initial_type = gh_scm2int(gh_list_ref(old_info, gh_int2scm(2)));
-
-  scm_acct = accountPickerBox(initial_string, initial_type);
+  QIFImportWindow * wind = 
+    gtk_object_get_data(GTK_OBJECT(user_data), "qif_window_struct");
+  char  * gnc_acct_text = NULL;
+  char  * qif_acct_text = NULL;
+  
+  int   initial_type;
+  SCM   scm_acct;
+  SCM   mapping_info;
+  SCM   scm_hash_ref = gh_eval_str("hash-ref");
+  
+  gtk_clist_get_text(GTK_CLIST(clist), row, 0, &qif_acct_text);
+  gtk_clist_get_text(GTK_CLIST(clist), row, 2, &gnc_acct_text);
 
+  mapping_info = gh_call2(scm_hash_ref, gh_caddr(wind->mapping_info),
+                          gh_str02scm(qif_acct_text));
+  initial_type = gh_scm2int(gh_list_ref(mapping_info, gh_int2scm(2)));
+  
+  scm_acct = accountPickerBox(gnc_acct_text, initial_type);
+  
   if(gh_list_p(scm_acct)) {
-    gh_call2(munge_func, old_info, scm_acct);
-
+    scm_list_set_x(mapping_info, gh_int2scm(1), gh_car(scm_acct));
+    scm_list_set_x(mapping_info, gh_int2scm(2), gh_cadr(scm_acct));
+    scm_list_set_x(mapping_info, gh_int2scm(5), gh_caddr(scm_acct));
+    
     gtk_clist_set_text(GTK_CLIST(clist), row, 2, 
                        gh_scm2newstr(gh_car(scm_acct), NULL));
     gtk_clist_set_text(GTK_CLIST(clist), row, 3,
@@ -574,7 +550,6 @@ gnc_ui_qif_import_category_line_select_cb(GtkCList * clist, gint row,
 }
 
 
-
 /********************************************************************\
  * update_file_page
  * update the left-side list and the right-side info. 
@@ -589,6 +564,7 @@ update_file_page(QIFImportWindow * wind) {
   SCM       scm_qiffile;
   SCM       qif_file_path;
   int       path_strlen;
+  int       scm_file_index = 0;
 
   /* clear the list */
   gtk_list_remove_items(GTK_LIST(wind->selected_file_list),
@@ -604,11 +580,11 @@ update_file_page(QIFImportWindow * wind) {
     new_list_item = 
       gtk_list_item_new_with_label(gh_scm2newstr(gh_call1(qif_file_path,
                                                           scm_qiffile),
-                                                 &path_strlen));
+                                                 &path_strlen));  
     gtk_object_set_data(GTK_OBJECT(new_list_item),
-                        "scm-object", GINT_TO_POINTER(scm_qiffile));
-    scm_protect_object(scm_qiffile);
-    
+                        "file_index", GINT_TO_POINTER(scm_file_index));
+    scm_file_index++;
+
     /* tack it on to the displayed list */
     new_loaded_file = g_list_alloc();
     new_loaded_file->next = NULL;
@@ -619,7 +595,7 @@ update_file_page(QIFImportWindow * wind) {
     /* now add the file to the loaded-files list */
     gtk_list_append_items(GTK_LIST(wind->selected_file_list), 
                           new_loaded_file);      
-
+    
     /* select_child will update the file info */
     if(scm_qiffile == wind->selected_file) {
       gtk_list_select_child(GTK_LIST(wind->selected_file_list), new_list_item);
@@ -642,13 +618,9 @@ update_file_page(QIFImportWindow * wind) {
 static void
 update_file_info(QIFImportWindow * wind, SCM qif_file) {
 
-  SCM   qif_file_radix_format;
-  SCM   qif_file_date_format;
   SCM   qif_file_currency;
   SCM   qif_file_path;
   SCM   qif_file_account;
-  SCM   scm_radix_format;
-  SCM   scm_date_format;
   SCM   scm_currency;
   SCM   scm_qif_account;
   SCM   scm_qif_path;
@@ -656,16 +628,12 @@ update_file_info(QIFImportWindow * wind, SCM qif_file) {
   int scm_strlen;
 
   /* look up the <qif-file> methods */
-  qif_file_radix_format = gh_eval_str("qif-file:radix-format");
-  qif_file_date_format  = gh_eval_str("qif-file:date-format");
   qif_file_currency     = gh_eval_str("qif-file:currency");
   qif_file_path         = gh_eval_str("qif-file:path");
-  qif_file_account      = gh_eval_str("qif-file:account");
+  qif_file_account      = gh_eval_str("qif-file:default-account");
   
   /* make sure the methods are loaded */
-  if((!gh_procedure_p(qif_file_radix_format)) ||
-     (!gh_procedure_p(qif_file_date_format)) ||
-     (!gh_procedure_p(qif_file_currency)) ||
+  if((!gh_procedure_p(qif_file_currency)) ||
      (!gh_procedure_p(qif_file_account)) ||
      (!gh_procedure_p(qif_file_path))) {
     gnc_error_dialog_parented(GTK_WINDOW(wind->dialog),
@@ -673,17 +641,7 @@ update_file_info(QIFImportWindow * wind, SCM qif_file) {
     return;
   }
   else {
-    /* stick the currently-selected qiffile scm in the window data */
-    gtk_object_set_data(GTK_OBJECT(wind->dialog), 
-                        "current_qif_file", GINT_TO_POINTER(qif_file));
-    
-    scm_protect_object(qif_file);
-
-    /* get the radix/date formats, currency etc from the Scheme side */
-    scm_radix_format  = gh_call1(qif_file_radix_format,
-                                 qif_file);
-    scm_date_format   = gh_call1(qif_file_date_format,
-                                 qif_file);
+    /* get the currency etc from the Scheme side */
     scm_currency      = gh_call1(qif_file_currency,
                                  qif_file);
     scm_qif_path      = gh_call1(qif_file_path,
@@ -702,42 +660,7 @@ update_file_info(QIFImportWindow * wind, SCM qif_file) {
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wind->acct_auto_button), 
                                  FALSE);
     gtk_entry_set_text(GTK_ENTRY(wind->acct_entry),
-                       gh_scm2newstr(scm_qif_account, &scm_strlen));
-    
-    /* set the option menu selections */
-    if(!strcmp(gh_symbol2newstr(scm_radix_format, &scm_strlen),
-               "unknown")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->radix_picker), 0);
-    }
-    else if(!strcmp(gh_symbol2newstr(scm_radix_format, &scm_strlen),
-                    "decimal")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->radix_picker), 1);
-    }
-    else if(!strcmp(gh_symbol2newstr(scm_radix_format, &scm_strlen),
-                    "comma")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->radix_picker), 2);
-    }
-
-    if(!strcmp(gh_symbol2newstr(scm_date_format, &scm_strlen),
-               "unknown")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->date_picker), 0);
-    }
-    else if(!strcmp(gh_symbol2newstr(scm_date_format, &scm_strlen),
-                    "m-d-y")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->date_picker), 1);
-    }
-    else if(!strcmp(gh_symbol2newstr(scm_date_format, &scm_strlen),
-                    "d-m-y")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->date_picker), 2);
-    }
-    else if(!strcmp(gh_symbol2newstr(scm_date_format, &scm_strlen),
-                    "y-m-d")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->date_picker), 3);
-    }
-    else if(!strcmp(gh_symbol2newstr(scm_date_format, &scm_strlen),
-                    "y-d-m")) {
-      gtk_option_menu_set_history(GTK_OPTION_MENU(wind->date_picker), 4);
-    }
+                       gh_scm2newstr(scm_qif_account, &scm_strlen));    
   }
 }
 
@@ -883,7 +806,6 @@ update_categories_page(QIFImportWindow * wind) {
              hash_data);
   }
   
-  
   /* now get the list of strings to display in the clist widget */
   /*   gnc_unprotect_object(wind->cat_display_info); */
   display_info = gh_call2(make_category_display, 
