@@ -14,8 +14,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 #include "top-level.h"
@@ -24,7 +28,9 @@
 #include "account-tree.h"
 #include "dialog-utils.h"
 #include "global-options.h"
+#include "gnc-currency-edit.h"
 #include "messages.h"
+#include "EuroUtils.h"
 #include "util.h"
 
 
@@ -83,12 +89,13 @@ GtkWidget *
 gnc_ui_create_account_label(int field_type)
 {
   GtkWidget *label;
-  gchar     *label_string = gnc_ui_get_account_field_name(field_type);
+  gchar *label_string;
+  const gchar *l = gnc_ui_get_account_field_name(field_type);
 
-  label_string = g_strconcat(label_string, ":", NULL);
+  label_string = g_strconcat(l, ":", NULL);
 
   label = gtk_label_new (label_string);
-  gtk_misc_set_alignment (GTK_MISC(label), 0.95, 0.5);
+  gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
   gtk_widget_show(label);
 
   g_free(label_string);
@@ -161,8 +168,8 @@ gnc_ui_account_field_box_create(AccountEditInfo * info,
   gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, TRUE, 0);
   gtk_widget_show(widget);
 
-  widget = gtk_entry_new();
-  info->currency_entry = GTK_EDITABLE(widget);
+  widget = gnc_currency_edit_new();
+  info->currency_entry = GTK_EDITABLE(GTK_COMBO(widget)->entry);
   gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, TRUE, 0);
   gtk_widget_show(widget);
     
@@ -201,6 +208,10 @@ gnc_get_source_name(gint source)
       return "T. Rowe Price";
     case SOURCE_VANGUARD :
       return "Vanguard";
+    case SOURCE_ASX :
+      return "ASX";
+    case SOURCE_TIAA_CREF :
+      return "TIAA-CREF";
   }
 
   PWARN("Unknown source");
@@ -225,6 +236,10 @@ gnc_get_source_code_name(gint source)
       return "TRPRICE";
     case SOURCE_VANGUARD :
       return "VANGUARD";
+    case SOURCE_ASX :
+      return "ASX";
+    case SOURCE_TIAA_CREF :
+      return "TIAACREF";
   }
 
   PWARN("Unknown source");
@@ -311,7 +326,7 @@ gnc_ui_account_source_box_create(AccountEditInfo * info)
   gtk_widget_show(hbox);
 
   widget = gtk_label_new(QUOTE_SRC_MSG);
-  gtk_misc_set_alignment (GTK_MISC(widget), 0.95, 0.5);
+  gtk_misc_set_alignment (GTK_MISC(widget), 1.0, 0.5);
   gtk_widget_show(widget);
 
   gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 5);
@@ -520,7 +535,7 @@ gnc_ui_install_field_strings(Account * account,
 			     gboolean new_code)
 {
   int accType;
-  gchar * old;
+  const char * old;
 
   xaccAccountSetName(account, strings->name);
   xaccAccountSetDescription(account, strings->description);
@@ -529,10 +544,7 @@ gnc_ui_install_field_strings(Account * account,
   if (safe_strcmp(old, strings->currency) != 0)
     xaccAccountSetCurrency(account, strings->currency);
 
-  if (safe_strcmp(strings->code, "") != 0)
-    xaccAccountSetCode(account, strings->code);
-  else
-    xaccAccountAutoCode(account, 4);
+  xaccAccountSetCode(account, strings->code);
 
   xaccAccountSetNotes(account, strings->notes);
 
@@ -556,7 +568,8 @@ gnc_ui_install_field_strings(Account * account,
 
 /* =========================================================== */
 
-char * gnc_ui_get_account_field_name(int field)
+const char *
+gnc_ui_get_account_field_name(int field)
 {
   assert((field >= 0) && (field < NUM_ACCOUNT_FIELDS));
 
@@ -586,6 +599,15 @@ char * gnc_ui_get_account_field_name(int field)
     case ACCOUNT_BALANCE :
       return BALN_STR;
       break;
+    case ACCOUNT_BALANCE_EURO :
+      return BALN_EURO_STR;
+      break;
+    case ACCOUNT_TOTAL :
+      return TOTAL_STR;
+      break;
+    case ACCOUNT_TOTAL_EURO :
+      return TOTAL_EURO_STR;
+      break;
   }
 
   assert(0);
@@ -594,33 +616,37 @@ char * gnc_ui_get_account_field_name(int field)
 
 
 double
-gnc_ui_get_account_full_balance(Account *account)
+gnc_ui_account_get_balance(Account *account, gboolean include_children)
 {
-  AccountGroup *acc_children;
   double balance;
-  int type;
 
-  assert(account != NULL);
+  if (account == NULL)
+    return 0.0;
 
-  acc_children = xaccAccountGetChildren (account);
-  type = xaccAccountGetType(account);
   balance = xaccAccountGetBalance (account);
 
-  /* if the account has children, add in their balance */
-  if (acc_children)
-    balance += xaccGroupGetBalance(acc_children);
+  if (include_children)
+  {
+    AccountGroup *children;
+
+    children = xaccAccountGetChildren (account);
+    balance += xaccGroupGetBalance (children);
+  }
 
   /* reverse sign if needed */
-  if (gnc_reverse_balance(account))
+  if (gnc_reverse_balance (account))
     balance = -balance;
 
   return balance;
 }
 
 
-char * gnc_ui_get_account_field_value_string(Account *account, int field)
+const char *
+gnc_ui_get_account_field_value_string(Account *account, int field)
 {
-  assert(account != NULL);
+  if (account == NULL)
+    return NULL;
+
   assert((field >= 0) && (field < NUM_ACCOUNT_FIELDS));
 
   switch (field)
@@ -648,9 +674,36 @@ char * gnc_ui_get_account_field_value_string(Account *account, int field)
       break;
     case ACCOUNT_BALANCE :
       {
-	double balance = gnc_ui_get_account_full_balance(account);
+        double balance = gnc_ui_account_get_balance(account, FALSE);
 
-	return xaccPrintAmount(balance, PRTSYM | PRTSEP);
+	return xaccPrintAmount(balance, PRTSYM | PRTSEP,
+			       xaccAccountGetCurrency(account));
+      }
+      break;
+    case ACCOUNT_BALANCE_EURO :
+      {
+	const char *account_currency = xaccAccountGetCurrency(account);
+        double balance = gnc_ui_account_get_balance(account, FALSE);
+	double euro_balance = gnc_convert_to_euro(account_currency, balance);
+
+	return xaccPrintAmount(euro_balance, PRTSYM | PRTSEP | PRTEUR, NULL);
+      }
+      break;
+    case ACCOUNT_TOTAL :
+      {
+	double balance = gnc_ui_account_get_balance(account, TRUE);
+
+	return xaccPrintAmount(balance, PRTSYM | PRTSEP,
+			       xaccAccountGetCurrency(account));
+      }
+      break;
+    case ACCOUNT_TOTAL_EURO :
+      {
+	const char *account_currency = xaccAccountGetCurrency(account);
+	double balance = gnc_ui_account_get_balance(account, TRUE);
+	double euro_balance = gnc_convert_to_euro(account_currency, balance);
+
+	return xaccPrintAmount(euro_balance, PRTSYM | PRTSEP | PRTEUR, NULL);
       }
       break;
   }
@@ -809,6 +862,8 @@ gnc_set_label_color(GtkWidget *label, double value)
     style->fg[GTK_STATE_NORMAL] = style->black;
 
   gtk_widget_set_style(label, style);
+
+  gtk_style_unref(style);
 #endif
 }
 
@@ -867,11 +922,11 @@ gnc_get_window_size(const char *prefix, int *width, int *height)
   if (gnc_lookup_boolean_option("General", "Save Window Geometry", FALSE))
   {
     name = g_strconcat(prefix, "_width", NULL);
-    w = gnc_lookup_number_range_option("__gui", name, 0.0);
+    w = gnc_lookup_number_option("__gui", name, 0.0);
     g_free(name);
 
     name = g_strconcat(prefix, "_height", NULL);
-    h = gnc_lookup_number_range_option("__gui", name, 0.0);
+    h = gnc_lookup_number_option("__gui", name, 0.0);
     g_free(name);
   }
   else
@@ -897,15 +952,74 @@ gnc_save_window_size(const char *prefix, int width, int height)
 
   name = g_strconcat(prefix, "_width", NULL);
   if (save)
-    gnc_set_number_range_option("__gui", name, width);
+    gnc_set_number_option("__gui", name, width);
   else
     gnc_set_option_default("__gui", name);
   g_free(name);
 
   name = g_strconcat(prefix, "_height", NULL);
   if (save)
-    gnc_set_number_range_option("__gui", name, height);
+    gnc_set_number_option("__gui", name, height);
   else
     gnc_set_option_default("__gui", name);
   g_free(name);
 }
+
+void
+gnc_fill_menu_with_data(GnomeUIInfo *info, gpointer data)
+{
+  if (info == NULL)
+    return;
+
+  while (1)
+  {
+    switch (info->type)
+    {
+      case GNOME_APP_UI_RADIOITEMS:
+      case GNOME_APP_UI_SUBTREE:
+      case GNOME_APP_UI_SUBTREE_STOCK:
+        gnc_fill_menu_with_data((GnomeUIInfo *) info->moreinfo, data);
+        break;
+      case GNOME_APP_UI_ENDOFINFO:
+        return;
+      default:
+        info->user_data = data;
+        break;
+    }
+
+    info++;
+  }
+}
+
+void
+gnc_option_menu_init(GtkWidget * w)
+{
+  GtkWidget * menu;
+  GtkWidget * active;
+  int i;
+
+  menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(w));
+
+  for(i = 0; i < g_list_length(GTK_MENU_SHELL(menu)->children); i++)
+  {
+    gtk_option_menu_set_history(GTK_OPTION_MENU(w), i);
+    active = gtk_menu_get_active(GTK_MENU(menu));
+    gtk_object_set_data(GTK_OBJECT(active), 
+                        "option_index",
+                        GINT_TO_POINTER(i));
+  }
+
+  gtk_option_menu_set_history(GTK_OPTION_MENU(w), 0);
+}
+
+int
+gnc_option_menu_get_active(GtkWidget * w)
+{
+  GtkWidget * menu;
+  GtkWidget * menuitem;
+
+  menu     = gtk_option_menu_get_menu(GTK_OPTION_MENU(w));
+  menuitem = gtk_menu_get_active(GTK_MENU(menu));
+  return GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(menuitem),
+                                             "option_index"));
+}
