@@ -13,8 +13,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "top-level.h"
@@ -241,7 +244,7 @@ gnc_ok_cancel_dialog_parented(gncUIWidget parent, const char *message,
       default_button = 1;
       break;
     default:
-      PWARN("gnc_verify_cancel_dialog: bad default button\n");
+      PWARN("bad default button\n");
       default_button = 0;
       break;
   }
@@ -300,7 +303,7 @@ gnc_verify_cancel_dialog_parented(GtkWidget *parent, const char *message,
       default_button = 2;
       break;
     default:
-      PWARN("gnc_verify_cancel_dialog: bad default button\n");
+      PWARN("bad default button\n");
       default_button = 0;
       break;
   }
@@ -334,10 +337,10 @@ gnc_verify_cancel_dialog_parented(GtkWidget *parent, const char *message,
  *                          "No" is the default button.             *
  * Return: true for "Yes", false for "No"                           *
 \********************************************************************/
-gncBoolean
-gnc_verify_dialog(const char *message, gncBoolean yes_is_default)
+gboolean
+gnc_verify_dialog(const char *message, gboolean yes_is_default)
 {
-  return gnc_verify_dialog_parented(GTK_WINDOW(gnc_get_ui_data()),
+  return gnc_verify_dialog_parented(gnc_get_ui_data(),
                                     message, yes_is_default);
 }
 
@@ -353,12 +356,12 @@ gnc_verify_dialog(const char *message, gncBoolean yes_is_default)
  *                          "No" is the default button.             *
  * Return: true for "Yes", false for "No"                           *
 \********************************************************************/
-gncBoolean
-gnc_verify_dialog_parented(GtkWindow *parent, const char *message,
-                           gncBoolean yes_is_default)
+gboolean
+gnc_verify_dialog_parented(gncUIWidget parent, const char *message,
+                           gboolean yes_is_default)
 {
   GtkWidget *verify_box = NULL;
-  
+
   verify_box = gnome_message_box_new(message,
 				     GNOME_MESSAGE_BOX_QUESTION,
 				     GNOME_STOCK_BUTTON_YES,
@@ -366,7 +369,7 @@ gnc_verify_dialog_parented(GtkWindow *parent, const char *message,
 				     NULL);
 
   if (parent != NULL)
-    gnome_dialog_set_parent(GNOME_DIALOG(verify_box), parent);
+    gnome_dialog_set_parent(GNOME_DIALOG(verify_box), GTK_WINDOW(parent));
 
   gnome_dialog_set_default(GNOME_DIALOG(verify_box), yes_is_default ? 0 : 1);
 
