@@ -15,8 +15,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "top-level.h"
@@ -25,7 +28,9 @@
 
 #include "gnucash.h"
 #include "messages.h"
+#include "Query.h"
 #include "reconcile-listP.h"
+#include "FileDialog.h"
 #include "date.h"
 #include "util.h"
 
@@ -82,6 +87,20 @@ gnc_reconcile_list_new(Account *account, GNCReconcileListType type)
   list->account = account;
   list->list_type = type;
 
+  list->query = xaccMallocQuery();
+
+  xaccQuerySetGroup(list->query, gncGetCurrentGroup());
+
+  /* match the account */
+  xaccQueryAddSingleAccountMatch(list->query, account, QUERY_OR);
+
+  if (type == RECLIST_CREDIT)
+    xaccQueryAddAmountMatch(list->query, 0.0, AMT_SGN_MATCH_CREDIT,
+                            AMT_MATCH_ATLEAST, QUERY_AND);
+  else
+    xaccQueryAddAmountMatch(list->query, 0.0, AMT_SGN_MATCH_DEBIT,
+                            AMT_MATCH_ATLEAST, QUERY_AND);
+
   return GTK_WIDGET(list);
 }
 
@@ -106,6 +125,7 @@ gnc_reconcile_list_init(GNCReconcileList *list)
   list->current_split = NULL;
   list->no_toggle = FALSE;
   list->always_unselect = FALSE;
+  list->query = NULL;
 
   while (titles[list->num_columns] != NULL)
     list->num_columns++;
@@ -297,6 +317,12 @@ gnc_reconcile_list_destroy(GtkObject *object)
     list->reconciled = NULL;
   }
 
+  if (list->query != NULL)
+  {
+    xaccFreeQuery(list->query);
+    list->query = NULL;
+  }
+
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
     (* GTK_OBJECT_CLASS(parent_class)->destroy) (object);
 }
@@ -432,7 +458,8 @@ gnc_reconcile_list_reconciled_balance(GNCReconcileList *list)
     if (g_hash_table_lookup(list->reconciled, split) == NULL)
       continue;
 
-    if((account_type == STOCK) || (account_type == MUTUAL))
+    if ((account_type == STOCK) || (account_type == MUTUAL) ||
+        (account_type == CURRENCY))
       total += xaccSplitGetShareAmount(split);
     else
       total += xaccSplitGetValue(split);
@@ -447,10 +474,11 @@ gnc_reconcile_list_reconciled_balance(GNCReconcileList *list)
  *   commit the reconcile information in the list                   *
  *                                                                  *
  * Args: list - list to commit                                      *
+ *       date - date to set as the reconcile date                   *
  * Returns: nothing                                                 *
 \********************************************************************/
 void
-gnc_reconcile_list_commit(GNCReconcileList *list)
+gnc_reconcile_list_commit(GNCReconcileList *list, time_t date)
 {
   GtkCList *clist = GTK_CLIST(list);
   Split *split;
@@ -467,7 +495,10 @@ gnc_reconcile_list_commit(GNCReconcileList *list)
     split = gtk_clist_get_row_data(clist, i);
 
     if (g_hash_table_lookup(list->reconciled, split) != NULL)
+    {
       xaccSplitSetReconcile(split, YREC);
+      xaccSplitSetDateReconciledSecs(split, date);
+    }
   }
 }
 
@@ -514,34 +545,63 @@ gnc_reconcile_list_changed(GNCReconcileList *list)
 }
 
 
+/********************************************************************\
+ * gnc_reconcile_list_set_sort_order                                *
+ *   sets the sorting order of splits in the list                   *
+ *                                                                  *
+ * Args: list - list to change the sort order for                   *
+ * Returns: nothing                                                 *
+\********************************************************************/
+void
+gnc_reconcile_list_set_sort_order(GNCReconcileList *list, sort_type_t key)
+{
+  g_return_if_fail(list != NULL);
+  g_return_if_fail(IS_GNC_RECONCILE_LIST(list));
+  g_return_if_fail(list->query != NULL);
+
+  xaccQuerySetSortOrder(list->query, key,
+                        (key == BY_STANDARD) ? BY_NONE : BY_STANDARD,
+                        BY_NONE);
+
+  if (list->list_type == RECLIST_DEBIT)
+    return;
+
+  xaccQuerySetSortIncreasing(list->query, !(key == BY_AMOUNT));
+}
+
 static void
 gnc_reconcile_list_fill(GNCReconcileList *list)
 {
   gchar *strings[list->num_columns + 1];
-  Transaction *trans;
-  Split *split;
+  GNCPrintAmountFlags flags = PRTSEP;
+  GNCAccountType account_type;
   gboolean reconciled;
-  int num_splits;
-  int account_type;
-  double amount;
+
+  Transaction *trans;
+  Split **splits;
+  Split *split;
+
+  const char *currency;
   char recn_str[2];
-  short shares = PRTSEP;
   char recn;
+
+  double amount;
   int row;
-  int i;
 
   account_type = xaccAccountGetType(list->account);
-  num_splits = xaccAccountGetNumSplits(list->account);
+  currency = xaccAccountGetCurrency(list->account);
   strings[4] = recn_str;
   strings[5] = NULL;
 
   if ((account_type == STOCK) || (account_type == MUTUAL) ||
       (account_type == CURRENCY))
-    shares |= PRTSHR;
+    flags |= PRTSHR;
 
-  for (i = 0; i < num_splits; i++)
+  splits = xaccQueryGetSplits(list->query);
+
+  for ( ; *splits != NULL; splits++)
   {
-    split = xaccAccountGetSplit(list->account, i);
+    split = *splits;
 
     recn = xaccSplitGetReconcile(split);
     if ((recn != NREC) && (recn != CREC))
@@ -552,17 +612,17 @@ gnc_reconcile_list_fill(GNCReconcileList *list)
     else
       amount = xaccSplitGetValue(split);
 
-    if ((amount < 0) && (list->list_type == RECLIST_CREDIT))
+    if ((amount < 0) && (list->list_type == RECLIST_DEBIT))
       continue;
-    if ((amount >= 0) && (list->list_type == RECLIST_DEBIT))
+    if ((amount >= 0) && (list->list_type == RECLIST_CREDIT))
       continue;
 
     trans = xaccSplitGetParent(split);
 
     strings[0] = xaccTransGetDateStr(trans);
-    strings[1] = xaccTransGetNum(trans);
-    strings[2] = xaccTransGetDescription(trans);
-    strings[3] = xaccPrintAmount(DABS(amount), shares);
+    strings[1] = (char *) xaccTransGetNum(trans);
+    strings[2] = (char *) xaccTransGetDescription(trans);
+    strings[3] = xaccPrintAmount(DABS(amount), flags, currency);
 
     reconciled = g_hash_table_lookup(list->reconciled, split) != NULL;
 
@@ -574,5 +634,7 @@ gnc_reconcile_list_fill(GNCReconcileList *list)
     gnc_reconcile_list_set_row_style(list, row, reconciled);
 
     list->num_splits++;
+
+    free (strings[0]);
   }
 }
