@@ -14,8 +14,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
 
@@ -47,6 +50,7 @@
 #include "date.h"
 #include "AccWindow.h"
 #include "SplitLedger.h"
+#include "guile-util.h"
 #include "splitreg.h"
 
 
@@ -63,6 +67,9 @@ static void gnc_configure_register_borders_cb(void *);
 static void gnc_configure_register_borders(void);
 static void gnc_configure_reverse_balance_cb(void *);
 static void gnc_configure_reverse_balance(void);
+static void gnc_configure_sr_label_callbacks(void);
+static void gnc_configure_min_decimal_places_cb(void *);
+static void gnc_configure_min_decimal_places(void);
 
 /** GLOBALS *********************************************************/
 /* This static indicates the debugging module that this .o belongs to.  */
@@ -80,6 +87,7 @@ static SCM account_separator_callback_id = SCM_UNDEFINED;
 static SCM register_colors_callback_id = SCM_UNDEFINED;
 static SCM register_borders_callback_id = SCM_UNDEFINED;
 static SCM reverse_balance_callback_id = SCM_UNDEFINED;
+static SCM min_decimal_places_callback_id = SCM_UNDEFINED;
 
 /* ============================================================== */
 
@@ -118,7 +126,7 @@ gnucash_ui_init()
   int fake_argc = 1;
   char *fake_argv[] = {"gnucash"};
 
-  ENTER ("gnucash_ui_init");
+  ENTER ("\n");
 
   /* We're going to have to have other ways to handle X and GUI
      specific args... */
@@ -164,13 +172,21 @@ gnucash_ui_init()
                                           NULL, "General",
                                           "Reversed-balance account types");
 
+    gnc_configure_min_decimal_places();
+    min_decimal_places_callback_id =
+      gnc_register_option_change_callback(gnc_configure_min_decimal_places_cb,
+                                          NULL, "General",
+                                          "Minimum displayed decimal places");
+
+    gnc_configure_sr_label_callbacks();
+
     mainWindow();
 
     gnucash_style_init();
     gnucash_color_init();
   }
 
-  LEAVE ("gnucash_ui_init");
+  LEAVE ("\n");
 
   return 0;
 }
@@ -214,6 +230,7 @@ gnc_ui_destroy (void)
   gnc_unregister_option_change_callback_id(register_colors_callback_id);
   gnc_unregister_option_change_callback_id(register_borders_callback_id);  
   gnc_unregister_option_change_callback_id(reverse_balance_callback_id);  
+  gnc_unregister_option_change_callback_id(min_decimal_places_callback_id);
 
   if (app != NULL)
   {
@@ -268,6 +285,61 @@ gnucash_ui_select_file()
 
 /* ============================================================== */
 
+static GNCAccountType
+sr_type_to_account_type(SplitRegisterType sr_type)
+{
+  switch (sr_type)
+  {
+    case BANK_REGISTER:
+      return BANK;
+    case CASH_REGISTER:
+      return CASH;
+    case ASSET_REGISTER:
+      return ASSET;
+    case CREDIT_REGISTER:
+      return CREDIT;
+    case LIABILITY_REGISTER:
+      return LIABILITY;
+    case INCOME_LEDGER:  
+    case INCOME_REGISTER:
+      return INCOME;
+    case EXPENSE_REGISTER:
+      return EXPENSE;
+    case STOCK_REGISTER:
+    case PORTFOLIO_LEDGER:
+      return STOCK;
+    case CURRENCY_REGISTER:
+      return CURRENCY;
+    case GENERAL_LEDGER:  
+      return NO_TYPE;
+    case EQUITY_REGISTER:
+      return EQUITY;
+    case SEARCH_LEDGER:
+      return NO_TYPE;
+    default:
+      return NO_TYPE;
+  }
+}
+
+static char *
+gnc_sr_debit_string(SplitRegisterType sr_type)
+{
+  return gnc_get_debit_string(sr_type_to_account_type(sr_type));
+}
+
+static char *
+gnc_sr_credit_string(SplitRegisterType sr_type)
+{
+  return gnc_get_credit_string(sr_type_to_account_type(sr_type));
+}
+
+static void
+gnc_configure_sr_label_callbacks()
+{
+  xaccSplitRegisterSetDebitStringGetter(gnc_sr_debit_string);
+  xaccSplitRegisterSetCreditStringGetter(gnc_sr_credit_string);
+}
+
 /* gnc_configure_date_format_cb
  *    Callback called when options change - sets dateFormat to the current
  *    value on the scheme side and refreshes register windows
@@ -383,7 +455,7 @@ gnc_configure_account_separator_cb(void *data)
 }
 
 /* gnc_configure_account_separator
- *    sets the accoutn separator to the
+ *    sets the account separator to the
  *    current value on the scheme side
  *
  * Args: Nothing
@@ -397,6 +469,38 @@ gnc_configure_account_separator(void)
   xaccSRSetAccountSeparator(separator);
 }
 
+/* gnc_configure_min_decimal_places_cb
+ *    Callback called when options change - sets
+ *    minimum decimal places to display.
+ *
+ * Args: Nothing
+ * Returns: Nothing
+ */
+static void 
+gnc_configure_min_decimal_places_cb(void *data)
+{
+  gnc_configure_min_decimal_places();
+  gnc_group_ui_refresh(gncGetCurrentGroup());
+  gnc_refresh_main_window();
+}
+
+/* gnc_configure_min_decimal_places
+ *    sets the minimum decimal places to display
+ *    to the current value on the scheme side
+ *
+ * Args: Nothing
+ * Returns: Nothing
+ */
+static void
+gnc_configure_min_decimal_places(void)
+{
+  int places = gnc_lookup_number_option ("General",
+                                         "Minimum displayed decimal places",
+                                         0.0);
+
+  gnc_set_mininum_decimal_places (places);
+}
+
 /* gnc_configure_register_colors_cb
  *    Callback called when options change - sets
  *    register colors to their guile values
@@ -541,6 +645,15 @@ gnc_configure_reverse_balance_cb(void *not_used)
 
 static gncBoolean reverse_type[NUM_ACCOUNT_TYPES];
 
+gncBoolean
+gnc_reverse_balance_type(int type)
+{
+  if ((type < 0) || (type >= NUM_ACCOUNT_TYPES))
+    return GNC_F;
+
+  return reverse_type[type];
+}
+
 gncBoolean
 gnc_reverse_balance(Account *account)
 {
@@ -594,7 +707,7 @@ gnc_configure_reverse_balance(void)
   }
   else
   {
-    PERR("gnc_configure_reverse_balance: bad value\n");
+    PERR("bad value\n");
 
     reverse_type[INCOME]  = GNC_T;
     reverse_type[EXPENSE] = GNC_T;
