@@ -15,8 +15,11 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
 #include "top-level.h"
@@ -29,11 +32,13 @@
 #include "AccWindow.h"
 #include "AdjBWindow.h"
 #include "global-options.h"
+#include "dialog-options.h"
 #include "FileDialog.h"
 #include "g-wrap.h"
 #include "gnucash.h"
 #include "MainWindow.h"
 #include "Destroy.h"
+#include "ui-callbacks.h"
 #include "enriched-messages.h"
 #include "RegWindow.h"
 #include "Refresh.h"
@@ -46,10 +51,15 @@
 #include "dialog-transfer.h"
 #include "dialog-edit.h"
 #include "dialog-qif-import.h"
+#include "dialog-find-transactions.h"
+#include "file-history.h"
+#include "EuroUtils.h"
 #include "Scrub.h"
 #include "util.h"
 #include "gnc.h"
 
+#include "gtkselect.h"
+
 
 /* This static indicates the debugging module that this .o belongs to.  */
 static short module = MOD_GUI;
@@ -64,35 +74,197 @@ enum {
   FMB_QUIT,
 };
 
+/*
+ * An accumulator for a given currency.
+ *
+ * This is used during the update to the status bar to contain the
+ * accumulation for a single currency. These are placed in a GList and
+ * kept around for the duration of the calculation. There may, in fact
+ * be better ways to do this, but none occurred.
+ */
+struct _GNCCurrencyAcc {
+  const char *currency;
+  double assets;
+  double profits;
+};
+typedef struct _GNCCurrencyAcc GNCCurrencyAcc;
+
+/*
+ * An item to appear in the selector box in the status bar.
+ *
+ * This is maintained for the duration, where there is one per currency,
+ * plus (eventually) one for the default currency accumulation (like the
+ * EURO).
+ */
+struct _GNCCurrencyItem {
+  const char *currency;
+  GtkWidget *listitem;
+  GtkWidget *assets_label;
+  GtkWidget *profits_label;
+  gint touched:1;
+};
+typedef struct _GNCCurrencyItem GNCCurrencyItem;
+
+/*
+ * Build a single currency item.
+ *
+ * This function handles the building of a single currency item for the
+ * selector. It looks like the old code in the update function, but now
+ * only handles a single currency.
+ */
+static GNCCurrencyItem *
+gnc_ui_build_currency_item(const char *currency)
+{
+  GtkWidget *label;
+  GtkWidget *topbox;
+  GtkWidget *hbox;
+  GtkWidget *listitem;
+  GNCCurrencyItem *item = g_new0(GNCCurrencyItem, 1);
+
+  item->currency = currency;
+
+  listitem = gtk_list_item_new();
+  item->listitem = listitem;
+
+  topbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(topbox);
+  gtk_container_add(GTK_CONTAINER(listitem), topbox);
+
+  hbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(topbox), hbox, FALSE, FALSE, 5);
+
+  label = gtk_label_new(currency);
+  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+  hbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(topbox), hbox, FALSE, FALSE, 5);
+
+  label = gtk_label_new(ASSETS_C_STR);
+  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+  label = gtk_label_new("");
+  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show(label);
+  item->assets_label = label;
+
+  hbox = gtk_hbox_new(FALSE, 2);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(topbox), hbox, FALSE, FALSE, 5);
+
+  label = gtk_label_new(PROFITS_C_STR);
+  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+  gtk_widget_show(label);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+  label = gtk_label_new("");
+  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+  gtk_widget_show(label);
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  item->profits_label = label;
+
+  gtk_widget_show(item->listitem);
+
+  return item;
+}
+
+
+/*
+ * Get a currency accumulator.
+ *
+ * This will search the given list, and if no accumulator is found, wil
+ * allocate a fresh one. This may cause problems with currencies that have
+ * the same name... let the buyer beware.
+ */
+static GNCCurrencyAcc *
+gnc_ui_get_currency_accumulator(GList **list, const char *currency)
+{
+  GList *current;
+  GNCCurrencyAcc *found;
+
+  for (current = g_list_first(*list); current;
+       current = g_list_next(current))
+  {
+    found = current->data;
+    if (safe_strcmp(found->currency, currency) == 0)
+      return found;
+  }
+
+  found = g_new0(GNCCurrencyAcc, 1);
+  found->currency = currency;
+  found->assets = 0.0;
+  found->profits = 0.0;
+  *list = g_list_append(*list, found);
+
+  return found;
+}
+
+/*
+ * Get a currency item.
+ *
+ * This will search the given list, and if no accumulator is found, will
+ * create a fresh one.
+ *
+ * It looks just like the function above, with some extra stuff to get the
+ * item into the list.
+ */
+static GNCCurrencyItem *
+gnc_ui_get_currency_item(GList **list, const char *currency, GtkWidget *holder)
+{
+  GList *current;
+  GNCCurrencyItem *found;
+
+  for (current = g_list_first(*list); current;
+       current = g_list_next(current))
+  {
+    found = current->data;
+    if (safe_strcmp(found->currency, currency) == 0)
+      return found;
+  }
+
+  found = gnc_ui_build_currency_item(currency);
+  *list = g_list_append(*list, found);
+
+  current = g_list_append(NULL, found->listitem);
+  gtk_select_append_items(GTK_SELECT(holder), current);
+
+  return found;
+}
 
 static void
-gnc_ui_refresh_statusbar()
+gnc_ui_accounts_recurse (AccountGroup *group, GList **currency_list,
+                         gboolean euro)
 {
-  GNCMainInfo *main_info;
-  double assets  = 0.0;
-  double profits = 0.0;
   double amount;
-  AccountGroup *group;
   AccountGroup *children;
   Account *account;
-  char *asset_string;
-  char *profit_string;
   int num_accounts;
   int account_type;
+  const char *account_currency;
+  GNCCurrencyAcc *currency_accum;
+  GNCCurrencyAcc *euro_accum = NULL;
   int i;
 
-  main_info = gnc_get_main_info();
-  if (main_info == NULL)
-    return;
+  if (euro)
+    euro_accum = gnc_ui_get_currency_accumulator(currency_list,
+						 EURO_TOTAL_STR);
 
-  group = gncGetCurrentGroup ();
   num_accounts = xaccGroupGetNumAccounts(group);
   for (i = 0; i < num_accounts; i++)
   {
     account = xaccGroupGetAccount(group, i);
 
     account_type = xaccAccountGetType(account);
+    account_currency = xaccAccountGetCurrency(account);
     children = xaccAccountGetChildren(account);
+    currency_accum = gnc_ui_get_currency_accumulator(currency_list,
+						     account_currency);
 
     switch (account_type)
     {
@@ -104,44 +276,190 @@ gnc_ui_refresh_statusbar()
       case CREDIT:
       case LIABILITY:
 	amount = xaccAccountGetBalance(account);
-	if (children != NULL)
-	  amount += xaccGroupGetBalance(children);
+        currency_accum->assets += amount;
+	if(euro)
+	  euro_accum->assets += gnc_convert_to_euro(account_currency, amount);
 
-        assets += amount;
+	if (children != NULL)
+	  gnc_ui_accounts_recurse(children, currency_list, euro);
 	break;
       case INCOME:
       case EXPENSE:
 	amount = xaccAccountGetBalance(account);
-	if (children != NULL)
-	  amount += xaccGroupGetBalance(children);
+        currency_accum->profits -= amount;
+	if(euro)
+	  euro_accum->profits -= gnc_convert_to_euro(account_currency, amount);
 
-        profits -= amount;
+	if (children != NULL)
+	  gnc_ui_accounts_recurse(children, currency_list, euro);
 	break;
       case EQUITY:
+        /* no-op, see comments at top about summing assets */
+	break;
       case CURRENCY:
       default:
 	break;
     }
   }
-
-  asset_string = xaccPrintAmount(assets, PRTSYM | PRTSEP);
-  gtk_label_set_text(GTK_LABEL(main_info->assets_label), asset_string);
-  gnc_set_label_color(main_info->assets_label, assets);
-
-  profit_string = xaccPrintAmount(profits, PRTSYM | PRTSEP);
-  gtk_label_set_text(GTK_LABEL(main_info->profits_label), profit_string);
-  gnc_set_label_color(main_info->profits_label, profits);
 }
 
-/* Required for compatibility with Motif code. */
+/* The gnc_ui_refresh_statusbar() subroutine redraws 
+ *    the statusbar at the bottom of the main window. The statusbar
+ *    includes two fields, titled 'profits' and 'assets'.  The total
+ *    assets equal the sum of all of the non-equity, non-income
+ *    accounts.  In theory, assets also equals the grand total
+ *    value of the equity accounts, but that assumes that folks are
+ *    using the equity account type correctly (which is not likely).
+ *    Thus we show the sum of assets, rather than the sum of equities.
+ *
+ * The EURO gets special treatment. There can be one line with
+ * EUR amounts and a EUR (total) line which summs up all EURO
+ * member currencies.
+ *
+ * There should be a 'grand total', too, which sums up all accounts
+ * converted to one common currency.
+ */
+static void
+gnc_ui_refresh_statusbar (void)
+{
+  GNCMainInfo *main_info;
+  AccountGroup *group;
+  char asset_string[256];
+  char profit_string[256];
+  const char *default_currency;
+  GNCCurrencyAcc *currency_accum;
+  GNCCurrencyItem *currency_item;
+  GList *currency_list;
+  GList *current;
+  gboolean euro;
+
+  default_currency = gnc_lookup_string_option("International",
+					      "Default Currency",
+					      "USD");
+
+  euro = gnc_lookup_boolean_option("International",
+				   "Enable EURO support",
+				   FALSE);
+
+  main_info = gnc_get_main_info();
+  if (main_info == NULL)
+    return;
+
+  currency_list = NULL;
+
+  /* Make sure there's at least one accumulator in the list. */
+  gnc_ui_get_currency_accumulator (&currency_list, default_currency);
+
+  group = gncGetCurrentGroup ();
+  gnc_ui_accounts_recurse(group, &currency_list, euro);
+
+  for (current = g_list_first(main_info->totals_list); current;
+       current = g_list_next(current))
+  {
+    currency_item = current->data;
+    currency_item->touched = 0;
+  }
+
+  for (current = g_list_first(currency_list); current;
+       current = g_list_next(current))
+  {
+    currency_accum = current->data;
+    currency_item = gnc_ui_get_currency_item(&main_info->totals_list,
+       					     currency_accum->currency,
+					     main_info->totals_combo);
+    currency_item->touched = 1;
+
+    xaccSPrintAmount(asset_string, currency_accum->assets,
+		     PRTSYM | PRTSEP, currency_accum->currency);
+    gtk_label_set_text(GTK_LABEL(currency_item->assets_label), asset_string);
+    gnc_set_label_color(currency_item->assets_label, currency_accum->assets);
+
+    xaccSPrintAmount(profit_string, currency_accum->profits,
+		     PRTSYM | PRTSEP, currency_accum->currency);
+    gtk_label_set_text(GTK_LABEL(currency_item->profits_label), profit_string);
+    gnc_set_label_color(currency_item->profits_label, currency_accum->profits);
+
+    g_free(currency_accum);
+    current->data = NULL;
+  }
+
+  g_list_free(currency_list);
+  currency_list = NULL;
+
+  current = g_list_first(main_info->totals_list);
+  while (current)
+  {
+    GList *next = current->next;
+
+    currency_item = current->data;
+    if (currency_item->touched == 0
+        && strcmp(currency_item->currency, default_currency) != 0)
+    {
+      currency_list = g_list_append(currency_list, currency_item->listitem);
+      main_info->totals_list = g_list_remove_link(main_info->totals_list,
+						  current);
+      g_free(currency_item);
+      current->data = NULL;
+      g_list_free_1(current);
+    }
+
+    current = next;
+  }
+
+  if (currency_list)
+  {
+    gtk_select_remove_items(GTK_SELECT(main_info->totals_combo),
+                            currency_list);
+    g_list_free(currency_list);
+  }
+}
+
+static void
+gnc_refresh_main_window_title()
+{
+  GtkWidget *main_window;
+  Session *session;
+  gchar *filename;
+  gchar *title;
+
+  main_window = gnc_get_ui_data();
+  if (main_window == NULL)
+    return;
+
+  session = gncGetCurrentSession();
+
+  if (session == NULL)
+    filename = UNTITLED_STR;
+  else
+    filename = xaccSessionGetFilePath(session);
+
+  if ((filename == NULL) || (*filename == '\0'))
+    filename = UNTITLED_STR;
+
+  title = g_strconcat("GnuCash - ", filename, NULL);
+
+  gtk_window_set_title(GTK_WINDOW(main_window), title);
+
+  g_free(title);
+}
+
 void
 gnc_refresh_main_window()
 {
   xaccRecomputeGroupBalance(gncGetCurrentGroup());
   gnc_ui_refresh_statusbar();
+  gnc_history_update_menu(GNOME_APP(gnc_get_ui_data()));
   gnc_account_tree_refresh_all();
+  gnc_refresh_main_window_title();
 }
 
+static void
+gnc_ui_find_transactions_cb ( GtkWidget *widget, gpointer data )
+{
+  gnc_ui_find_transactions_dialog_create(NULL);
+}
+
+
 static void
 gnc_ui_exit_cb ( GtkWidget *widget, gpointer data )
 {
@@ -190,7 +508,7 @@ gnc_ui_delete_account ( Account *account )
   xaccFreeAccount(account);
 
   /* Step 4: Refresh things */
-  gnc_ui_refresh_statusbar();
+  gnc_refresh_main_window();
   gnc_group_ui_refresh(gncGetCurrentGroup());
 }
 
@@ -201,7 +519,7 @@ gnc_ui_delete_account_cb ( GtkWidget *widget, gpointer data )
 
   if (account)
   {
-    gchar *name;
+    const gchar *name;
     gchar *message;
 
     name = xaccAccountGetName(account);
@@ -429,6 +747,9 @@ gnc_ui_mainWindow_destroy_cb(GtkObject *object, gpointer user_data)
   gnc_unregister_option_change_callback_id
     (main_info->main_window_change_callback_id);
 
+  gnc_unregister_option_change_callback_id
+    (main_info->euro_change_callback_id);
+
   gnc_unregister_option_change_callback_id
     (main_info->toolbar_change_callback_id);
 
@@ -576,7 +897,18 @@ gnc_configure_account_tree(void *data)
       new_avi.show_field[ACCOUNT_SECURITY] = TRUE;
 
     else if (safe_strcmp(node->data, "balance") == 0)
+    {
       new_avi.show_field[ACCOUNT_BALANCE] = TRUE;
+      if(gnc_lookup_boolean_option("International", "Enable EURO support", FALSE))
+	new_avi.show_field[ACCOUNT_BALANCE_EURO] = TRUE;
+    }
+
+    else if (safe_strcmp(node->data, "total") == 0)
+    {
+      new_avi.show_field[ACCOUNT_TOTAL] = TRUE;
+      if(gnc_lookup_boolean_option("International", "Enable EURO support", FALSE))
+	new_avi.show_field[ACCOUNT_TOTAL_EURO] = TRUE;
+    }
   }
 
   gnc_free_list_option_value(list);
@@ -589,26 +921,24 @@ gnc_configure_account_tree(void *data)
     gnc_account_tree_set_view_info(tree, &new_avi);
 }
 
+static void
+gnc_euro_change(void *data)
+{
+  gnc_ui_refresh_statusbar();
+  gnc_configure_account_tree(data);
+  gnc_group_ui_refresh(gncGetCurrentGroup());
+}
+
 static void
 gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
 {
   GSList *list;
 
-  GnomeUIInfo toolbar[] = 
+  static GnomeUIInfo toolbar[] = 
   {
     { GNOME_APP_UI_ITEM,
-      OPEN_STR,
-      TOOLTIP_OPEN_FILE,
-      gnc_ui_filemenu_cb, 
-      GINT_TO_POINTER(FMB_OPEN),
-      NULL,
-      GNOME_APP_PIXMAP_STOCK, 
-      GNOME_STOCK_PIXMAP_OPEN,
-      0, 0, NULL
-    },
-    { GNOME_APP_UI_ITEM,
-      SAVE_STR,
-      TOOLTIP_SAVE_FILE,
+      SAVE_STR_N,
+      TOOLTIP_SAVE_FILE_N,
       gnc_ui_filemenu_cb, 
       GINT_TO_POINTER(FMB_SAVE),
       NULL,
@@ -617,8 +947,8 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
       0, 0, NULL
     },
     { GNOME_APP_UI_ITEM,
-      IMPORT_STR,
-      TOOLTIP_IMPORT_QIF,
+      IMPORT_STR_N,
+      TOOLTIP_IMPORT_QIF_N,
       gnc_ui_filemenu_cb, 
       GINT_TO_POINTER(FMB_IMPORT),
       NULL,
@@ -628,8 +958,8 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
     },
     GNOMEUIINFO_SEPARATOR,
     { GNOME_APP_UI_ITEM, 
-      OPEN_STR,
-      TOOLTIP_OPEN,
+      OPEN_STR_N,
+      TOOLTIP_OPEN_N,
       gnc_ui_mainWindow_toolbar_open, 
       NULL,
       NULL,
@@ -638,8 +968,8 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
       0, 0, NULL 
     },
     { GNOME_APP_UI_ITEM,
-      EDIT_STR,
-      TOOLTIP_EDIT,
+      EDIT_STR_N,
+      TOOLTIP_EDIT_N,
       gnc_ui_mainWindow_toolbar_edit, 
       NULL,
       NULL,
@@ -649,8 +979,8 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
     },
     GNOMEUIINFO_SEPARATOR,
     { GNOME_APP_UI_ITEM,
-      NEW_STR,
-      TOOLTIP_NEW,
+      NEW_STR_N,
+      TOOLTIP_NEW_N,
       gnc_ui_add_account, 
       NULL,
       NULL,
@@ -659,8 +989,8 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
       0, 0, NULL
     },
     { GNOME_APP_UI_ITEM,
-      DELETE_STR,
-      TOOLTIP_DELETE,
+      DELETE_STR_N,
+      TOOLTIP_DELETE_N,
       gnc_ui_delete_account_cb, 
       NULL,
       NULL,
@@ -670,8 +1000,19 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
     },
     GNOMEUIINFO_SEPARATOR,
     { GNOME_APP_UI_ITEM,
-      EXIT_STR,
-      TOOLTIP_EXIT,
+      FIND_STR_N,
+      TOOLTIP_FIND_N,
+      gnc_ui_find_transactions_cb, 
+      NULL,
+      NULL,
+      GNOME_APP_PIXMAP_STOCK,
+      GNOME_STOCK_PIXMAP_SEARCH,
+      0, 0, NULL
+    },
+    GNOMEUIINFO_SEPARATOR,
+    { GNOME_APP_UI_ITEM,
+      EXIT_STR_N,
+      TOOLTIP_EXIT_N,
       gnc_ui_exit_cb, 
       NULL,
       NULL,
@@ -686,9 +1027,9 @@ gnc_main_create_toolbar(GnomeApp *app, GNCMainInfo *main_info)
 
   list = main_info->account_sensitives;
 
+  list = g_slist_prepend(list, toolbar[3].widget);
   list = g_slist_prepend(list, toolbar[4].widget);
-  list = g_slist_prepend(list, toolbar[5].widget);
-  list = g_slist_prepend(list, toolbar[8].widget);
+  list = g_slist_prepend(list, toolbar[7].widget);
 
   main_info->account_sensitives = list;
 }
@@ -700,9 +1041,9 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
   GtkWidget *popup;
   GSList *list;
 
-  GnomeUIInfo filemenu[] = {
-    GNOMEUIINFO_MENU_NEW_ITEM(NEW_FILE_STR,
-                              TOOLTIP_NEW_FILE,
+  static GnomeUIInfo filemenu[] = {
+    GNOMEUIINFO_MENU_NEW_ITEM(NEW_FILE_STR_N,
+                              TOOLTIP_NEW_FILE_N,
                               gnc_ui_filemenu_cb,
                               GINT_TO_POINTER(FMB_NEW)),
     GNOMEUIINFO_MENU_OPEN_ITEM(gnc_ui_filemenu_cb,
@@ -714,7 +1055,7 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_SEPARATOR,
     {
       GNOME_APP_UI_ITEM,
-      IMPORT_QIF_E_STR, TOOLTIP_IMPORT_QIF,
+      IMPORT_QIF_E_STR_N, TOOLTIP_IMPORT_QIF_N,
       gnc_ui_filemenu_cb, GINT_TO_POINTER(FMB_IMPORT), NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CONVERT,
       'i', GDK_CONTROL_MASK, NULL
@@ -725,10 +1066,10 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_END
   };
 
-  GnomeUIInfo optionsmenu[] = {
+  static GnomeUIInfo optionsmenu[] = {
     {
       GNOME_APP_UI_ITEM,
-      PREFERENCES_MENU_E_STR, TOOLTIP_PREFERENCES,
+      PREFERENCES_MENU_E_STR_N, TOOLTIP_PREFERENCES_N,
       gnc_ui_options_cb, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
       0, 0, NULL
@@ -736,24 +1077,24 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_END
   };
 
-  GnomeUIInfo scrubmenu[] = {
+  static GnomeUIInfo scrubmenu[] = {
     {
       GNOME_APP_UI_ITEM,
-      SCRUB_ACC_MENU_STR, TOOLTIP_SCRUB_ACCT,
+      SCRUB_ACC_MENU_STR_N, TOOLTIP_SCRUB_ACCT_N,
       gnc_ui_mainWindow_scrub, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       0, 0, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      SCRUB_SUB_MENU_STR, TOOLTIP_SCRUB_SUB,
+      SCRUB_SUB_MENU_STR_N, TOOLTIP_SCRUB_SUB_N,
       gnc_ui_mainWindow_scrub_sub, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       0, 0, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      SCRUB_ALL_MENU_STR, TOOLTIP_SCRUB_ALL,
+      SCRUB_ALL_MENU_STR_N, TOOLTIP_SCRUB_ALL_N,
       gnc_ui_mainWindow_scrub_all, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       0, 0, NULL
@@ -761,24 +1102,24 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_END
   };
 
-  GnomeUIInfo accountsmenu[] = {
+  static GnomeUIInfo accountsmenu[] = {
     {
       GNOME_APP_UI_ITEM,
-      OPEN_ACC_MENU_STR, TOOLTIP_OPEN,
+      OPEN_ACC_MENU_STR_N, TOOLTIP_OPEN_N,
       gnc_ui_mainWindow_toolbar_open, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
       'o', GDK_CONTROL_MASK, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      OPEN_SUB_MENU_STR, TOOLTIP_OPEN_SUB,
+      OPEN_SUB_MENU_STR_N, TOOLTIP_OPEN_SUB_N,
       gnc_ui_mainWindow_toolbar_open_subs, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
       0, 0, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      EDIT_ACC_MENU_STR, TOOLTIP_EDIT,
+      EDIT_ACC_MENU_STR_N, TOOLTIP_EDIT_N,
       gnc_ui_mainWindow_toolbar_edit, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
       'e', GDK_CONTROL_MASK, NULL
@@ -786,21 +1127,21 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_SEPARATOR,
     {
       GNOME_APP_UI_ITEM,
-      RECONCILE_MENU_E_STR, TOOLTIP_RECONCILE,
+      RECONCILE_MENU_E_STR_N, TOOLTIP_RECONCILE_N,
       gnc_ui_mainWindow_reconcile, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       'r', GDK_CONTROL_MASK, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      TRANSFER_MENU_E_STR, TOOLTIP_TRANSFER,
+      TRANSFER_MENU_E_STR_N, TOOLTIP_TRANSFER_N,
       gnc_ui_mainWindow_transfer, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       't', GDK_CONTROL_MASK, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      ADJ_BALN_MENU_E_STR, TOOLTIP_ADJUST,
+      ADJ_BALN_MENU_E_STR_N, TOOLTIP_ADJUST_N,
       gnc_ui_mainWindow_adjust_balance, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       'b', GDK_CONTROL_MASK, NULL
@@ -808,27 +1149,27 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_SEPARATOR,
     {
       GNOME_APP_UI_ITEM,
-      NEW_ACC_MENU_E_STR, TOOLTIP_NEW,
+      NEW_ACC_MENU_E_STR_N, TOOLTIP_NEW_N,
       gnc_ui_add_account, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_ADD,
       0, 0, NULL
     },
     {
       GNOME_APP_UI_ITEM,
-      DEL_ACC_MENU_STR, TOOLTIP_DELETE,
+      DEL_ACC_MENU_STR_N, TOOLTIP_DELETE_N,
       gnc_ui_delete_account_cb, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REMOVE,
       0, 0, NULL
     },
     GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_SUBTREE(SCRUB_MENU_STR, scrubmenu),
+    GNOMEUIINFO_SUBTREE(SCRUB_MENU_STR_N, scrubmenu),
     GNOMEUIINFO_END
   };  
 
-  GnomeUIInfo helpmenu[] = {
+  static GnomeUIInfo helpmenu[] = {
     {
       GNOME_APP_UI_ITEM,
-      HELP_MENU_STR, TOOLTIP_HELP,
+      MAN_MENU_STR_N, TOOLTIP_MAN_N,
       gnc_ui_help_cb, NULL, NULL,
       GNOME_APP_PIXMAP_NONE, NULL,
       0, 0, NULL
@@ -837,9 +1178,9 @@ gnc_main_create_menus(GnomeApp *app, GtkWidget *account_tree,
     GNOMEUIINFO_END
   };
 
-  GnomeUIInfo mainmenu[] = {
+  static GnomeUIInfo mainmenu[] = {
     GNOMEUIINFO_MENU_FILE_TREE(filemenu),
-    GNOMEUIINFO_SUBTREE(ACCOUNTS_MENU_STR, accountsmenu),
+    GNOMEUIINFO_SUBTREE(ACCOUNTS_MENU_STR_N, accountsmenu),
     GNOMEUIINFO_MENU_SETTINGS_TREE(optionsmenu),
     GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
     GNOMEUIINFO_END
@@ -957,6 +1298,11 @@ mainWindow()
     gnc_register_option_change_callback(gnc_configure_account_tree, NULL,
                                         "Main Window", NULL);
 
+  main_info->euro_change_callback_id =
+    gnc_register_option_change_callback(gnc_euro_change, NULL,
+                                        "International",
+                                        "Enable EURO support");
+
   gtk_signal_connect(GTK_OBJECT(main_info->account_tree), "activate_account",
 		     GTK_SIGNAL_FUNC (gnc_account_tree_activate_cb), NULL);
 
@@ -983,32 +1329,21 @@ mainWindow()
 
   /* create the label containing the account balances */
   {
-    GtkWidget *label;
-    GtkWidget *hbox;
+    GtkWidget *combo_box;
+    const char *default_currency;
+    GNCCurrencyItem *def_item;
 
-    hbox = gtk_hbox_new(FALSE, 2);
-    gtk_box_pack_end(GTK_BOX(statusbar), hbox, FALSE, FALSE, 5);
-
-    label = gtk_label_new(ASSETS_C_STR);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.95, 0.5);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-    label = gtk_label_new("");
-    gtk_misc_set_alignment(GTK_MISC(label), 0.95, 0.5);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-    main_info->assets_label = label;
-
-    hbox = gtk_hbox_new(FALSE, 2);
-    gtk_box_pack_end(GTK_BOX(statusbar), hbox, FALSE, FALSE, 5);
-
-    label = gtk_label_new(PROFITS_C_STR);
-    gtk_misc_set_alignment(GTK_MISC(label), 0.95, 0.5);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-    label = gtk_label_new("");
-    gtk_misc_set_alignment(GTK_MISC(label), 0.95, 0.5);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-    main_info->profits_label = label;
+    default_currency = gnc_lookup_string_option("International",
+						"Default Currency",
+						"USD");
+    combo_box = gtk_select_new();
+    main_info->totals_combo = combo_box;
+    main_info->totals_list = NULL;
+    def_item = gnc_ui_get_currency_item(&main_info->totals_list,
+					default_currency,
+				        main_info->totals_combo);
+    gtk_select_select_child(GTK_SELECT(combo_box), def_item->listitem);
+    gtk_box_pack_end(GTK_BOX(statusbar), combo_box, FALSE, FALSE, 5);
   }
 
   /* create scrolled window */
