@@ -1,15 +1,3 @@
-/*
- * FILE:
- * basiccell.c
- *
- * FUNCTION: 
- * Implements the base class for the cell handler object.
- * See the header file for additional documentation.
- *
- * HISTORY:
- * Copyright (c) 1998 Linas Vepstas
- */
-
 /********************************************************************\
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -22,10 +10,26 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
+/*
+ * FILE:
+ * basiccell.c
+ *
+ * FUNCTION: 
+ * Implements the base class for the cell handler object.
+ * See the header file for additional documentation.
+ *
+ * HISTORY:
+ * Copyright (c) 1998 Linas Vepstas
+ */
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -158,4 +162,15 @@ xaccBasicCellGetHelp (BasicCell *cell)
   return cell->get_help_value(cell);
 }
 
+/* ===================================================== */
+
+void
+xaccBasicCellSetChanged (BasicCell *cell, gncBoolean changed)
+{
+  if (cell == NULL)
+    return;
+
+  cell->changed = changed ? 0xffffffff : 0;
+}
+
 /* ================== end of file ====================== */
