@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -185,7 +189,10 @@
 #ifndef __XACC_BASIC_CELL_H__
 #define __XACC_BASIC_CELL_H__
 
+#include <glib.h>
+
 #include "gnc-common.h"
+#include "gnc-ui-common.h"
 
 /* define a bitmask */
 #define XACC_CELL_ALLOW_NONE       0x0
@@ -200,8 +207,8 @@ struct _BasicCell {
 
   /* cell attributes */
   /* hack alert -- may want to redesign color to use named color strings. */
-  uint32 bg_color;        /* background color, ARGB format */
-  uint32 fg_color;        /* forground (text) color ARGB format */
+  guint32 bg_color;       /* background color, ARGB format */
+  guint32 fg_color;       /* forground (text) color ARGB format */
   short use_fg_color;     /* if 0, above is ignored */
   short use_bg_color;     /* if 0, above is ignored */
 
@@ -264,5 +271,7 @@ void         xaccSetBasicCellValue (BasicCell *, const char *);
 void         xaccSetBasicCellBlankHelp (BasicCell *, const char *);
 char *       xaccBasicCellGetHelp (BasicCell *);
 
+void         xaccBasicCellSetChanged (BasicCell *, gncBoolean);
+
 #endif /* __XACC_BASIC_CELL_H__ */
 /* ------------------ end of file ---------------------- */
