@@ -1,14 +1,3 @@
-/*
- * FILE:
- * cellblock.c
- * 
- * FUNCTION:
- * implements a rectangular array of cells. See the header file for
- * additional documentation.
- *
- * HISTORY:
- * Copyright (c) 1998 Linas Vepstas
- */
 /********************************************************************\
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -21,10 +10,26 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
+/*
+ * FILE:
+ * cellblock.c
+ * 
+ * FUNCTION:
+ * implements a rectangular array of cells. See the header file for
+ * additional documentation.
+ *
+ * HISTORY:
+ * Copyright (c) 1998 Linas Vepstas
+ */
+
 #include <stdlib.h>
 #include "cellblock.h"
 
@@ -74,6 +79,7 @@ FreeCellBlockMem (CellBlock *arr)
          if (arr->cells[i]) free (arr->cells[i]);
       }
       free (arr->cells);
+      arr->cells = NULL;
    }
 
    /* free label array, if any */
@@ -81,18 +87,24 @@ FreeCellBlockMem (CellBlock *arr)
       for (i=0; i<oldrows; i++) {
          if (arr->cell_types[i]) free (arr->cell_types[i]);
       }
+      free (arr->cell_types);
+      arr->cell_types = NULL;
    }
-   
+
    /* free right traversal chain */
    if (arr->right_traverse_r) {
       for (i=0; i<oldrows; i++) {
          if (arr->right_traverse_r[i]) free (arr->right_traverse_r[i]);
       }
+      free (arr->right_traverse_r);
+      arr->right_traverse_r = NULL;
    }
    if (arr->right_traverse_c) {
       for (i=0; i<oldrows; i++) {
          if (arr->right_traverse_c[i]) free (arr->right_traverse_c[i]);
       }
+      free (arr->right_traverse_c);
+      arr->right_traverse_c = NULL;
    }
 
    /* free left traversal chain */
@@ -100,11 +112,15 @@ FreeCellBlockMem (CellBlock *arr)
       for (i=0; i<oldrows; i++) {
          if (arr->left_traverse_r[i]) free (arr->left_traverse_r[i]);
       }
+      free (arr->left_traverse_r);
+      arr->left_traverse_r = NULL;
    }
    if (arr->left_traverse_c) {
       for (i=0; i<oldrows; i++) {
          if (arr->left_traverse_c[i]) free (arr->left_traverse_c[i]);
       }
+      free (arr->left_traverse_c);
+      arr->left_traverse_c = NULL;
    }
 
    /* free widths, alignments */
