@@ -1,3 +1,23 @@
+/********************************************************************\
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
 /* 
  * FILE:
  * cellblock.h
@@ -34,21 +54,6 @@
  * HISTORY:
  * Copyright (c) 1988 Linas Vepstas
  */
-/********************************************************************\
- * This program is free software; you can redistribute it and/or    *
- * modify it under the terms of the GNU General Public License as   *
- * published by the Free Software Foundation; either version 2 of   *
- * the License, or (at your option) any later version.              *
- *                                                                  *
- * This program is distributed in the hope that it will be useful,  *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
- * GNU General Public License for more details.                     *
- *                                                                  *
- * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
-\********************************************************************/
 
 #ifndef __XACC_CELL_BLOCK_H__
 #define __XACC_CELL_BLOCK_H__
@@ -93,9 +98,9 @@ struct _CellBlock {
    * The passive_bg_color2 is the default color for cell backgrounds
    * in other rows of the cellblock when it is not highlighted.
    */
-  uint32 active_bg_color;
-  uint32 passive_bg_color;
-  uint32 passive_bg_color2;
+  guint32 active_bg_color;
+  guint32 passive_bg_color;
+  guint32 passive_bg_color2;
 
   /* other attributes */
   short         *widths;        /* column widths */
@@ -119,7 +124,7 @@ struct _CellBlock {
    *  (exit_r, exit_c) is the last cell of this tab group.
    */
 
-  /* the last-reneter row and column should contain the very last
+  /* the last-reenter row and column should contain the very last
    * cell when the cursor was traversed out of.  They determine 
    * the first cell that will be entered (since the first follows 
    * the last).
