@@ -10,13 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
  *                                                                  *
- *   Author: Rob Clark                                              *
- * Internet: rclark@cs.hmc.edu                                      *
- *  Address: 609 8th Street                                         *
- *           Huntington Beach, CA 92648-4632                        *
 \********************************************************************/
 
 /*
@@ -66,6 +65,21 @@ xaccParseDate (struct tm *parsed, const char * datestr)
    parsed->tm_mday = iday;
    parsed->tm_mon = imonth-1;
    parsed->tm_year = iyear-1900;
+   parsed->tm_isdst = -1;
+
+  if (mktime (parsed) == -1)
+  {
+    time_t secs = time (NULL);
+
+    *parsed = *localtime (&secs);
+
+    parsed->tm_sec = 0;
+    parsed->tm_min = 0;
+    parsed->tm_hour = 0;
+    parsed->tm_isdst = -1;
+  }
+
+  mktime (parsed);
 }
 
 /* ================================================ */
@@ -363,7 +377,7 @@ xaccCommitDateCell (DateCell *cell)
    char buff[30];
 
    if (!cell) return;
-   ENTER ("xaccCommitDateCell(): value is %s \n", cell->cell.value);
+   ENTER ("value is %s \n", cell->cell.value);
    xaccParseDate (&(cell->date), cell->cell.value);
    printDate (buff, cell->date.tm_mday, 
                     cell->date.tm_mon+1, 
@@ -371,7 +385,20 @@ xaccCommitDateCell (DateCell *cell)
 
    if (cell->cell.value) free (cell->cell.value);
    cell->cell.value = strdup (buff);
-   LEAVE ("xaccCommitDateCell(): value is %s \n", cell->cell.value);
+   LEAVE ("value is %s \n", cell->cell.value);
+}
+
+/* ================================================ */
+
+void
+xaccDateCellGetDate (DateCell *cell, Timespec *ts)
+{
+  if (!cell || !ts) return;
+
+  xaccParseDate (&(cell->date), cell->cell.value);
+
+  ts->tv_sec = mktime(&cell->date);
+  ts->tv_nsec = 0;
 }
 
 /* ================================================ */
