@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -26,12 +30,6 @@
  * Copyright (c) 2000 Linas Vepstas <linas@linas.org>
  */
 
-/*
-   TODO: We have no use for the generic ComboCell->menuitems.  These
-   should probably be killed.  Each GUI should probably handle it's
-   own strings.
-*/
-
 #include "config.h"
 
 #include <gnome.h>
@@ -56,10 +54,6 @@ typedef struct _PopBox
 	ItemEdit     *item_edit;
 	GNCItemList  *item_list;
 
-	gint     select_item_signal;
-        gint     change_item_signal;
-	gint     key_press_signal;
-
 	gboolean signals_connected; /* list signals connected? */
 
 	gboolean list_in_sync; /* list in sync with menustrings? */
@@ -166,6 +160,16 @@ change_item_cb (GNCItemList *item_list, char *item_string, gpointer data)
         box->in_list_select = FALSE;
 }
 
+static void
+activate_item_cb (GNCItemList *item_list, char *item_string, gpointer data)
+{
+	ComboCell *cell = (ComboCell *) data;
+	PopBox *box = (PopBox *) cell->cell.gui_private;
+
+        item_edit_hide_list(box->item_edit);
+        box->list_popped = FALSE;
+}
+
 static void
 key_press_item_cb (GNCItemList *item_list, GdkEventKey *event, gpointer data)
 {
@@ -195,14 +199,7 @@ combo_disconnect_signals (ComboCell *cell)
         if (GTK_OBJECT_DESTROYED(GTK_OBJECT(box->item_list)))
                 return;
 
-	gtk_signal_disconnect(GTK_OBJECT(box->item_list),
-			      box->select_item_signal);
-
-	gtk_signal_disconnect(GTK_OBJECT(box->item_list),
-			      box->change_item_signal);
-
-	gtk_signal_disconnect(GTK_OBJECT(box->item_list),
-			      box->key_press_signal);
+	gtk_signal_disconnect_by_data (GTK_OBJECT(box->item_list), cell);
 
 	box->signals_connected = FALSE;
 }
@@ -218,21 +215,17 @@ combo_connect_signals (ComboCell *cell)
         if (GTK_OBJECT_DESTROYED(GTK_OBJECT(box->item_list)))
                 return;
 
-	box->select_item_signal =
-		gtk_signal_connect(GTK_OBJECT(box->item_list), "select_item",
-				   GTK_SIGNAL_FUNC(select_item_cb),
-				   (gpointer) cell);
+        gtk_signal_connect(GTK_OBJECT(box->item_list), "select_item",
+                           GTK_SIGNAL_FUNC(select_item_cb), cell);
 
-	box->change_item_signal =
-		gtk_signal_connect(GTK_OBJECT(box->item_list), "change_item",
-				   GTK_SIGNAL_FUNC(change_item_cb),
-				   (gpointer) cell);
+        gtk_signal_connect(GTK_OBJECT(box->item_list), "change_item",
+                           GTK_SIGNAL_FUNC(change_item_cb), cell);
 
-	box->key_press_signal =
-		gtk_signal_connect(GTK_OBJECT(box->item_list),
-				   "key_press_event",
-				   GTK_SIGNAL_FUNC(key_press_item_cb),
-				   (gpointer) cell);
+        gtk_signal_connect(GTK_OBJECT(box->item_list), "activate_item",
+                           GTK_SIGNAL_FUNC(activate_item_cb), cell);
+
+        gtk_signal_connect(GTK_OBJECT(box->item_list), "key_press_event",
+                           GTK_SIGNAL_FUNC(key_press_item_cb), cell);
 
 	box->signals_connected = TRUE;
 }
@@ -245,14 +238,7 @@ block_list_signals (ComboCell *cell)
 	if (!box->signals_connected)
 		return;
 
-        gtk_signal_handler_block(GTK_OBJECT(box->item_list),
-                                 box->select_item_signal);
-
-        gtk_signal_handler_block(GTK_OBJECT(box->item_list),
-                                 box->change_item_signal);
-
-        gtk_signal_handler_block(GTK_OBJECT(box->item_list),
-                                 box->key_press_signal);
+        gtk_signal_handler_block_by_data (GTK_OBJECT(box->item_list), cell);
 }
 
 static void
@@ -263,14 +249,7 @@ unblock_list_signals (ComboCell *cell)
 	if (!box->signals_connected)
 		return;
 
-        gtk_signal_handler_unblock(GTK_OBJECT(box->item_list),
-                                   box->select_item_signal);
-
-        gtk_signal_handler_unblock(GTK_OBJECT(box->item_list),
-                                   box->change_item_signal);
-
-        gtk_signal_handler_unblock(GTK_OBJECT(box->item_list),
-                                   box->key_press_signal);
+        gtk_signal_handler_unblock_by_data (GTK_OBJECT(box->item_list), cell);
 }
 
 /* =============================================== */
@@ -302,6 +281,14 @@ destroyCombo (BasicCell *bcell)
 
 /* =============================================== */
 
+static void
+menustring_free(gpointer string, gpointer user_data)
+{
+        g_free(string);
+}
+
+/* =============================================== */
+
 void xaccDestroyComboCell (ComboCell *cell)
 {
 	PopBox *box = (PopBox *) cell->cell.gui_private;
@@ -309,8 +296,9 @@ void xaccDestroyComboCell (ComboCell *cell)
 	destroyCombo(&(cell->cell));
 
 	if (box != NULL) {
-		g_list_foreach(box->menustrings, (GFunc) g_free, NULL);
+		g_list_foreach(box->menustrings, menustring_free, NULL);
 		g_list_free(box->menustrings);
+                box->menustrings = NULL;
 
                 xaccFreeQuickFill(box->qf);
                 box->qf = NULL;
@@ -324,7 +312,6 @@ void xaccDestroyComboCell (ComboCell *cell)
 
 	cell->cell.gui_private = NULL;
 	cell->cell.realize = NULL;
-	cell->cell.set_value = NULL;
 
 	xaccDestroyBasicCell(&(cell->cell));
 }
@@ -345,7 +332,7 @@ xaccClearComboCellMenu (ComboCell * cell)
         if (box->menustrings == NULL)
                 return;
 
-        g_list_foreach(box->menustrings, (GFunc) g_free, NULL);
+        g_list_foreach(box->menustrings, menustring_free, NULL);
         g_list_free(box->menustrings);
         box->menustrings = NULL;
 
@@ -416,7 +403,7 @@ xaccAddComboCellMenuItem (ComboCell *cell, char * menustr)
                 block_list_signals(cell);
 
 		gnc_item_list_append(box->item_list, menustr);
-                if (strcmp(menustr, cell->cell.value) == 0)
+                if (safe_strcmp(menustr, cell->cell.value) == 0)
                         gnc_item_list_select(box->item_list, menustr);
 
                 unblock_list_signals(cell);
