@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -49,8 +53,8 @@ gnucash_header_update (GnomeCanvasItem *item, double *affine,
 
         item->x1 = 0;
         item->y1 = 0;
-        item->x2 = INT_MAX;
-        item->y2 = INT_MAX;
+        item->x2 = INT_MAX/2 -1;
+        item->y2 = INT_MAX/2 -1;
 }
 
 
@@ -161,7 +165,7 @@ gnucash_header_request_redraw (GnucashHeader *header)
 {
         GnomeCanvas *canvas = GNOME_CANVAS_ITEM(header)->canvas;
 
-        gnome_canvas_request_redraw (canvas, 0, 0, INT_MAX, INT_MAX);
+        gnome_canvas_request_redraw (canvas, 0, 0, INT_MAX/2 -1, INT_MAX/2 -1);
 }
 
 
@@ -223,6 +227,7 @@ gnucash_header_reconfigure (GnucashHeader *header)
         GnomeCanvas *canvas;
         GtkWidget *widget;
         GnucashSheet *sheet;
+        SheetBlockStyle *old_style;
 
         g_return_if_fail (header != NULL);
         g_return_if_fail (GNUCASH_IS_HEADER (header));
@@ -230,6 +235,7 @@ gnucash_header_reconfigure (GnucashHeader *header)
         canvas = GNOME_CANVAS_ITEM(header)->canvas;
         widget = GTK_WIDGET (header->sheet);
         sheet = GNUCASH_SHEET(header->sheet);
+        old_style = header->style;
 
         header->style = header->sheet->cursor_style[header->type];
 
@@ -246,7 +252,8 @@ gnucash_header_reconfigure (GnucashHeader *header)
         w = MAX (widget->allocation.width, header->style->dimensions->width);
         h = header->style->dimensions->height;
 
-        if (header->height != h || header->width != w) {
+        if (header->height != h || header->width != w ||
+            header->style != old_style) {
                 header->height = h;
                 header->width = w;
 
