@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -1032,7 +1036,9 @@ item_edit_show_list (ItemEdit *item_edit)
         gint y_offset;
         gint list_x, list_y;
         gint list_height;
+        gint list_width;
         gint view_height;
+        gint view_width;
 
         g_return_if_fail(item_edit != NULL);
 	g_return_if_fail(IS_ITEM_EDIT(item_edit));
@@ -1042,6 +1048,7 @@ item_edit_show_list (ItemEdit *item_edit)
 
         sheet = item_edit->sheet;
         view_height = GTK_WIDGET(sheet)->allocation.height;
+        view_width  = GTK_WIDGET(sheet)->allocation.width;
         gnome_canvas_get_scroll_offsets(GNOME_CANVAS(sheet), NULL, &y_offset);
         item_edit_get_pixel_coords (item_edit, &x, &y, &w, &h);
 
@@ -1080,6 +1087,23 @@ item_edit_show_list (ItemEdit *item_edit)
                       GTK_ARROW_UP, GTK_SHADOW_OUT);
 
         gtk_widget_grab_focus(GTK_WIDGET(item_edit->item_list->clist));
+
+        /* Make sure the list gets shown/sized correctly */
+        while (gtk_events_pending())
+                gtk_main_iteration();
+
+        gnc_item_list_show_selected(item_edit->item_list);
+
+        list_width = GTK_WIDGET(item_edit->item_list->clist)->allocation.width;
+        if (list_width > (view_width - list_x))
+        {
+                list_x -= (list_width - (view_width - list_x));
+                list_x  = MAX (0, list_x);
+
+                gnome_canvas_item_set(GNOME_CANVAS_ITEM(item_edit->item_list),
+                                      "x", (gdouble) list_x,
+                                      NULL);
+        }
 }
 
 
