@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -275,8 +279,11 @@ gnucash_sheet_activate_cursor_cell (GnucashSheet *sheet,
                                            &cursor_pos, &start_sel, &end_sel);
 
 	if (new_text != NULL)
+        {
 		gnucash_sheet_cell_set_from_table (sheet, virt_row, virt_col,
 						   cell_row, cell_col);
+                gnucash_sheet_redraw_block (sheet, virt_row, virt_col);
+        }
 	else
         {
 		gnucash_sheet_start_editing_at_cursor (sheet);
@@ -677,7 +684,7 @@ gnucash_sheet_redraw_all (GnucashSheet *sheet)
         g_return_if_fail (GNUCASH_IS_SHEET(sheet));
 
         gnome_canvas_request_redraw (
-                GNOME_CANVAS (sheet), 0, 0, INT_MAX, INT_MAX);
+                GNOME_CANVAS (sheet), 0, 0, INT_MAX/2 -1, INT_MAX/2 -1);
 }
 
 
@@ -708,7 +715,8 @@ gnucash_sheet_redraw_block (GnucashSheet *sheet, gint row, gint col)
                 y += canvas->layout.yoffset - canvas->zoom_yofs;
                 
                 h = style->dimensions->height;
-                w = MIN(style->dimensions->width, GTK_WIDGET(sheet)->allocation.width);
+                w = MIN(style->dimensions->width,
+                        GTK_WIDGET(sheet)->allocation.width);
 
                 gnome_canvas_request_redraw (canvas, x, y, x+w, y+h);
         }
@@ -727,6 +735,8 @@ gnucash_sheet_destroy (GtkObject *object)
                 for (j = 0; j < sheet->num_virt_cols; j++)
                         gnucash_sheet_block_destroy(sheet, i, j);
 
+        g_hash_table_destroy (sheet->blocks);
+
         for (i = GNUCASH_CURSOR_HEADER; i < GNUCASH_CURSOR_LAST; i++)
                 gnucash_sheet_style_destroy(sheet, sheet->cursor_style[i]);
 
