@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -29,19 +33,22 @@
 #include <string.h>
 #include <locale.h>
 
+#include "gnc-common.h"
 #include "util.h"
 
 #include "basiccell.h"
 #include "pricecell.h"
 
+/* GUI-dependent */
+extern void xaccPriceGUIInit (PriceCell *);
 
 static void PriceSetValue (BasicCell *, const char *);
 static char * xaccPriceCellPrintValue (PriceCell *cell);
 
 /* set the color of the text to red, if the value is negative */
 /* hack alert -- the actual color should probably be configurable */
-#define COLORIZE(cell,amt) {			\
-   if (0.0 > amt) {				\
+#define COLORIZE(cell,amount) {			\
+   if ((0.0 > amount) && !DEQ(amount, 0.0)) {	\
       /* red */					\
       cell->cell.fg_color = 0xff0000;		\
    } else {					\
@@ -111,7 +118,9 @@ PriceMV (BasicCell *_cell,
         /* accept only numbers or a decimal point or a thousands sep */
         if (!isdigit(change[i]) &&
             (decimal_point != change[i]) &&
-            (thousands_sep != change[i]))
+            (thousands_sep != change[i]) &&
+            ('('           != change[i]) &&
+            (')'           != change[i]))
           return NULL;
 
         if (decimal_point == change[i])
@@ -193,16 +202,20 @@ xaccInitPriceCell (PriceCell *cell)
    cell->amount = 0.0;
    cell->blank_zero = GNC_T;
    cell->monetary = GNC_T;
+   cell->is_currency = GNC_F;
    cell->shares_value = GNC_F;
 
    SET (&(cell->cell), "");
+   COLORIZE (cell, 0.0);
 
-   cell->cell.use_fg_color = 1;
+   cell->cell.use_fg_color = GNC_T;
    cell->cell.enter_cell = PriceEnter;
    cell->cell.modify_verify = PriceMV;
    cell->cell.leave_cell = PriceLeave;
    cell->cell.set_value = PriceSetValue;
    cell->cell.get_help_value = PriceHelp;
+
+   xaccPriceGUIInit( cell);
 }
 
 /* ================================================ */
@@ -210,8 +223,8 @@ xaccInitPriceCell (PriceCell *cell)
 void
 xaccDestroyPriceCell (PriceCell *cell)
 {
-   cell->amount = 0.0;
-   xaccDestroyBasicCell (&(cell->cell));
+  cell->amount = 0.0;
+  xaccDestroyBasicCell (&(cell->cell));
 }
 
 /* ================================================ */
@@ -220,7 +233,7 @@ static char *
 xaccPriceCellPrintValue (PriceCell *cell)
 {
   static char buff[PRTBUF];
-  short flags = PRTSEP;
+  GNCPrintAmountFlags flags = PRTSEP;
 
   if (cell->blank_zero && DEQ(cell->amount, 0.0)) {
      strcpy(buff, "");
@@ -230,31 +243,53 @@ xaccPriceCellPrintValue (PriceCell *cell)
   if (cell->shares_value)
     flags |= PRTSHR;
 
-  xaccSPrintAmount(buff, cell->amount, flags);
+  if (cell->is_currency)
+    flags |= PRTCUR;
+
+  xaccSPrintAmount(buff, cell->amount, flags, NULL);
 
   return buff;
 }
 
 /* ================================================ */
+
 double
 xaccGetPriceCellValue (PriceCell *cell)
 {
-  assert(cell != NULL);
+  if (cell == NULL)
+    return 0.0;
 
   return cell->amount;
 }
 
-void xaccSetPriceCellValue (PriceCell * cell, double amt)
+void
+xaccSetPriceCellValue (PriceCell * cell, double amount)
 {
    char *buff;
 
-   cell->amount = amt;
+   if (cell == NULL)
+     return;
+
+   cell->amount = amount;
    buff = xaccPriceCellPrintValue (cell);
 
    SET ( &(cell->cell), buff);
 
    /* set the cell color to red if the value is negative */
-   COLORIZE (cell, amt);
+   COLORIZE (cell, amount);
+}
+
+void
+xaccSetPriceCellBlank (PriceCell *cell)
+{
+  if (cell == NULL)
+    return;
+
+  cell->amount = 0.0;
+
+  SET ( &(cell->cell), "");
+
+  COLORIZE (cell, 0.0);
 }
 
 /* ================================================ */
@@ -262,7 +297,8 @@ void xaccSetPriceCellValue (PriceCell * cell, double amt)
 void
 xaccSetPriceCellSharesValue (PriceCell * cell, gncBoolean shares_value)
 {
-  assert(cell != NULL);
+  if (cell == NULL)
+    return;
 
   cell->shares_value = shares_value;
 }
@@ -272,41 +308,56 @@ xaccSetPriceCellSharesValue (PriceCell * cell, gncBoolean shares_value)
 void
 xaccSetPriceCellMonetary (PriceCell * cell, gncBoolean monetary)
 {
-  assert(cell != NULL);
+  if (cell == NULL)
+    return;
 
   cell->monetary = monetary;
 }
 
 /* ================================================ */
 
+void
+xaccSetPriceCellIsCurrency (PriceCell *cell, gncBoolean is_currency)
+{
+  if (cell == NULL)
+    return;
+
+  cell->is_currency = is_currency;
+}
+
+/* ================================================ */
+
 void
 xaccSetPriceCellBlankZero (PriceCell *cell, gncBoolean blank_zero)
 {
-  assert(cell != NULL);
+  if (cell == NULL)
+    return;
 
   cell->blank_zero = blank_zero;
 }
 
 /* ================================================ */
 
-void xaccSetDebCredCellValue (PriceCell * deb, 
-                              PriceCell * cred, double amt)
+void xaccSetDebCredCellValue (PriceCell * debit,
+                              PriceCell * credit,
+                              double amount)
 {
-   deb->cell.fg_color = 0xff0000;
-   cred->cell.fg_color = 0x0;
+   debit->cell.fg_color = 0xff0000;
+   credit->cell.fg_color = 0x0;
 
-   if (0.0 < amt) {
-      xaccSetPriceCellValue (cred, amt);
-      xaccSetPriceCellValue (deb, 0.0);
-      if (!deb->blank_zero) {
-        SET(&deb->cell, "");
+   /* debits are positive, credits are negative */
+   if (amount > 0.0) {
+      xaccSetPriceCellValue (debit, amount);
+      xaccSetPriceCellValue (credit, 0.0);
+      if (!credit->blank_zero) {
+        SET(&credit->cell, "");
       }
    } else {
-      xaccSetPriceCellValue (cred, 0.0);
-      if (!cred->blank_zero) {
-        SET(&deb->cell, "");
+      xaccSetPriceCellValue (debit, 0.0);
+      if (!debit->blank_zero) {
+        SET(&debit->cell, "");
       }
-      xaccSetPriceCellValue (deb, -amt);
+      xaccSetPriceCellValue (credit, -amount);
    }
 }
 
@@ -316,9 +367,17 @@ static void
 PriceSetValue (BasicCell *_cell, const char *str)
 {
    PriceCell *cell = (PriceCell *) _cell;
-   double amt = xaccParseAmount (str, cell->monetary);
+   double amount;
 
-   xaccSetPriceCellValue (cell, amt);
+   if (str == NULL)
+     str = "";
+
+   if (!cell->blank_zero && (*str == '\0'))
+     xaccSetPriceCellBlank(cell);
+   else {
+     amount = xaccParseAmount (str, cell->monetary);
+     xaccSetPriceCellValue (cell, amount);
+   }
 }
 
 /* --------------- end of file ---------------------- */
