@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -56,13 +60,14 @@
 
 typedef struct _PriceCell
 {
-   BasicCell cell;
+  BasicCell cell;
 
-   double amount;           /* the amount associated with this cell */
+  double amount;           /* the amount associated with this cell */
 
-   gncBoolean blank_zero;   /* controls printing of zero values */
-   gncBoolean monetary;     /* controls parsing of values */
-   gncBoolean shares_value; /* true if a shares values */
+  gncBoolean blank_zero;   /* controls printing of zero values */
+  gncBoolean monetary;     /* controls parsing of values */
+  gncBoolean is_currency;  /* controls printint of values */
+  gncBoolean shares_value; /* true if a shares values */
 } PriceCell;
 
 /* installs a callback to handle price recording */
@@ -76,16 +81,22 @@ double       xaccGetPriceCellValue (PriceCell *cell);
 /* updates amount, string format is three decimal places */
 void         xaccSetPriceCellValue (PriceCell *cell, double amount);
 
+/* Sets the cell as blank, regardless of the blank_zero value */
+void         xaccSetPriceCellBlank (PriceCell *cell);
+
 /* determines whether 0 values are left blank or printed.
  * defaults to true. */
 void         xaccSetPriceCellBlankZero (PriceCell *cell, gncBoolean);
 
 /* The xaccSetPriceCellMonetary() sets a flag which determines
  *    how string amounts are parsed, either as monetary or
- *    non-monetary amounts. The default is monetary.
- */
+ *    non-monetary amounts. The default is monetary. */
 void         xaccSetPriceCellMonetary (PriceCell *, gncBoolean);
 
+/* The xaccSetPriceCellCurrency() sets a flag which causes
+ *    the amount to be printed as a currency price. */
+void         xaccSetPriceCellIsCurrency (PriceCell *, gncBoolean);
+
 /* The xaccSetPriceCellSharesValue() sets a flag which determines
  * whether the quantity is printed as a shares value or not. */
 void         xaccSetPriceCellSharesValue (PriceCell *, gncBoolean);
