@@ -1,3 +1,23 @@
+/********************************************************************\
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
 /*
  * FILE:
  * quickfillcell.h
@@ -20,21 +40,6 @@
  * HISTORY:
  * Copyright (c) 1997, 1998 Linas Vepstas
  */
-/********************************************************************\
- * This program is free software; you can redistribute it and/or    *
- * modify it under the terms of the GNU General Public License as   *
- * published by the Free Software Foundation; either version 2 of   *
- * the License, or (at your option) any later version.              *
- *                                                                  *
- * This program is distributed in the hope that it will be useful,  *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
- * GNU General Public License for more details.                     *
- *                                                                  *
- * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
-\********************************************************************/
 
 #ifndef __XACC_FILL_CELL_C__
 #define __XACC_FILL_CELL_C__
@@ -44,22 +49,24 @@
 
 typedef struct _QuickFillCell
 {
-   BasicCell cell;
-   QuickFill *qfRoot;       /* root of quickfill-tree 
-                             * handled by this cell */
-   QuickFill *qf;           /* current position in tree */
+  BasicCell cell;
+  QuickFill *qfRoot;       /* root of quickfill-tree 
+                            * handled by this cell */
+  QuickFill *qf;           /* current position in tree */
 
-   QuickFillSort sort;      /* determines order of strings matched.
-                             * default is QUICKFILL_LIFO. */
+  QuickFillSort sort;      /* determines order of strings matched.
+                            * default is QUICKFILL_LIFO. */
+
+  char *original;          /* original string entered in original case */
 } QuickFillCell;
 
-/* installs a callback to handle price recording */
 QuickFillCell *  xaccMallocQuickFillCell (void);
 void             xaccInitQuickFillCell (QuickFillCell *);
 void             xaccDestroyQuickFillCell (QuickFillCell *);
 
 void             xaccSetQuickFillCellValue (QuickFillCell *, const char *);
 void             xaccSetQuickFillSort (QuickFillCell *, QuickFillSort);
+void             xaccSetQuickFillOriginal (QuickFillCell *, const char *);
 
 /* GUI-dependent */
 void             xaccQuickFillGUIInit (QuickFillCell *);
