@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -30,19 +34,24 @@
  * Copyright (c) 1998, 1999, 2000 Linas Vepstas
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "messages.h"
 #include "recncell.h"
 #include "splitreg.h"
 #include "table-allgui.h"
 #include "textcell.h"
+#include "messages.h"
 #include "util.h"
 
 /* This static indicates the debugging module that this .o belongs to.  */
 static short module = MOD_REGISTER;
 
+static SRStringGetter debit_getter = NULL;
+static SRStringGetter credit_getter = NULL;
+
 typedef struct _CellBuffer CellBuffer;
 struct _CellBuffer
 {
@@ -92,30 +101,6 @@ static SplitRegisterColors reg_colors = {
 };
 
 
-/* utility defines for cell configuration data */
-#define DATE_CELL      0
-#define NUM_CELL       1
-#define DESC_CELL      2
-#define RECN_CELL      3   
-#define SHRS_CELL      4
-#define BALN_CELL      5
-#define ACTN_CELL      6
-#define XFRM_CELL      7
-#define XTO_CELL       8
-#define MEMO_CELL      9
-#define CRED_CELL      10
-#define DEBT_CELL      11
-#define PRIC_CELL      12
-#define VALU_CELL      13
-
-/* NCRED & NDEBT handle minus the usual quantities */
-#define NCRED_CELL     14
-#define NDEBT_CELL     15
-
-/* MXFRM is the "mirrored" transfer-from account */
-#define MXFRM_CELL     16
-
-
 #define DATE_CELL_WIDTH    11
 #define NUM_CELL_WIDTH      7
 #define ACTN_CELL_WIDTH     7
@@ -161,7 +146,21 @@ static SplitRegisterColors reg_colors = {
    hcell = reg->header_label_cells[NAME##_CELL];		\
    xaccSetBasicCellValue (hcell, label);			\
 }
-   
+
+/* ============================================== */
+
+void
+xaccSplitRegisterSetDebitStringGetter(SRStringGetter getter)
+{
+  debit_getter = getter;
+}
+
+void
+xaccSplitRegisterSetCreditStringGetter(SRStringGetter getter)
+{
+  credit_getter = getter;
+}
+
 /* ============================================== */
 /* configLabels merely puts strings into the label cells 
  * it does *not* copy them to the header cursor */
@@ -169,91 +168,66 @@ static SplitRegisterColors reg_colors = {
 static void
 configLabels (SplitRegister *reg)
 {
-   BasicCell *hc;
-   int type = (reg->type) & REG_TYPE_MASK;
+  BasicCell *hc;
+  int type = (reg->type) & REG_TYPE_MASK;
+  char *string;
 
-   LABEL (DATE,  DATE_STR);
-   LABEL (NUM,   NUM_STR);
-   LABEL (DESC,  DESC_STR);
-   LABEL (RECN,  "R");
-   LABEL (SHRS,  TOT_SHRS_STR);
-   LABEL (BALN,  BALN_STR);
-   LABEL (ACTN,  ACTION_STR);
-   LABEL (XFRM,  XFRM_STR);
-   LABEL (MXFRM, XFRM_STR);
-   LABEL (XTO,   XFTO_STR);
-   LABEL (MEMO,  MEMO_STR);
-   LABEL (CRED,  CREDIT_STR);
-   LABEL (DEBT,  DEBIT_STR);
-   LABEL (PRIC,  PRICE_STR);
-   LABEL (VALU,  VALUE_STR);
+  LABEL (DATE,  DATE_STR);
+  LABEL (NUM,   NUM_STR);
+  LABEL (DESC,  DESC_STR);
+  LABEL (RECN,  "R");
+  LABEL (SHRS,  TOTAL_SHARES_STR);
+  LABEL (BALN,  BALN_STR);
+  LABEL (ACTN,  ACTION_STR);
+  LABEL (XFRM,  ACCOUNT_STR);
+  LABEL (MXFRM, TRANSFER_STR);
+  LABEL (XTO,   ACCOUNT_STR);
+  LABEL (MEMO,  MEMO_STR);
+  LABEL (CRED,  CREDIT_STR);
+  LABEL (DEBT,  DEBIT_STR);
+  LABEL (PRIC,  PRICE_STR);
+  LABEL (VALU,  VALUE_STR);
 
+  if (debit_getter != NULL)
+  {
+    string = debit_getter(type);
+    if (string != NULL)
+    {
+      LABEL (DEBT, string);
+      free(string);
+    }
+  }
 
-   /* setup custom labels for the debit/credit columns */
-   switch (type) {
-      case BANK_REGISTER:
-         LABEL (DEBT,  PAYMENT_STR);
-         LABEL (CRED,  DEPOSIT_STR);
-         break;
-      case CASH_REGISTER:
-         LABEL (DEBT,  SPEND_STR);
-         LABEL (CRED,  RECEIVE_STR);
-         break;
-      case ASSET_REGISTER:
-         LABEL (DEBT,  DEPR_STR);
-         LABEL (CRED,  APPR_STR);
-         break;
-      case CREDIT_REGISTER:
-         LABEL (DEBT,  CHARGE_STR);
-         LABEL (CRED,  PAYMENT_STR);
-         break;
-      case LIABILITY_REGISTER:
-         LABEL (DEBT,  INCREASE_STR);
-         LABEL (CRED,  DECREASE_STR);
-         break;
-      case INCOME_LEDGER:  
-      case INCOME_REGISTER:
-         LABEL (DEBT,  INCOME_STR);
-         LABEL (CRED,  CHARGE_STR);
-         break;
-      case EXPENSE_REGISTER:
-         LABEL (DEBT,  REBATE_STR);
-         LABEL (CRED,  EXPENSE_STR);
-         break;
-      case GENERAL_LEDGER:  
-      case EQUITY_REGISTER:
-         LABEL (DEBT,  SURPLUS_STR);
-         LABEL (CRED,  DEFICIT_STR);
-         break;
-      case STOCK_REGISTER:
-      case PORTFOLIO:
-      case CURRENCY_REGISTER:
-         LABEL (DEBT,  SOLD_STR);
-         LABEL (CRED,  BOUGHT_STR);
-         break;
-      default:
-         break;
-   }
+  if (credit_getter != NULL)
+  {
+    string = credit_getter(type);
+    if (string != NULL)
+    {
+      LABEL (CRED, string);
+      free(string);
+    }
+  }
 
-   /* copy debit, dredit strings to ndebit, ncredit cells */
-   hc = reg->header_label_cells[DEBT_CELL];
-   LABEL (NDEBT,  hc->value);
-   hc = reg->header_label_cells[CRED_CELL];
-   LABEL (NCRED,  hc->value);
+  /* copy debit, dredit strings to ndebit, ncredit cells */
+  hc = reg->header_label_cells[DEBT_CELL];
+  LABEL (NDEBT,  hc->value);
+  hc = reg->header_label_cells[CRED_CELL];
+  LABEL (NCRED,  hc->value);
 }
 
 /* ============================================== */
 /* configAction strings into the action cell */
-/* hack alert -- this stuf really, really should be in a config file ... */
+/* hack alert -- this stuff really, really should be in a config file ... */
 
 static void
 configAction (SplitRegister *reg)
 {
    int type = (reg->type) & REG_TYPE_MASK;
 
-   /* setup custom labels for the debit/credit columns */
+   /* setup strings in the action pull-down */
    switch (type) {
       case BANK_REGISTER:
+      case SEARCH_LEDGER:  /* broken ! FIXME bg */
          xaccAddComboCellMenuItem ( reg->actionCell, DEPOSIT_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, WITHDRAW_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, CHECK_STR);
@@ -279,7 +253,12 @@ configAction (SplitRegister *reg)
          xaccAddComboCellMenuItem ( reg->actionCell, FEE_STR);
          break;
       case CREDIT_REGISTER:
+         xaccAddComboCellMenuItem ( reg->actionCell, ATM_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, CREDIT_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, FEE_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, INT_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, ONLINE_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
          break;
       case LIABILITY_REGISTER:
@@ -289,7 +268,7 @@ configAction (SplitRegister *reg)
          xaccAddComboCellMenuItem ( reg->actionCell, INT_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, PAYMENT_STR);
          break;
-      case INCOME_LEDGER:  
+      case INCOME_LEDGER:
       case INCOME_REGISTER:
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
@@ -301,14 +280,14 @@ configAction (SplitRegister *reg)
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
          break;
-      case GENERAL_LEDGER:  
+      case GENERAL_LEDGER:
       case EQUITY_REGISTER:
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, EQUITY_STR);
          break;
       case STOCK_REGISTER:
-      case PORTFOLIO:
+      case PORTFOLIO_LEDGER:
       case CURRENCY_REGISTER:
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
@@ -368,7 +347,6 @@ configLayout (SplitRegister *reg)
 {
    CellBlock *curs, *header;
    int type = (reg->type) & REG_TYPE_MASK;
-   // int style = (reg->type) & REG_STYLE_MASK;
    int i;
 
    /* define header for macros */
@@ -393,9 +371,6 @@ configLayout (SplitRegister *reg)
       case INCOME_REGISTER:
       case EXPENSE_REGISTER:
       case EQUITY_REGISTER:
-
-      case INCOME_LEDGER:     /* hack alert do xto cell too */
-      case GENERAL_LEDGER:    /* hack alert do xto cell too */
       {
          curs = reg->double_cursor;
          FANCY (DATE,   date,     0,  0);
@@ -414,6 +389,7 @@ configLayout (SplitRegister *reg)
          FANCY (DATE,   date,     0,  0);
          FANCY (NUM,    num,      1,  0);
          FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    xto,      3,  0);
          BASIC (RECN,   recn,     4,  0);
          FANCY (DEBT,   debit,    5,  0);
          FANCY (CRED,   credit,   6,  0);
@@ -426,7 +402,6 @@ configLayout (SplitRegister *reg)
          FANCY (NDEBT,  ndebit,   5,  0);
          FANCY (NCRED,  ncredit,  6,  0);
 
-         /* basic common 8 column config */
          curs = reg->single_cursor;
          FANCY (DATE,   date,     0,  0);
          FANCY (NUM,    num,      1,  0);
@@ -440,12 +415,58 @@ configLayout (SplitRegister *reg)
          break;
       }
 
+      /* --------------------------------------------------------- */
+      case INCOME_LEDGER:
+      case GENERAL_LEDGER:
+      case SEARCH_LEDGER:
+      {
+         curs = reg->double_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    xto,      3,  0);
+         FANCY (MXFRM,  mxfrm,    4,  0);
+         BASIC (RECN,   recn,     5,  0);
+         FANCY (DEBT,   debit,    6,  0);
+         FANCY (CRED,   credit,   7,  0);
+
+         FANCY (ACTN,   action,   1,  1);
+         FANCY (MEMO,   memo,     2,  1);
+
+         curs = reg->trans_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    mxfrm,    3,  0);
+         FANCY (XFRM,   xto,      4,  0);
+         BASIC (RECN,   recn,     5,  0);
+         FANCY (DEBT,   debit,    6,  0);
+         FANCY (CRED,   credit,   7,  0);
+
+         curs = reg->split_cursor;
+         FANCY (ACTN,   action,   1,  0);
+         FANCY (MEMO,   memo,     2,  0);
+         FANCY (XFRM,   xfrm,     4,  0);
+         FANCY (NDEBT,  ndebit,   6,  0);
+         FANCY (NCRED,  ncredit,  7,  0);
+
+         curs = reg->single_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    xto,      3,  0);
+         FANCY (MXFRM,  mxfrm,    4,  0);
+         BASIC (RECN,   recn,     5,  0);
+         FANCY (DEBT,   debit,    6,  0);
+         FANCY (CRED,   credit,   7,  0);
+
+         break;
+      }
+
       /* --------------------------------------------------------- */
       case STOCK_REGISTER:
-      case PORTFOLIO:
       case CURRENCY_REGISTER:
       {
-         /* prep the second row of the double style */
          curs = reg->double_cursor;
          FANCY (DATE,   date,     0,  0);
          FANCY (NUM,    num,      1,  0);
@@ -457,16 +478,16 @@ configLayout (SplitRegister *reg)
          FANCY (PRIC,   price,    7,  0);
          FANCY (VALU,   value,    8,  0);
          FANCY (SHRS,   shrs,     9,  0);
-         FANCY (BALN,   balance,  10, 0);
+         FANCY (BALN,   balance, 10,  0);
 
          FANCY (ACTN,   action,   1,  1);
          FANCY (MEMO,   memo,     2,  1);
 
-         /* only the transaction cursor gets used */
          curs = reg->trans_cursor;
          FANCY (DATE,   date,     0,  0);
          FANCY (NUM,    num,      1,  0);
          FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    xto,      3,  0);
          BASIC (RECN,   recn,     4,  0);
          FANCY (DEBT,   debit,    5,  0);
          FANCY (CRED,   credit,   6,  0);
@@ -474,7 +495,7 @@ configLayout (SplitRegister *reg)
          FANCY (VALU,   value,    8,  0);
          FANCY (SHRS,   shrs,     9,  0);
          FANCY (BALN,   balance,  10, 0);
-    
+
          curs = reg->split_cursor;
          FANCY (ACTN,   action,   1,  0);
          FANCY (MEMO,   memo,     2,  0);
@@ -482,7 +503,6 @@ configLayout (SplitRegister *reg)
          FANCY (NDEBT,  ndebit,   5,  0);
          FANCY (NCRED,  ncredit,  6,  0);
 
-         /* 11 column config */
          curs = reg->single_cursor;
          FANCY (DATE,   date,     0,  0);
          FANCY (NUM,    num,      1,  0);
@@ -494,25 +514,75 @@ configLayout (SplitRegister *reg)
          FANCY (PRIC,   price,    7,  0);
          FANCY (VALU,   value,    8,  0);
          FANCY (SHRS,   shrs,     9,  0);
-         FANCY (BALN,   balance,  10, 0);
+         FANCY (BALN,   balance, 10,  0);
 
          break;
       }
+
+      /* --------------------------------------------------------- */
+      case PORTFOLIO_LEDGER:
+      {
+         curs = reg->double_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    xto,      3,  0);
+         FANCY (MXFRM,  mxfrm,    4,  0);
+         BASIC (RECN,   recn,     5,  0);
+         FANCY (DEBT,   debit,    6,  0);
+         FANCY (CRED,   credit,   7,  0);
+         FANCY (PRIC,   price,    8,  0);
+         FANCY (VALU,   value,    9,  0);
+         FANCY (SHRS,   shrs,    10,  0);
+
+         FANCY (ACTN,   action,   1,  1);
+         FANCY (MEMO,   memo,     2,  1);
+
+         curs = reg->trans_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    mxfrm,    3,  0);
+         FANCY (XFRM,   xto,      4,  0);
+         BASIC (RECN,   recn,     5,  0);
+         FANCY (DEBT,   debit,    6,  0);
+         FANCY (CRED,   credit,   7,  0);
+         FANCY (PRIC,   price,    8,  0);
+         FANCY (VALU,   value,    9,  0);
+         FANCY (SHRS,   shrs,    10,  0);
+
+         curs = reg->split_cursor;
+         FANCY (ACTN,   action,   1,  0);
+         FANCY (MEMO,   memo,     2,  0);
+         FANCY (XFRM,   xfrm,     4,  0);
+         FANCY (NDEBT,  ndebit,   6,  0);
+         FANCY (NCRED,  ncredit,  7,  0);
+
+         curs = reg->single_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (XTO,    xto,      3,  0);
+         FANCY (MXFRM,  mxfrm,    4,  0);
+         BASIC (RECN,   recn,     5,  0);
+         FANCY (DEBT,   debit,    6,  0);
+         FANCY (CRED,   credit,   7,  0);
+         FANCY (PRIC,   price,    8,  0);
+         FANCY (VALU,   value,    9,  0);
+         FANCY (SHRS,   shrs,    10,  0);
+
+         break;
+      }
+
       /* --------------------------------------------------------- */
       default:
-         PERR ("configLayout(): "
-           "unknown register type %d \n", type);
+         PERR ("unknown register type %d \n", type);
          break;
    }
 }
 
 /* ============================================== */
-/* define the traversal order */
-/* negative cells mean "traverse out of table" */
-/* hack alert -- redesign so that we hop from one row to the next, if desired. */
-/* hack alert -- if show_tamount or show_samount is set then don't traverse there */
-/* hack alert -- fix show_txfrm also ... */
-
+/* define the traversal order -- negative cells mean "traverse out of table" */
 
 /* Right Traversals */
 
@@ -840,43 +910,49 @@ configCursors (SplitRegister *reg)
 static void
 mallocCursors (SplitRegister *reg)
 {
-   int type = (reg->type) & REG_TYPE_MASK;
+  int type = (reg->type) & REG_TYPE_MASK;
 
-   switch (type) {
-      case BANK_REGISTER:
-      case CASH_REGISTER:
-      case ASSET_REGISTER:
-      case CREDIT_REGISTER:
-      case LIABILITY_REGISTER:
-      case INCOME_REGISTER:
-      case EXPENSE_REGISTER:
-      case EQUITY_REGISTER:
+  switch (type) {
+    case BANK_REGISTER:
+    case CASH_REGISTER:
+    case ASSET_REGISTER:
+    case CREDIT_REGISTER:
+    case LIABILITY_REGISTER:
+    case INCOME_REGISTER:
+    case EXPENSE_REGISTER:
+    case EQUITY_REGISTER:
+      reg->num_cols = 8;
+      break;
 
-      case INCOME_LEDGER:     /* hack alert do xto cell too */
-      case GENERAL_LEDGER:    /* hack alert do xto cell too */
-         reg->num_cols = 8;
-         break;
+    case INCOME_LEDGER:
+    case GENERAL_LEDGER:
+    case SEARCH_LEDGER:
+      reg->num_cols = 8;
+      break;
 
-      case STOCK_REGISTER:
-      case PORTFOLIO:
-      case CURRENCY_REGISTER:
-         reg->num_cols = 11;
-         break;
-      default:
-         break;
-   }
+    case STOCK_REGISTER:
+    case CURRENCY_REGISTER:
+      reg->num_cols = 11;
+      break;
 
-   reg->num_header_rows = 1;
-   reg->header = xaccMallocCellBlock (reg->num_header_rows, reg->num_cols);
+    case PORTFOLIO_LEDGER:
+      reg->num_cols = 11;
+      break;
 
-   /* cursors used in the single & double line displays */
-   reg->single_cursor = xaccMallocCellBlock (1, reg->num_cols);
-   reg->double_cursor = xaccMallocCellBlock (2, reg->num_cols);
+    default:
+      break;
+  }
 
-   /* the two cursors used for multi-line and dynamic displays */
-   reg->trans_cursor = xaccMallocCellBlock (1, reg->num_cols);
-   reg->split_cursor = xaccMallocCellBlock (1, reg->num_cols);
+  reg->num_header_rows = 1;
+  reg->header = xaccMallocCellBlock (reg->num_header_rows, reg->num_cols);
 
+  /* cursors used in the single & double line displays */
+  reg->single_cursor = xaccMallocCellBlock (1, reg->num_cols);
+  reg->double_cursor = xaccMallocCellBlock (2, reg->num_cols);
+
+  /* the two cursors used for multi-line and dynamic displays */
+  reg->trans_cursor = xaccMallocCellBlock (1, reg->num_cols);
+  reg->split_cursor = xaccMallocCellBlock (1, reg->num_cols);
 }
 
 /* ============================================== */
@@ -985,8 +1061,13 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    /* the xfer cells */
    xaccSetBasicCellBlankHelp (&reg->mxfrmCell->cell, XFER_CELL_HELP);
    xaccSetBasicCellBlankHelp (&reg->xfrmCell->cell, XFER_CELL_HELP);
+   xaccSetBasicCellBlankHelp (&reg->xtoCell->cell, XFER_TO_CELL_HELP);
+
    xaccComboCellSetIgnoreString (reg->mxfrmCell, SPLIT_STR);
+   xaccComboCellSetIgnoreString (reg->xtoCell, SPLIT_STR);
+
    xaccComboCellSetIgnoreHelp (reg->mxfrmCell, TOOLTIP_MULTI_SPLIT);
+   xaccComboCellSetIgnoreHelp (reg->xtoCell, TOOLTIP_MULTI_SPLIT);
 
    /* the memo cell */
    xaccSetBasicCellBlankHelp (&reg->memoCell->cell, MEMO_CELL_HELP);
@@ -994,14 +1075,18 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    /* the desc cell */
    xaccSetBasicCellBlankHelp (&reg->descCell->cell, DESC_CELL_HELP);
 
-   /* balance cell does not accept input; its display only.  */
-   /* however, we *do* want it to shadow the true cell contents when 
+   /* The balance cell does not accept input; it's for display only.
+    * however, we *do* want it to shadow the true cell contents when 
     * the cursor is repositioned.  Othewise, it will just display 
     * whatever previous bogus value it contained.
     */
    reg->balanceCell->cell.input_output = XACC_CELL_ALLOW_SHADOW;
    reg->shrsCell->cell.input_output = XACC_CELL_ALLOW_SHADOW;
 
+   /* by default, don't blank zeros on the balance or price cells. */
+   xaccSetPriceCellBlankZero(reg->balanceCell, GNC_F);
+   xaccSetPriceCellBlankZero(reg->priceCell, GNC_F);
+
    /* The reconcile cell should only be entered with the pointer,
     * and only then when the user clicks directly on the cell.
     */
@@ -1023,13 +1108,16 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
 
    /* number format for share quantities in stock ledgers */
    switch (type & REG_TYPE_MASK) {
-      case STOCK_REGISTER:
-      case PORTFOLIO:
       case CURRENCY_REGISTER:
+        xaccSetPriceCellIsCurrency (reg->priceCell, GNC_T);
+
+      case STOCK_REGISTER:
+      case PORTFOLIO_LEDGER:
          xaccSetPriceCellSharesValue (reg->debitCell, GNC_T);
          xaccSetPriceCellSharesValue (reg->creditCell, GNC_T);
          xaccSetPriceCellSharesValue (reg->ndebitCell, GNC_T);
          xaccSetPriceCellSharesValue (reg->ncreditCell, GNC_T);
+         xaccSetPriceCellIsCurrency (reg->priceCell, GNC_T);
 
          xaccSetBasicCellBlankHelp (&reg->priceCell->cell, PRICE_CELL_HELP);
          xaccSetBasicCellBlankHelp (&reg->valueCell->cell, VALUE_CELL_HELP);
@@ -1050,7 +1138,6 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
 
    /* -------------------------------- */   
    phys_r = header->numRows;
-   reg->cursor_phys_col = 0;
    reg->cursor_phys_row = phys_r;  /* cursor on first line past header */
    reg->cursor_virt_row = 1;
 
@@ -1093,7 +1180,7 @@ xaccConfigSplitRegister (SplitRegister *reg, int newtype)
    reg->type = newtype;
 
    /* Make sure that any GUI elements associated with this reconfig 
-    * are properly initialized.  */
+    * are properly initialized. */
    xaccCreateCursor (reg->table, reg->single_cursor);
    xaccCreateCursor (reg->table, reg->double_cursor);
    xaccCreateCursor (reg->table, reg->trans_cursor);
@@ -1228,19 +1315,9 @@ xaccSplitRegisterClearChangeFlag (SplitRegister *reg)
 
 /* ============================================== */
 
-CursorType
-xaccSplitRegisterGetCursorType (SplitRegister *reg)
+static CursorType
+sr_cellblock_cursor_type(SplitRegister *reg, CellBlock *cursor)
 {
-  Table *table;
-  CellBlock *cursor;
-
-  assert(reg);
-
-  table = reg->table;
-  if (table == NULL)
-    return CURSOR_NONE;
-
-  cursor = table->current_cursor;
   if (cursor == NULL)
     return CURSOR_NONE;
 
@@ -1257,6 +1334,237 @@ xaccSplitRegisterGetCursorType (SplitRegister *reg)
 
 /* ============================================== */
 
+CursorType
+xaccSplitRegisterGetCursorType (SplitRegister *reg)
+{
+  Table *table;
+
+  if (reg == NULL)
+    return CURSOR_NONE;
+
+  table = reg->table;
+  if (table == NULL)
+    return CURSOR_NONE;
+
+  return sr_cellblock_cursor_type(reg, table->current_cursor);
+}
+
+/* ============================================== */
+
+CursorType
+xaccSplitRegisterGetCursorTypeRowCol (SplitRegister *reg,
+                                      int virt_row, int virt_col)
+{
+  Table *table;
+
+  if (reg == NULL)
+    return CURSOR_NONE;
+
+  table = reg->table;
+  if (table == NULL)
+    return CURSOR_NONE;
+
+  if ((virt_row < 0) || (virt_row >= table->num_virt_rows) ||
+      (virt_col < 0) || (virt_col >= table->num_virt_cols))
+    return CURSOR_NONE;
+
+  return sr_cellblock_cursor_type(reg, table->handlers[virt_row][virt_col]);
+}
+
+/* ============================================== */
+
+CellType
+sr_cell_type (SplitRegister *reg, void * cell)
+{
+  if (cell == reg->dateCell)
+    return DATE_CELL;
+
+  if (cell == reg->numCell)
+    return NUM_CELL;
+
+  if (cell == reg->descCell)
+    return DESC_CELL;
+
+  if (cell == reg->recnCell)
+    return RECN_CELL;
+
+  if (cell == reg->shrsCell)
+    return SHRS_CELL;
+
+  if (cell == reg->balanceCell)
+    return BALN_CELL;
+
+  if (cell == reg->actionCell)
+    return ACTN_CELL;
+
+  if (cell == reg->xfrmCell)
+    return XFRM_CELL;
+
+  if (cell == reg->mxfrmCell)
+    return MXFRM_CELL;
+
+  if (cell == reg->xtoCell)
+    return XTO_CELL;
+
+  if (cell == reg->memoCell)
+    return MEMO_CELL;
+
+  if (cell == reg->creditCell)
+    return CRED_CELL;
+
+  if (cell == reg->debitCell)
+    return DEBT_CELL;
+
+  if (cell == reg->priceCell)
+    return PRIC_CELL;
+
+  if (cell == reg->valueCell)
+    return VALU_CELL;
+
+  if (cell == reg->ncreditCell)
+    return NCRED_CELL;
+
+  if (cell == reg->ndebitCell)
+    return NDEBT_CELL;
+
+  return NO_CELL;
+}
+
+/* ============================================== */
+
+CellType
+xaccSplitRegisterGetCellType (SplitRegister *reg)
+{
+  Table *table;
+
+  if (reg == NULL)
+    return NO_CELL;
+
+  table = reg->table;
+  if (table == NULL)
+    return NO_CELL;
+
+  return xaccSplitRegisterGetCellTypeRowCol(reg,
+                                            table->current_cursor_phys_row,
+                                            table->current_cursor_phys_col);
+}
+
+/* ============================================== */
+
+static BasicCell *
+sr_current_cell (SplitRegister *reg)
+{
+  Table *table;
+  Locator *locator;
+  CellBlock *cellblock;
+  int phys_row, phys_col;
+  int virt_row, virt_col;
+  int cell_row, cell_col;
+
+  if (reg == NULL)
+    return NULL;
+
+  table = reg->table;
+  if (table == NULL)
+    return NULL;
+
+  phys_row = table->current_cursor_phys_row;
+  phys_col = table->current_cursor_phys_col;
+
+  if ((phys_row < 0) || (phys_row >= table->num_phys_rows) ||
+      (phys_col < 0) || (phys_col >= table->num_phys_cols))
+    return NULL;
+
+  locator = table->locators[phys_row][phys_col];
+
+  virt_row = locator->virt_row;
+  virt_col = locator->virt_col;
+  cell_row = locator->phys_row_offset;
+  cell_col = locator->phys_col_offset;
+
+  cellblock = table->handlers[virt_row][virt_col];
+
+  return cellblock->cells[cell_row][cell_col];
+}
+
+/* ============================================== */
+
+CellType
+xaccSplitRegisterGetCellTypeRowCol (SplitRegister *reg,
+                                    int phys_row, int phys_col)
+{
+  BasicCell *cell;
+
+  cell = sr_current_cell (reg);
+  if (cell == NULL)
+    return NO_CELL;
+
+  return sr_cell_type (reg, cell);
+}
+
+/* ============================================== */
+
+gncBoolean
+xaccSplitRegisterGetCellRowCol (SplitRegister *reg, CellType cell_type,
+                                int *p_phys_row, int *p_phys_col)
+{
+  Table *table;
+  Locator *locator;
+  CellBlock *cellblock;
+  int phys_row, phys_col;
+  int virt_row, virt_col;
+  int cell_row, cell_col;
+
+  if (reg == NULL)
+    return GNC_F;
+
+  table = reg->table;
+  if (table == NULL)
+    return GNC_F;
+
+  phys_row = table->current_cursor_phys_row;
+  phys_col = table->current_cursor_phys_col;
+
+  if ((phys_row < 0) || (phys_row >= table->num_phys_rows) ||
+      (phys_col < 0) || (phys_col >= table->num_phys_cols))
+    return GNC_F;
+
+  locator = table->locators[phys_row][phys_col];
+
+  virt_row = locator->virt_row;
+  virt_col = locator->virt_col;
+
+  cellblock = table->handlers[virt_row][virt_col];
+
+  for (cell_row = 0; cell_row < cellblock->numRows; cell_row++)
+    for (cell_col = 0; cell_col < cellblock->numCols; cell_col++)
+    {
+      BasicCell *cell = cellblock->cells[cell_row][cell_col];
+
+      if (sr_cell_type (reg, cell) == cell_type)
+      {
+        RevLocator *rev_locator;
+
+        rev_locator = table->rev_locators[virt_row][virt_col];
+
+        phys_row = rev_locator->phys_row + cell_row;
+        phys_col = rev_locator->phys_col + cell_col;
+
+        if (p_phys_row != NULL)
+          *p_phys_row = phys_row;
+
+        if (p_phys_col != NULL)
+          *p_phys_col = phys_col;
+
+        return GNC_T;
+      }
+    }
+
+  return GNC_F;
+}
+
+/* ============================================== */
+
 SplitRegisterBuffer *
 xaccMallocSplitRegisterBuffer ()
 {
