@@ -10,8 +10,12 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
 /*
@@ -20,9 +24,9 @@
  *
  * FUNCTION:
  * Implements a basic display register/ledger.
- * This object makes specific cell have specific properties
- * (price, text, date, etc).  and specific names that correspond.
- * it also determines the actual physical layout, arrangement
+ * This object makes specific cells have specific properties
+ * (price, text, date, etc.) and specific names that correspond.
+ * It also determines the actual physical layout, arrangement
  * of columns, etc.
  *
  * Handles splits
@@ -57,29 +61,64 @@
 /* defined register types */
 /* "registers" are single-account display windows.
  * "ledgers" are multiple-account display windows */
-#define BANK_REGISTER       1
-#define CASH_REGISTER       2
-#define ASSET_REGISTER      3
-#define CREDIT_REGISTER     4
-#define LIABILITY_REGISTER  5
-#define INCOME_REGISTER     6
-#define EXPENSE_REGISTER    7
-#define EQUITY_REGISTER     8
-#define STOCK_REGISTER      9
-#define CURRENCY_REGISTER   10
+typedef enum
+{
+  BANK_REGISTER       =  1,
+  CASH_REGISTER       =  2,
+  ASSET_REGISTER      =  3,
+  CREDIT_REGISTER     =  4,
+  LIABILITY_REGISTER  =  5,
+  INCOME_REGISTER     =  6,
+  EXPENSE_REGISTER    =  7,
+  EQUITY_REGISTER     =  8,
+  STOCK_REGISTER      =  9,
+  CURRENCY_REGISTER   = 10,
+
+  GENERAL_LEDGER      = 11,
+  INCOME_LEDGER       = 12,
+  PORTFOLIO_LEDGER    = 13,
+  SEARCH_LEDGER       = 14
+} SplitRegisterType;
 
-#define GENERAL_LEDGER      11
-#define INCOME_LEDGER       12
-#define PORTFOLIO           13
 #define REG_TYPE_MASK       0xff
 
+/* These values are used to identify the cells in the register. */
+typedef enum
+{
+  NO_CELL    = -1,
+  DATE_CELL  =  0,
+  NUM_CELL   =  1,
+  DESC_CELL  =  2,
+  RECN_CELL  =  3,
+  SHRS_CELL  =  4,
+  BALN_CELL  =  5,
+  ACTN_CELL  =  6,
+  XFRM_CELL  =  7,
+  XTO_CELL   =  8,
+  MEMO_CELL  =  9,
+  CRED_CELL  = 10,
+  DEBT_CELL  = 11,
+  PRIC_CELL  = 12,
+  VALU_CELL  = 13,
+
+  /* NCRED & NDEBT handle minus the usual quantities */
+  NCRED_CELL = 14,
+  NDEBT_CELL = 15,
+
+  /* MXFRM is the "mirrored" transfer-from account */
+  MXFRM_CELL = 16
+} CellType;
+
 /*
  * enumerated display styles 
- * REG_DOUBLE_LINE  -- show two lines per transaction
- * REG_MULTI_LINE   -- show multiple lines per transaction
- * REG_DYNAMIC      -- dynamically expand edited transaction
+ * REG_SINGLE_LINE    -- show one line per transaction
+ * REG_DOUBLE_LINE    -- show two lines per transaction
+ * REG_MULTI_LINE     -- show multiple lines per transaction
+ * REG_SINGLE_DYNAMIC -- dynamically expand edited transaction,
+ *                       all other transactions on one line
+ * REG_DOUBLE_DYNAMIC -- dynamically expand edited transaction,
+ *                       all other transactions on two lines
  */
-
 #define REG_SINGLE_LINE        (1 << 8)
 #define REG_DOUBLE_LINE        (2 << 8)
 #define REG_MULTI_LINE         (3 << 8)
@@ -165,10 +204,8 @@ struct _SplitRegister {
    int num_phys_rows;
    int num_virt_rows;
 
-   int cursor_virt_row;
-
    int cursor_phys_row;
-   int cursor_phys_col;
+   int cursor_virt_row;
 
    BasicCell *header_label_cells[NUM_CELLS];
 
@@ -186,25 +223,29 @@ typedef struct _SplitRegisterColors SplitRegisterColors;
 
 struct _SplitRegisterColors
 {
-  uint32 single_cursor_active_bg_color;
-  uint32 single_cursor_passive_bg_color;
-  uint32 single_cursor_passive_bg_color2;
+  guint32 single_cursor_active_bg_color;
+  guint32 single_cursor_passive_bg_color;
+  guint32 single_cursor_passive_bg_color2;
 
-  uint32 double_cursor_active_bg_color;
-  uint32 double_cursor_passive_bg_color;
-  uint32 double_cursor_passive_bg_color2;
+  guint32 double_cursor_active_bg_color;
+  guint32 double_cursor_passive_bg_color;
+  guint32 double_cursor_passive_bg_color2;
 
   gncBoolean double_alternate_virt;
 
-  uint32 trans_cursor_active_bg_color;
-  uint32 trans_cursor_passive_bg_color;
+  guint32 trans_cursor_active_bg_color;
+  guint32 trans_cursor_passive_bg_color;
 
-  uint32 split_cursor_active_bg_color;
-  uint32 split_cursor_passive_bg_color;
+  guint32 split_cursor_active_bg_color;
+  guint32 split_cursor_passive_bg_color;
 
-  uint32 header_bg_color;
+  guint32 header_bg_color;
 };
 
+typedef char* (*SRStringGetter) (SplitRegisterType);
+
+void            xaccSplitRegisterSetDebitStringGetter(SRStringGetter getter);
+void            xaccSplitRegisterSetCreditStringGetter(SRStringGetter getter);
 
 SplitRegister * xaccMallocSplitRegister (int type);
 void            xaccInitSplitRegister (SplitRegister *, int type);
@@ -223,6 +264,26 @@ void            xaccSplitRegisterClearChangeFlag (SplitRegister *reg);
 /* Returns the type of the current cursor */
 CursorType      xaccSplitRegisterGetCursorType (SplitRegister *reg);
 
+/* Returns the type of the cursor at the given virtual row and column. */
+CursorType      xaccSplitRegisterGetCursorTypeRowCol (SplitRegister *reg,
+                                                      int virt_row,
+                                                      int virt_col);
+/* Returns the type of the current cell */
+CellType        xaccSplitRegisterGetCellType (SplitRegister *reg);
+
+/* Returns the type of the cell at the given physical row and column. */
+CellType        xaccSplitRegisterGetCellTypeRowCol (SplitRegister *reg,
+                                                    int phys_row,
+                                                    int phys_col);
+
+/* Returns the physical row and column in the current cursor of the
+ * given cell using the pointer values. The function returns true if
+ * the given cell type is in the current cursor, false otherwise. */
+gncBoolean      xaccSplitRegisterGetCellRowCol (SplitRegister *reg,
+                                                CellType cell_type,
+                                                int *p_phys_row,
+                                                int *p_phys_col);
+
 /* Functions for working with split register buffers */
 SplitRegisterBuffer * xaccMallocSplitRegisterBuffer ();
 void xaccDestroySplitRegisterBuffer (SplitRegisterBuffer *srb);
