@@ -1,14 +1,3 @@
-/*
- * FILE:
- * table-allgui.c
- *
- * FUNCTION:
- * Implements the gui-independent parts of the table infrastructure.
- *
- * HISTORY:
- * Copyright (c) 1998,1999,2000 Linas Vepstas
- */
-
 /********************************************************************\
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -21,10 +10,25 @@
  * GNU General Public License for more details.                     *
  *                                                                  *
  * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
 \********************************************************************/
 
+/*
+ * FILE:
+ * table-allgui.c
+ *
+ * FUNCTION:
+ * Implements the gui-independent parts of the table infrastructure.
+ *
+ * HISTORY:
+ * Copyright (c) 1998,1999,2000 Linas Vepstas
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -277,8 +281,8 @@ xaccTableResize (Table * table,
                new_phys_rows,
                new_phys_cols,
                (table->bg_colors),
-               uint32,
-               ((uint32) 0xffffff),  /* white */
+               guint32,
+               ((guint32) 0xffffff), /* white */
                NOOP);                /* no-op */
 
    /* resize the foreground color array (black text) */
@@ -287,9 +291,9 @@ xaccTableResize (Table * table,
                new_phys_rows,
                new_phys_cols,
                (table->fg_colors),
-               uint32,
-               ((uint32) 0x0),      /* black */
-               NOOP);               /* no-op */
+               guint32,
+               ((guint32) 0x0),      /* black */
+               NOOP);                /* no-op */
 
 
    /* resize the user-data hooks */
@@ -399,7 +403,7 @@ makePassive (Table *table)
    for (i=0; i<curs->numRows; i++) {
       for (j=0; j<curs->numCols; j++) {
          BasicCell *cell;
-         uint32 color;
+         guint32 color;
 
          if (table->alternate_bg_colors) {
            if ((virt_row % 2) == 1)
@@ -431,14 +435,15 @@ makePassive (Table *table)
 /* ==================================================== */
 
 static void 
-doMoveCursor (Table *table, int new_phys_row, int new_phys_col, int do_move_gui)
+doMoveCursor (Table *table, int new_phys_row, int new_phys_col,
+              int do_move_gui)
 {
    int i,j;
    int phys_row_origin, phys_col_origin;
    int new_virt_row, new_virt_col;
    CellBlock *curs;
 
-   ENTER("doMoveCursor(): new_phys=(%d %d) do_move_gui=%d\n", 
+   ENTER("new_phys=(%d %d) do_move_gui=%d\n", 
          new_phys_row, new_phys_col, do_move_gui);
 
    /* Change the cell background colors to their "passive" values.
@@ -460,7 +465,7 @@ doMoveCursor (Table *table, int new_phys_row, int new_phys_col, int do_move_gui)
        * passive again. */
       makePassive (table);
       if (do_move_gui)
-        xaccRefreshCursorGUI(table, FALSE);
+        xaccRefreshCursorGUI(table, GNC_F);
    }
 
    /* check for out-of-bounds conditions (which may be deliberate) */
@@ -503,7 +508,7 @@ doMoveCursor (Table *table, int new_phys_row, int new_phys_col, int do_move_gui)
             }
          }
       }
-      LEAVE("doMoveCursor(): out of bounds\n");
+      LEAVE("out of bounds\n");
       return;
    }
 
@@ -530,10 +535,13 @@ doMoveCursor (Table *table, int new_phys_row, int new_phys_col, int do_move_gui)
    phys_col_origin = new_phys_col;
    phys_col_origin -= table->locators[new_phys_row][new_phys_col]->phys_col_offset;
 
-   /* setting the previous traversal value to the last of a traversal chain will
-    * guarantee that first entry into a register will occur at the first cell */
-   table->prev_phys_traverse_row  = phys_row_origin + curs->last_reenter_traverse_row;
-   table->prev_phys_traverse_col  = phys_col_origin + curs->last_reenter_traverse_col; 
+   /* setting the previous traversal value to the last of a traversal chain
+    * will guarantee that the first entry into a register will occur at the
+    * first cell */
+   table->prev_phys_traverse_row = (phys_row_origin +
+                                    curs->last_reenter_traverse_row);
+   table->prev_phys_traverse_col = (phys_col_origin +
+                                    curs->last_reenter_traverse_col);
 
    /* update the cell values to reflect the new position */
    for (i=0; i<curs->numRows; i++) {
@@ -575,20 +583,24 @@ doMoveCursor (Table *table, int new_phys_row, int new_phys_col, int do_move_gui)
    }
 
    curs->user_data = table->user_data[new_virt_row][new_virt_col];
-   LEAVE("doMoveCursor(): did move\n");
+   LEAVE("did move\n");
 }
 
 /* ==================================================== */
 
 void xaccMoveCursor (Table *table, int new_phys_row, int new_phys_col)
 {
-   doMoveCursor (table, new_phys_row, new_phys_col, 0);
+  if (!table) return;
+
+  doMoveCursor (table, new_phys_row, new_phys_col, 0);
 }
 
 /* same as above, but be sure to deal with GUI elements as well */
 void xaccMoveCursorGUI (Table *table, int new_phys_row, int new_phys_col)
 {
-   doMoveCursor (table, new_phys_row, new_phys_col, 1);
+  if (!table) return;
+
+  doMoveCursor (table, new_phys_row, new_phys_col, 1);
 }
 
 /* ==================================================== */
@@ -601,6 +613,8 @@ void xaccCommitCursor (Table *table)
    int phys_row, phys_col;
    int phys_row_origin, phys_col_origin;
 
+   if (!table) return;
+
    curs = table->current_cursor;
    if (!curs) return;
 
@@ -628,7 +642,7 @@ void xaccCommitCursor (Table *table)
          if (cell) {
             int iphys = i + phys_row_origin;
             int jphys = j + phys_col_origin;
-            /*PINFO ("xaccCommitCursor(): rowcol (%d,%d) oldval=%s newval=%s\n",
+            /*PINFO ("rowcol (%d,%d) oldval=%s newval=%s\n",
               iphys, jphys, table->entries[iphys][jphys], cell->value);*/
             if (table->entries[iphys][jphys]) {
                free (table->entries[iphys][jphys]);
@@ -660,6 +674,7 @@ xaccRefreshHeader (Table *table)
    int i,j;
    CellBlock *arr;
 
+   if (!table) return;
    if (!(table->entries)) return;
 
    /* copy header data into entries cache */
@@ -698,7 +713,7 @@ xaccVerifyCursorPosition (Table *table, int phys_row, int phys_col)
    gncBoolean do_commit = GNC_F;
    gncBoolean moved_cursor = GNC_F;
 
-   if (!table) return FALSE;
+   if (!table) return GNC_F;
 
    /* Someone may be trying to intentionally invalidate the cursor, 
     * in which case the physical addresses could be out of bounds.
@@ -758,6 +773,8 @@ xaccGetUserData (Table *table, int phys_row, int phys_col)
 {
    int virt_row, virt_col;
 
+   if (!table) return NULL;
+
    /* check for out-of-bounds conditions */
    if ((0 > phys_row) || (0 > phys_col) ||
       (phys_row >= table->num_phys_rows) ||
@@ -812,7 +829,9 @@ wrapVerifyCursorPosition (Table *table, int row, int col)
    const int save_phys_col = table->current_cursor_phys_col;
    gncBoolean moved_cursor;
 
-   ENTER("wrapVerifyCursorPosition(): (%d %d) val=%s\n", 
+   if (!table) return;
+
+   ENTER("(%d %d) val=%s\n", 
          row,col, table->entries[row][col]);
 
    /* VerifyCursor will do all sorts of gui-independent machinations */
@@ -826,7 +845,7 @@ wrapVerifyCursorPosition (Table *table, int row, int col)
                           save_phys_row, save_phys_col, GNC_F);
    }
 
-   LEAVE ("wrapVerifyCursorPosition()\n");
+   LEAVE ("\n");
 }
 
 /* ==================================================== */
@@ -834,10 +853,12 @@ wrapVerifyCursorPosition (Table *table, int row, int col)
 void        
 xaccRefreshCursorGUI (Table * table, gncBoolean do_scroll)
 {
-   doRefreshCursorGUI (table, table->current_cursor,
-                       table->current_cursor_phys_row,
-                       table->current_cursor_phys_col,
-                       do_scroll);
+  if (!table) return;
+
+  doRefreshCursorGUI (table, table->current_cursor,
+                      table->current_cursor_phys_row,
+                      table->current_cursor_phys_col,
+                      do_scroll);
 }
 
 /* ==================================================== */
@@ -851,6 +872,8 @@ gnc_register_cell_valid(Table *table, int row, int col, gncBoolean exact_cell)
   int virt_row, virt_col;
   CellBlock *arr, *header;
 
+  if (!table) return GNC_F;
+
   /* can't edit outside of the physical space */
   invalid = (0 > row) || (0 > col) ;
   invalid = invalid || (row >= table->num_phys_rows);
@@ -859,7 +882,7 @@ gnc_register_cell_valid(Table *table, int row, int col, gncBoolean exact_cell)
   /* In case we're called after table has been destroyed. */
   invalid = invalid || (table->handlers == NULL);
 
-  if(invalid) return GNC_F;
+  if (invalid) return GNC_F;
 
   /* header rows cannot be modified */
   /* hack alert -- assumes that header is first cell */
@@ -921,8 +944,7 @@ gnc_table_enter_update(Table *table, int row, int col, int *cursor_position,
 
   const char * (*enter) (BasicCell *, const char *, int *, int *, int *);
 
-  ENTER("gnc_table_enter_update(): "
-        "enter %d %d (relrow=%d relcol=%d) cell=%p val=%s\n", 
+  ENTER("enter %d %d (relrow=%d relcol=%d) cell=%p val=%s\n", 
          row, col, rel_row, rel_col, 
          arr->cells[rel_row][rel_col], table->entries[row][col]);
 
@@ -973,7 +995,7 @@ gnc_table_enter_update(Table *table, int row, int col, int *cursor_position,
   table->prev_phys_traverse_row = row;
   table->prev_phys_traverse_col = col;
 
-  LEAVE("gnc_table_enter_update(): return %s\n", retval);
+  LEAVE("return %s\n", retval);
   return retval;
 }
 
@@ -989,7 +1011,7 @@ gnc_table_leave_update(Table *table, int row, int col,
   const char * (*leave) (BasicCell *, const char *);
   const char *retval = NULL;
   
-  ENTER("gnc_table_leave_update(): proposed (%d %d) rel(%d %d) \"%s\"\n",
+  ENTER("proposed (%d %d) rel(%d %d) \"%s\"\n",
         row, col, rel_row, rel_col, callback_text);
 
   if (!callback_text) callback_text = "";
@@ -1034,7 +1056,7 @@ gnc_table_leave_update(Table *table, int row, int col,
      retval = NULL;
   }
 
-  LEAVE("gnc_table_leave_update(): return %s\n", retval);
+  LEAVE("return %s\n", retval);
 
   return retval;
 }
@@ -1066,7 +1088,7 @@ gnc_table_modify_update(Table *table,
 
   const char *retval = NULL;
 
-  ENTER ("gnc_table_modify_update()\n");
+  ENTER ("\n");
 
   /* OK, if there is a callback for this cell, call it */
   mv = arr->cells[rel_row][rel_col]->modify_verify;
@@ -1104,8 +1126,7 @@ gnc_table_modify_update(Table *table,
       free(help_str);
   }
 
-  LEAVE ("gnc_table_modify_update(): "
-         "change %d %d (relrow=%d relcol=%d) cell=%p val=%s\n", 
+  LEAVE ("change %d %d (relrow=%d relcol=%d) cell=%p val=%s\n", 
          row, col, rel_row, rel_col, 
          arr->cells[rel_row][rel_col], table->entries[row][col]);
   
@@ -1208,7 +1229,7 @@ gnc_table_traverse_update(Table *table, int row, int col,
 {
   CellBlock *arr = table->current_cursor;
 
-  ENTER("gnc_table_traverse_update(): proposed (%d %d) -> (%d %d)\n",
+  ENTER("proposed (%d %d) -> (%d %d)\n",
         row, col, *dest_row, *dest_col);
 
   /* first, make sure our destination cell is valid.  If it is out of
@@ -1216,18 +1237,18 @@ gnc_table_traverse_update(Table *table, int row, int col,
   if ((*dest_row >= table->num_phys_rows) || (*dest_row < 0) ||
       (*dest_col >= table->num_phys_cols) || (*dest_col < 0)) 
   {
-    PERR("gnc_table_traverse_update: destination (%d, %d) out of bounds (%d, %d)\n",
-      *dest_row, *dest_col, table->num_phys_rows, table->num_phys_cols);
+    PERR("destination (%d, %d) out of bounds (%d, %d)\n",
+         *dest_row, *dest_col, table->num_phys_rows, table->num_phys_cols);
     return GNC_T;
   }
 
   /* next, check the current row and column.  If they are out of bounds
    * we can recover by treating the traversal as a mouse point. This can
-   * occur whenever the Xbae widget is resized smaller. */
+   * occur whenever the register widget is resized smaller, maybe?. */
   if ((row >= table->num_phys_rows) || (row < 0) ||
       (col >= table->num_phys_cols) || (col < 0)) {
 
-    PINFO("gnc_table_traverse_update: source (%d, %d) out of bounds (%d, %d)\n",
+    PINFO("source (%d, %d) out of bounds (%d, %d)\n",
 	  row, col, table->num_phys_rows, table->num_phys_cols);
     table->prev_phys_traverse_row = *dest_row;
     table->prev_phys_traverse_col = *dest_col;
@@ -1254,12 +1275,6 @@ gnc_table_traverse_update(Table *table, int row, int col,
         }
       }
 
-      if (gnc_register_cell_valid(table, *dest_row, *dest_col, GNC_F))
-	break;
-
-      if (!gnc_table_find_valid_cell_horiz(table, dest_row, dest_col, GNC_F))
-        return GNC_T;
-
       break;
 
     case GNC_TABLE_TRAVERSE_UP:
@@ -1317,12 +1332,12 @@ gnc_table_traverse_update(Table *table, int row, int col,
 
   /* Call the table traverse callback for any modifications. */
   if (table->traverse)
-    (table->traverse) (table, dest_row, dest_col, table->client_data);
+    (table->traverse) (table, dest_row, dest_col, dir, table->client_data);
 
   table->prev_phys_traverse_row = *dest_row;
   table->prev_phys_traverse_col = *dest_col;
 
-  LEAVE("gnc_table_traverse_update(): dest_row = %d, dest_col = %d\n",
+  LEAVE("dest_row = %d, dest_col = %d\n",
         *dest_row, *dest_col);
 
   return GNC_F;
