@@ -1,3 +1,23 @@
+/********************************************************************\
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
 /*
  * FILE:
  * table-allgui.h
@@ -76,25 +96,11 @@
  * Copyright (c) 1998,1999,2000 Linas Vepstas
  */
 
-/********************************************************************\
- * This program is free software; you can redistribute it and/or    *
- * modify it under the terms of the GNU General Public License as   *
- * published by the Free Software Foundation; either version 2 of   *
- * the License, or (at your option) any later version.              *
- *                                                                  *
- * This program is distributed in the hope that it will be useful,  *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
- * GNU General Public License for more details.                     *
- *                                                                  *
- * You should have received a copy of the GNU General Public License*
- * along with this program; if not, write to the Free Software      *
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
-\********************************************************************/
-
 #ifndef __XACC_TABLE_ALLGUI_H__
 #define __XACC_TABLE_ALLGUI_H__
 
+#include <glib.h>
+
 #ifdef MOTIF
 #include "table-motif.h"
 #endif 
@@ -120,7 +126,7 @@ typedef enum {
   GNC_TABLE_TRAVERSE_DOWN
 } gncTableTraversalDir;
 
-/* the Locator structure is used provide a mapping from
+/* The Locator structure is used provide a mapping from
  * the physical array of cells to the logical array of 
  * virtual cell blocks.
  *
@@ -197,13 +203,14 @@ struct _Table {
 
   /* callback that is called when the cursor is moved */
   /* hack alert -- this should be a callback list, actually */
-  void (*move_cursor) (Table *, int *p_new_phys_row, 
-                                int *p_new_phys_col, 
+  void (*move_cursor) (Table *, int *p_new_phys_row,
+                                int *p_new_phys_col,
                                 void *client_data);
 
   /* callback that is called to determine traversal */
-  void  (*traverse)  (Table *,  int *p_new_phys_row, 
-                                int *p_new_phys_col, 
+  void  (*traverse)  (Table *,  int *p_new_phys_row,
+                                int *p_new_phys_col,
+                                gncTableTraversalDir dir,
                                 void *client_data);
 
   TableSetHelpFunc set_help;
@@ -217,8 +224,8 @@ struct _Table {
   /* background colors for each cell, format ARGB, 
    * and foreground (text) colors, format ARGB,
    * of dimension num_phys_rows * num_phys_cols */
-  uint32 **bg_colors;
-  uint32 **fg_colors;
+  guint32 **bg_colors;
+  guint32 **fg_colors;
 
   /* Determines whether the passive background
    * colors alternate between odd and even virt
@@ -413,7 +420,9 @@ gnc_table_traverse_update(Table *table,
                           int *dest_row,
                           int *dest_col);
 
-/* Find the closest valid horizontal cell */
+/* Find the closest valid horizontal cell. If exact_cell is true,
+ * cells that must be explicitly selected by the user (as opposed
+ * to just tabbing into), are considered valid cells. */
 gncBoolean
 gnc_table_find_valid_cell_horiz(Table *table, int *row, int *col,
                                 gncBoolean exact_cell);
