@@ -1,3 +1,20 @@
+;; This program is free software; you can redistribute it and/or    
+;; modify it under the terms of the GNU General Public License as   
+;; published by the Free Software Foundation; either version 2 of   
+;; the License, or (at your option) any later version.              
+;;                                                                  
+;; This program is distributed in the hope that it will be useful,  
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
+;; GNU General Public License for more details.                     
+;;                                                                  
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
+;; Boston, MA  02111-1307,  USA       gnu@gnu.org
+
 (define (gnc:startup)
   (gnc:debug "starting up.")
   (if (not (gnc:handle-command-line-args))
@@ -10,8 +27,6 @@
   (gnc:load "srfi/srfi-1.unclear.scm")
   (gnc:load "srfi/srfi-1.r5rs.scm")
 
-  (gnc:load "depend.scm")
-
   ;; Now we can load a bunch of files.
 
   (gnc:depend "doc.scm")
@@ -19,19 +34,8 @@
   (gnc:depend "text-export.scm")
   (gnc:depend "report.scm")
   (gnc:depend "report/report-list.scm")
-
-  (gnc:config-var-value-set! gnc:*load-path* #f
-                             (cons (string-append gnc:_share-dir-default_ 
-                                                  "/scm/qif-import")
-                                   (gnc:config-var-value-get gnc:*load-path*)))
-
-  (gnc:config-var-value-set! gnc:*load-path* #f
-                             (cons (string-append gnc:_share-dir-default_ 
-                                                  "/scm/printing")
-                                   (gnc:config-var-value-get gnc:*load-path*)))
-
-  (gnc:depend "qif-import.scm")
-  (gnc:depend "print-check.scm")
+  (gnc:depend "qif-import/qif-import.scm")
+  (gnc:depend "printing/print-check.scm")
 
   ;; Load the system configs
   (if (not (gnc:load-system-config-if-needed))
@@ -54,7 +58,10 @@
           (gnc:config-var-value-get gnc:*arg-show-help*))
       (begin
         (gnc:prefs-show-usage)
-        (gnc:shutdown 0))))
+        (gnc:shutdown 0)))
+
+  (if (gnc:config-var-value-get gnc:*loglevel*)
+      (gnc:set-log-level-global (gnc:config-var-value-get gnc:*loglevel*))))
 
 
 (define (gnc:shutdown exit-status)
@@ -80,15 +87,17 @@
 (define (gnc:main)
 
   ;; Now the fun begins.
-
   (gnc:startup)
 
   (if (not (= (gnc:lowlev-app-init) 0))
       (gnc:shutdown 0))
 
-  (if (pair? gnc:*command-line-files*)
-      ;; You can only open single files right now...
-      (gnc:ui-open-file (car gnc:*command-line-files*)))
+  (let ((ok (not (gnc:config-var-value-get gnc:*arg-no-file*)))
+        (file (if (pair? gnc:*command-line-files*)
+                  (car gnc:*command-line-files*)
+                  (gnc:history-get-last))))
+    (if (and ok (string? file))
+      (gnc:ui-open-file file)))
 
   ;; add a hook to save the user configs on shutdown
   (gnc:hook-add-dangler gnc:*shutdown-hook* gnc:save-global-options)
