@@ -1,4 +1,21 @@
 ;; Scheme code for supporting options
+;;
+;; This program is free software; you can redistribute it and/or    
+;; modify it under the terms of the GNU General Public License as   
+;; published by the Free Software Foundation; either version 2 of   
+;; the License, or (at your option) any later version.              
+;;                                                                  
+;; This program is distributed in the hope that it will be useful,  
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
+;; GNU General Public License for more details.                     
+;;                                                                  
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
+;; Boston, MA  02111-1307,  USA       gnu@gnu.org
 
 (define (gnc:make-option
 	 ;; The category of this option
@@ -112,6 +129,28 @@
              (else (list #f "string-option: not a string"))))
      #f #f )))
 
+;; currency options use a specialized widget for entering currencies
+;; in the GUI implementation. Currencies must be 3-letter ISO codes
+;; represented as strings.
+(define (gnc:make-currency-option
+	 section
+	 name
+	 sort-tag
+         documentation-string
+	 default-value)
+  (let* ((value default-value)
+         (value->string (lambda () (gnc:value->string value))))
+    (gnc:make-option
+     section name sort-tag 'currency documentation-string
+     (lambda () value)
+     (lambda (x) (set! value x))
+     (lambda () default-value)
+     (gnc:restore-form-generator value->string)
+     (lambda (x)
+       (cond ((string? x)(list #t x))
+             (else (list #f "currency-option: not a currency code"))))
+     #f #f )))
+
 (define (gnc:make-simple-boolean-option
 	 section
 	 name
@@ -165,7 +204,9 @@
 
 ;; account-list options use the option-data as a boolean value.  If
 ;; true, the gui should allow the user to select multiple accounts.
-;; values are always a list of accounts.
+;; Internally, values are always a list of guids. Externally, both
+;; guids and account pointers may be used to set the value of the
+;; option. The option always returns a list of account pointers.
 (define (gnc:make-account-list-option
          section
          name
@@ -174,7 +215,17 @@
          default-getter
          value-validator
          multiple-selection)
-  (let ((option (default-getter))
+
+  (define (convert-to-guid item)
+    (if (string? item)
+        item
+        (gnc:account-get-guid item)))
+  (define (convert-to-account item)
+    (if (string? item)
+        (gnc:account-lookup item)
+        item))
+
+  (let ((option (map convert-to-guid (default-getter)))
         (option-set #f)
         (validator
          (if (not value-validator)
@@ -182,17 +233,19 @@
              value-validator)))
     (gnc:make-option
      section name sort-tag 'account-list documentation-string
-     (lambda () (if option-set option (default-getter)))
+     (lambda () (map convert-to-account (if option-set
+                                            option
+                                            (default-getter))))
      (lambda (account-list)
        (let* ((result (validator account-list))
               (valid (car result))
               (value (cadr result)))
          (if valid
              (begin
-               (set! option value)
+               (set! option (map convert-to-guid value))
                (set! option-set #t))
              (gnc:error "Illegal account list value set"))))
-     default-getter
+     (lambda () (map convert-to-account (default-getter)))
      #f
      validator
      multiple-selection #f)))
@@ -316,6 +369,21 @@
      (list lower-bound upper-bound num-decimals step-size)
      #f)))
 
+(define (gnc:make-internal-option
+         section
+         name
+         default-value)
+  (let* ((value default-value)
+         (value->string (lambda () (gnc:value->string value))))
+    (gnc:make-option
+     section name "" 'internal #f
+     (lambda () value)
+     (lambda (x) (set! value x))
+     (lambda () default-value)
+     (gnc:restore-form-generator value->string)
+     (lambda (x) (list #t x))
+     #f #f)))
+
 ;; Color options store rgba values in a list.
 ;; The option-data is a list, whose first element
 ;; is the range of possible rgba values and whose
@@ -513,6 +581,14 @@
         (hash-for-each run-callback callback-hash))
     (clear-changes))
 
+  (define default-section #f)
+
+  (define (set-default-section section-name)
+    (set! default-section section-name))
+
+  (define (get-default-section)
+    default-section)
+
   (define (dispatch key)
     (case key
       ((lookup) lookup-option)
@@ -524,6 +600,8 @@
       ((generate-restore-forms) generate-restore-forms)
       ((clear-changes) clear-changes)
       ((run-callbacks) run-callbacks)
+      ((set-default-section) set-default-section)
+      ((get-default-section) get-default-section)
       (else (gnc:warn "options: bad key: " key "\n"))))
 
   dispatch)
@@ -559,6 +637,12 @@
 (define (gnc:options-run-callbacks options)
   ((options 'run-callbacks)))
 
+(define (gnc:options-set-default-section options section-name)
+  ((options 'set-default-section) section-name))
+
+(define (gnc:options-get-default-section options)
+  ((options 'get-default-section)))
+
 (define (gnc:send-options db_handle options)
   (gnc:options-for-each
    (lambda (option)
@@ -574,11 +658,20 @@
                (close port)))))
 
 (define (gnc:options-register-translatable-strings options)
+  (define (external-name? name)
+    (cond ((not (string? name)) #f)
+          ((< (string-length name) 2) #t)
+          ((not (eq? (string-ref name 0) #\_)) #t)
+          ((not (eq? (string-ref name 1) #\_)) #t)
+          (else #f)))
+
   (gnc:options-for-each-general
    (lambda (section hash)
-     (gnc:register-translatable-strings section))
+     (if (external-name? section)
+         (gnc:register-translatable-strings section)))
    (lambda (option)
-     (gnc:register-translatable-strings (gnc:option-name option))
+     (if (not (eq? (gnc:option-type option) 'internal))
+         (gnc:register-translatable-strings (gnc:option-name option)))
      (gnc:register-translatable-strings (gnc:option-documentation option))
      (let ((getter (gnc:option-strings-getter option)))
        (if getter
